@@ -1,33 +1,33 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "elementeditor.h"
 
 #include <typeinfo>
 
 #include <QCheckBox>
-#include <QTabWidget>
 #include <QLabel>
 #include <QLayout>
 #include <QBuffer>
 #include <QTextStream>
 #include <QApplication>
 #include <QFileInfo>
+#include <QMenu>
+#include <QScrollArea>
 
 #include <KPushButton>
 #include <KMessageBox>
@@ -36,16 +36,16 @@
 #include <KConfigGroup>
 #include <kio/netaccess.h>
 
-#include <entry.h>
-#include <comment.h>
-#include <macro.h>
-#include <preamble.h>
-#include <element.h>
-#include <file.h>
-#include <fileexporterblg.h>
+#include "menulineedit.h"
+#include "entry.h"
+#include "comment.h"
+#include "macro.h"
+#include "preamble.h"
+#include "element.h"
+#include "file.h"
 #include "elementwidgets.h"
-#include "elementeditor.h"
 #include "checkbibtex.h"
+#include "hidingtabwidget.h"
 
 class ElementEditor::ElementEditorPrivate : public ElementEditor::ApplyElementInterface
 {
@@ -62,29 +62,35 @@ private:
     ReferenceWidget *referenceWidget;
     ElementWidget *sourceWidget;
     KPushButton *buttonCheckWithBibTeX;
-    QCheckBox *checkBoxForceShowAllWidgets;
+
+    /// Settings management through a push button with menu
     KSharedConfigPtr config;
+    KPushButton *buttonOptions;
+    QAction *actionForceShowAllWidgets, *actionLimitKeyboardTabStops;
 
 public:
     QSharedPointer<Element> element;
-    QTabWidget *tab;
+    HidingTabWidget *tab;
     bool elementChanged, elementUnapplied;
 
-    ElementEditorPrivate(ElementEditor *parent)
+    ElementEditorPrivate(bool scrollable, ElementEditor *parent)
             : file(NULL), p(parent), previousWidget(NULL), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), elementChanged(false), elementUnapplied(false) {
-        createGUI();
+        createGUI(scrollable);
     }
 
     ~ElementEditorPrivate() {
-        // FIXME delete tab;
-        while (!widgets.isEmpty()) {
-            QWidget *w = widgets.last();
-            widgets.pop_back();
-            delete w;
-        }
+        clearWidgets();
     }
 
-    void setElement(QSharedPointer<Element> element, const File *file) {
+    void clearWidgets() {
+        for (int i = widgets.count() - 1; i >= 0; --i) {
+            QWidget *w = widgets[i];
+            w->deleteLater();
+        }
+        widgets.clear();
+    }
+
+    void setElement(QSharedPointer<Element> element, File *file) {
         this->element = element;
         this->file = file;
         referenceWidget->setOriginalElement(element);
@@ -101,26 +107,27 @@ public:
             if (previousWidget == NULL)
                 previousWidget = widget; ///< memorize the first tab
             int index = tab->addTab(widget, widget->icon(), widget->label());
-            tab->setTabEnabled(index, false);
+            tab->hideTab(index);
         }
 
         ElementWidget *widget = new PreambleWidget(tab);
         connect(widget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
         widgets << widget;
         int index = tab->addTab(widget, widget->icon(), widget->label());
-        tab->setTabEnabled(index, false);
+        tab->hideTab(index);
 
         widget = new MacroWidget(tab);
         connect(widget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
         widgets << widget;
         index = tab->addTab(widget, widget->icon(), widget->label());
-        tab->setTabEnabled(index, false);
+        tab->hideTab(index);
 
-        widget = new FilesWidget(tab);
-        connect(widget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
-        widgets << widget;
-        index = tab->addTab(widget, widget->icon(), widget->label());
-        tab->setTabEnabled(index, false);
+        FilesWidget *filesWidget = new FilesWidget(tab);
+        filesWidget->setApplyElementInterface(this);
+        connect(filesWidget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
+        widgets << filesWidget;
+        index = tab->addTab(filesWidget, filesWidget->icon(), filesWidget->label());
+        tab->hideTab(index);
 
         QStringList blacklistedFields;
 
@@ -138,23 +145,22 @@ public:
         connect(widget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
         widgets << widget;
         index = tab->addTab(widget, widget->icon(), widget->label());
-        tab->setTabEnabled(index, false);
+        tab->hideTab(index);
 
         sourceWidget = new SourceWidget(tab);
         connect(sourceWidget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
         widgets << sourceWidget;
         index = tab->addTab(sourceWidget, sourceWidget->icon(), sourceWidget->label());
-        tab->setTabEnabled(index, false);
+        tab->hideTab(index);
     }
 
-    void createGUI() {
-        /// load configuration
-        const QString configGroupName = QLatin1String("User Interface");
-        const QString keyEnableAllWidgets = QLatin1String("EnableAllWidgets");
+    void createGUI(bool scrollable) {
+        /// load configuration for options push button
+        static const QString configGroupName = QLatin1String("User Interface");
+        static const QString keyEnableAllWidgets = QLatin1String("EnableAllWidgets");
         KConfigGroup configGroup(config, configGroupName);
         const bool showAll = configGroup.readEntry(keyEnableAllWidgets, true);
-
-        widgets.clear();
+        const bool limitKeyboardTabStops = configGroup.readEntry(MenuLineEdit::keyLimitKeyboardTabStops, false);
 
         QBoxLayout *vLayout = new QVBoxLayout(p);
 
@@ -165,16 +171,38 @@ public:
         vLayout->addWidget(referenceWidget, 0);
         widgets << referenceWidget;
 
-        tab = new QTabWidget(p);
-        vLayout->addWidget(tab, 10);
+        if (scrollable) {
+            QScrollArea *sa = new QScrollArea(p);
+            tab = new HidingTabWidget(sa);
+            sa->setFrameStyle(0);
+            sa->setWidget(tab);
+            sa->setWidgetResizable(true);
+            sa->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+            sa->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+            vLayout->addWidget(sa, 10);
+        } else {
+            tab = new HidingTabWidget(p);
+            vLayout->addWidget(tab, 10);
+        }
 
         QBoxLayout *hLayout = new QHBoxLayout();
         vLayout->addLayout(hLayout, 0);
 
-        checkBoxForceShowAllWidgets = new QCheckBox(i18n("Show all fields"), p);
-        checkBoxForceShowAllWidgets->setChecked(showAll);
-        hLayout->addWidget(checkBoxForceShowAllWidgets, 0);
-        connect(checkBoxForceShowAllWidgets, SIGNAL(toggled(bool)), p, SLOT(updateReqOptWidgets()));
+        /// Push button with menu to toggle various options
+        buttonOptions = new KPushButton(KIcon("configure"), i18n("Options"), p);
+        hLayout->addWidget(buttonOptions, 0);
+        QMenu *menuOptions = new QMenu(buttonOptions);
+        buttonOptions->setMenu(menuOptions);
+
+        /// Option to show all fields or only those require for current entry type
+        actionForceShowAllWidgets = menuOptions->addAction(i18n("Show all fields"), p, SLOT(updateReqOptWidgets()));
+        actionForceShowAllWidgets->setCheckable(true);
+        actionForceShowAllWidgets->setChecked(showAll);
+
+        /// Option to disable tab key focus to reach/visit various non-editable widgets
+        actionLimitKeyboardTabStops = menuOptions->addAction(i18n("Tab key visits only editable fields"), p, SLOT(limitKeyboardTabStops()));
+        actionLimitKeyboardTabStops->setCheckable(true);
+        actionLimitKeyboardTabStops->setChecked(limitKeyboardTabStops);
 
         hLayout->addStretch(10);
 
@@ -198,14 +226,15 @@ public:
                 const int index = tab->indexOf(widget);
                 const bool canEdit = widget->canEdit(element.data());
 
-                if (index >= 0) {
-                    tab->setTabEnabled(index, canEdit);
-                    if (canEdit && index < firstEnabledTab)
-                        firstEnabledTab = index;
-                } else {
+                if (widget == referenceWidget) {
                     /// Reference widget
                     widget->setVisible(canEdit);
                     widget->setEnabled(canEdit);
+                } else {
+                    if (canEdit) tab->showTab(widget);
+                    else if (index >= 0) tab->hideTab(index);
+                    if (canEdit && index >= 0 && index < firstEnabledTab)
+                        firstEnabledTab = index;
                 }
             }
             if (firstEnabledTab < 1024)
@@ -225,14 +254,44 @@ public:
     }
 
     void apply(QSharedPointer<Element> element) {
-        if (referenceWidget != NULL)
-            referenceWidget->apply(element);
-        ElementWidget *currentElementWidget = dynamic_cast<ElementWidget *>(tab->currentWidget());
-        Q_ASSERT_X(currentElementWidget != NULL || tab->currentWidget() == NULL, "ElementEditor::ElementEditorPrivate::apply", "Could not cast currentWidget to ElementWidget");
-        for (WidgetList::ConstIterator it = widgets.constBegin(); it != widgets.constEnd(); ++it)
-            if ((*it) != currentElementWidget && (*it) != sourceWidget)
-                (*it)->apply(element);
-        currentElementWidget->apply(element);
+        if (tab->currentWidget() == sourceWidget) {
+            /// Very simple if source view is active: BibTeX code contains
+            /// all necessary data
+            sourceWidget->apply(element);
+        } else {
+            /// Start by assigning the current internal element's
+            /// data to the output element
+            QSharedPointer<Entry> e = element.dynamicCast<Entry>();
+            if (!e.isNull())
+                *e = *internalEntry;
+            else {
+                QSharedPointer<Macro> m = element.dynamicCast<Macro>();
+                if (!m.isNull())
+                    *m = *internalMacro;
+                else {
+                    QSharedPointer<Comment> c = element.dynamicCast<Comment>();
+                    if (!c.isNull())
+                        *c = *internalComment;
+                    else {
+                        QSharedPointer<Preamble> p = element.dynamicCast<Preamble>();
+                        if (!p.isNull())
+                            *p = *internalPreamble;
+                        else
+                            Q_ASSERT_X(element.isNull(), "ElementEditor::ElementEditorPrivate::apply(QSharedPointer<Element> element)", "element is not NULL but could not be cast on a valid Element sub-class");
+                    }
+                }
+            }
+
+            /// The internal element may be outdated (only updated on tab switch),
+            /// so apply the reference widget's data on the output element
+            if (referenceWidget != NULL)
+                referenceWidget->apply(element);
+            /// The internal element may be outdated (only updated on tab switch),
+            /// so apply the current widget's data on the output element
+            ElementWidget *currentElementWidget = qobject_cast<ElementWidget *>(tab->currentWidget());
+            if (currentElementWidget != NULL)
+                currentElementWidget->apply(element);
+        }
     }
 
     void reset() {
@@ -241,9 +300,9 @@ public:
         reset(element);
 
         /// show checkbox to enable all fields only if editing an entry
-        checkBoxForceShowAllWidgets->setVisible(!internalEntry.isNull());
+        actionForceShowAllWidgets->setVisible(!internalEntry.isNull());
         /// Disable widgets if necessary
-        if (!checkBoxForceShowAllWidgets->isChecked())
+        if (!actionForceShowAllWidgets->isChecked())
             updateReqOptWidgets();
     }
 
@@ -296,23 +355,40 @@ public:
         apply(tempEntry);
 
         /// update the enabled/disabled state of required and optional widgets/fields
-        bool forceVisible = checkBoxForceShowAllWidgets->isChecked();
+        bool forceVisible = actionForceShowAllWidgets->isChecked();
         foreach(ElementWidget *elementWidget, widgets) {
             elementWidget->showReqOptWidgets(forceVisible, tempEntry->type());
         }
 
         /// save configuration
-        QString const configGroupName = QLatin1String("User Interface");
-        QString const keyEnableAllWidgets = QLatin1String("EnableAllWidgets");
+        static const QString configGroupName = QLatin1String("User Interface");
+        static const QString keyEnableAllWidgets = QLatin1String("EnableAllWidgets");
         KConfigGroup configGroup(config, configGroupName);
-        configGroup.writeEntry(keyEnableAllWidgets, checkBoxForceShowAllWidgets->isChecked());
+        configGroup.writeEntry(keyEnableAllWidgets, actionForceShowAllWidgets->isChecked());
         config->sync();
     }
 
-    void switchTo(QWidget *newTab) {
-        bool isSourceWidget = newTab == sourceWidget;
-        ElementWidget *newWidget = dynamic_cast<ElementWidget *>(newTab);
-        if (previousWidget != NULL && newWidget != NULL) {
+    void limitKeyboardTabStops() {
+        /// save configuration
+        static const QString configGroupName = QLatin1String("User Interface");
+        KConfigGroup configGroup(config, configGroupName);
+        configGroup.writeEntry(MenuLineEdit::keyLimitKeyboardTabStops, actionLimitKeyboardTabStops->isChecked());
+        config->sync();
+
+        /// notify all listening MenuLineEdit widgets to change their behavior
+        NotificationHub::publishEvent(MenuLineEdit::MenuLineConfigurationChangedEvent);
+    }
+
+    void switchTo(QWidget *futureTab) {
+        /// Switched from source widget to another widget?
+        const bool isToSourceWidget = futureTab == sourceWidget;
+        /// Switch from some widget to the source widget?
+        const bool isFromSourceWidget = previousWidget == sourceWidget;
+        /// Interprete future widget as an ElementWidget
+        ElementWidget *futureWidget = qobject_cast<ElementWidget *>(futureTab);
+        /// Past and future ElementWidget values are valid?
+        if (previousWidget != NULL && futureWidget != NULL) {
+            /// Assign to temp wihch internal variable holds current state
             QSharedPointer<Element> temp;
             if (!internalEntry.isNull())
                 temp = internalEntry;
@@ -322,18 +398,23 @@ public:
                 temp = internalComment;
             else if (!internalPreamble.isNull())
                 temp = internalPreamble;
-            Q_ASSERT(!temp.isNull());
+            Q_ASSERT_X(!temp.isNull(), "void ElementEditor::ElementEditorPrivate::switchTo(QWidget *newTab)", "temp is NULL");
 
+            /// Past widget writes its state to the internal state
             previousWidget->apply(temp);
-            if (isSourceWidget && referenceWidget != NULL) referenceWidget->apply(temp);
-            newWidget->reset(temp);
-            if (referenceWidget != NULL && dynamic_cast<SourceWidget *>(previousWidget) != NULL)
+            /// Before switching to source widget, store internally reference widget's state
+            if (isToSourceWidget && referenceWidget != NULL) referenceWidget->apply(temp);
+            /// Tell future widget to initialize itself based on internal state
+            futureWidget->reset(temp);
+            /// When switchin from source widget to another widget, initialize reference widget
+            if (isFromSourceWidget && referenceWidget != NULL)
                 referenceWidget->reset(temp);
         }
-        previousWidget = newWidget;
+        previousWidget = futureWidget;
 
+        /// Enable/disable tabs
         for (WidgetList::Iterator it = widgets.begin(); it != widgets.end(); ++it)
-            (*it)->setEnabled(!isSourceWidget || *it == newTab);
+            (*it)->setEnabled(!isToSourceWidget || *it == futureTab);
     }
 
     /**
@@ -354,10 +435,13 @@ public:
             (*it)->setModified(newIsModified);
     }
 
+    void referenceWidgetSetEntryIdByDefault() {
+        referenceWidget->setEntryIdByDefault();
+    }
 };
 
-ElementEditor::ElementEditor(QWidget *parent)
-        : QWidget(parent), d(new ElementEditorPrivate(this))
+ElementEditor::ElementEditor(bool scrollable, QWidget *parent)
+        : QWidget(parent), d(new ElementEditorPrivate(scrollable, this))
 {
     connect(d->tab, SIGNAL(currentChanged(int)), this, SLOT(tabChanged()));
 }
@@ -407,14 +491,14 @@ void ElementEditor::reset()
     emit modified(false);
 }
 
-void ElementEditor::setElement(QSharedPointer<Element> element, const File *file)
+void ElementEditor::setElement(QSharedPointer<Element> element, File *file)
 {
     d->setElement(element, file);
     d->reset();
     emit modified(false);
 }
 
-void ElementEditor::setElement(QSharedPointer<const Element> element, const File *file)
+void ElementEditor::setElement(QSharedPointer<const Element> element, File *file)
 {
     QSharedPointer<Element> clone;
     QSharedPointer<const Entry> entry = element.dynamicCast<const Entry>();
@@ -457,6 +541,17 @@ bool ElementEditor::elementUnapplied()
     return d->elementUnapplied;
 }
 
+QWidget *ElementEditor::currentPage() const
+{
+    return d->tab->currentWidget();
+}
+
+void ElementEditor::setCurrentPage(QWidget *page)
+{
+    if (d->tab->indexOf(page) >= 0)
+        d->tab->setCurrentWidget(page);
+}
+
 void ElementEditor::tabChanged()
 {
     d->switchTo(d->tab->currentWidget());
@@ -469,8 +564,10 @@ void ElementEditor::checkBibTeX()
 
 void ElementEditor::childModified(bool m)
 {
-    if (m)
+    if (m) {
         d->elementUnapplied = true;
+        d->referenceWidgetSetEntryIdByDefault();
+    }
     emit modified(m);
 }
 
@@ -478,3 +575,8 @@ void ElementEditor::updateReqOptWidgets()
 {
     d->updateReqOptWidgets();
 }
+
+void ElementEditor::limitKeyboardTabStops()
+{
+    d->limitKeyboardTabStops();
+}
