@@ -1,39 +1,38 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "elementwidgets.h"
 
 #include <typeinfo>
 
 #include <QLayout>
+#include <QBuffer>
 #include <QLabel>
 #include <QTreeWidget>
 #include <QFileInfo>
 #include <QDropEvent>
 #include <QMenu>
+#include <QStyle>
 
 #include <KPushButton>
 #include <KGlobalSettings>
 #include <KLocale>
 #include <KLineEdit>
 #include <KComboBox>
-#include <KSharedConfig>
-#include <KConfigGroup>
 #include <KDebug>
 #include <KMimeType>
 #include <KRun>
@@ -41,24 +40,24 @@
 
 #include "idsuggestions.h"
 #include "fileinfo.h"
-#include <kbibtexnamespace.h>
-#include <bibtexentries.h>
-#include <bibtexfields.h>
-#include <fileimporterbibtex.h>
-#include <fileexporterbibtex.h>
-#include <file.h>
-#include <fieldinput.h>
-#include <entry.h>
-#include <macro.h>
-#include <preamble.h>
-#include <fieldlineedit.h>
-#include "elementwidgets.h"
+#include "kbibtexnamespace.h"
+#include "bibtexentries.h"
+#include "bibtexfields.h"
+#include "fileimporterbibtex.h"
+#include "fileexporterbibtex.h"
+#include "file.h"
+#include "fieldinput.h"
+#include "entry.h"
+#include "macro.h"
+#include "preamble.h"
+#include "fieldlineedit.h"
 
 static const unsigned int interColumnSpace = 16;
 static const QStringList keyStart = QStringList() << Entry::ftUrl << QLatin1String("postscript") << Entry::ftLocalFile << Entry::ftDOI << Entry::ftFile << QLatin1String("ee") << QLatin1String("biburl");
 static const char *PropertyIdSuggestion = "PropertyIdSuggestion";
 
-ElementWidget::ElementWidget(QWidget *parent): QWidget(parent), isReadOnly(false), m_isModified(false)
+ElementWidget::ElementWidget(QWidget *parent)
+        : QWidget(parent), isReadOnly(false), m_file(NULL), m_isModified(false)
 {
     // nothing
 };
@@ -241,10 +240,10 @@ void EntryConfiguredWidget::layoutGUI(bool forceVisible, const QString &entryTyp
                 /// this ugly conversion is necessary because we have a "^" (xor) and "|" (and/or)
                 /// syntax to differentiate required items (not used yet, but will be used
                 /// later if missing required items are marked).
-                QString visible = bit->requiredItems.join(",");
-                visible += "," + bit->optionalItems.join(",");
-                visible = visible.replace("|", ",").replace("^", ",");
-                visibleItems = visible.split(",");
+                QString visible = bit->requiredItems.join(QLatin1String(","));
+                visible += QLatin1Char(',') + bit->optionalItems.join(QLatin1String(","));
+                visible = visible.replace(QLatin1Char('|'), QLatin1Char(',')).replace(QLatin1Char('^'), QLatin1Char(','));
+                visibleItems = visible.split(QLatin1String(","));
                 break;
             }
         }
@@ -326,7 +325,7 @@ void EntryConfiguredWidget::layoutGUI(bool forceVisible, const QString &entryTyp
 }
 
 ReferenceWidget::ReferenceWidget(QWidget *parent)
-        : ElementWidget(parent), m_applyElement(NULL), m_element(QSharedPointer<Element>())
+        : ElementWidget(parent), m_applyElement(NULL), m_entryIdManuallySet(false), m_element(QSharedPointer<Element>())
 {
     createGUI();
 }
@@ -339,7 +338,7 @@ bool ReferenceWidget::apply(QSharedPointer<Element> element) const
     QSharedPointer<Entry> entry = element.dynamicCast<Entry>();
     if (!entry.isNull()) {
         BibTeXEntries *be = BibTeXEntries::self();
-        QString type = QString::null;
+        QString type;
         if (entryType->currentIndex() < 0 || entryType->lineEdit()->isModified())
             type = be->format(entryType->lineEdit()->text(), KBibTeX::cUpperCamelCase);
         else
@@ -364,7 +363,7 @@ bool ReferenceWidget::reset(QSharedPointer<const Element> element)
     /// if signals are not deactivated, the "modified" signal would be emitted when
     /// resetting the widgets' values
     disconnect(entryType, SIGNAL(editTextChanged(QString)), this, SLOT(gotModified()));
-    disconnect(entryId, SIGNAL(textChanged(QString)), this, SLOT(gotModified()));
+    disconnect(entryId, SIGNAL(textChanged(QString)), this, SLOT(entryIdManuallyChanged()));
 
     bool result = false;
     QSharedPointer<const Entry> entry = element.dynamicCast<const Entry>();
@@ -382,7 +381,14 @@ bool ReferenceWidget::reset(QSharedPointer<const Element> element)
                 entryType->setCurrentIndex(index);
                 break;
             }
+
         entryId->setText(entry->id());
+        /// New entries have no values. Use this fact
+        /// to recognize new entries, for which it is
+        /// allowed to automatic set their ids
+        /// if a default id suggestion had been specified.
+        m_entryIdManuallySet = entry->count() > 0;
+
         result = true;
     } else {
         entryType->setEnabled(false);
@@ -396,7 +402,7 @@ bool ReferenceWidget::reset(QSharedPointer<const Element> element)
     }
 
     connect(entryType, SIGNAL(editTextChanged(QString)), this, SLOT(gotModified()));
-    connect(entryId, SIGNAL(textChanged(QString)), this, SLOT(gotModified()));
+    connect(entryId, SIGNAL(textChanged(QString)), this, SLOT(entryIdManuallyChanged()));
 
     return result;
 }
@@ -411,7 +417,7 @@ void ReferenceWidget::setReadOnly(bool isReadOnly)
 
 QString ReferenceWidget::label()
 {
-    return QString::null;
+    return QString();
 }
 
 KIcon ReferenceWidget::icon()
@@ -470,7 +476,7 @@ void ReferenceWidget::createGUI()
     buttonSuggestId->setMenu(suggestionsMenu);
 
     connect(entryType->lineEdit(), SIGNAL(textEdited(QString)), this, SLOT(gotModified()));
-    connect(entryId, SIGNAL(textEdited(QString)), this, SLOT(gotModified()));
+    connect(entryId, SIGNAL(textEdited(QString)), this, SLOT(entryIdManuallyChanged()));
     connect(entryType->lineEdit(), SIGNAL(textEdited(QString)), this, SIGNAL(entryTypeChanged()));
     connect(entryType, SIGNAL(currentIndexChanged(int)), this, SIGNAL(entryTypeChanged()));
     connect(suggestionsMenu, SIGNAL(aboutToShow()), this, SLOT(prepareSuggestionsMenu()));
@@ -489,7 +495,7 @@ void ReferenceWidget::prepareSuggestionsMenu()
 
     /// Keep track of shown suggestions to avoid duplicates
     QSet<QString> knownIdSuggestion;
-    QString defaultSuggestion = idSuggestions->defaultFormatId(*crossrefResolvedEntry.data());
+    const QString defaultSuggestion = idSuggestions->defaultFormatId(*crossrefResolvedEntry.data());
 
     foreach(const QString &suggestionBase, idSuggestions->formatIdList(*crossrefResolvedEntry.data())) {
         bool isDefault = suggestionBase == defaultSuggestion;
@@ -520,18 +526,56 @@ void ReferenceWidget::prepareSuggestionsMenu()
 
 void ReferenceWidget::insertSuggestionFromAction()
 {
-    QAction *action = dynamic_cast<QAction *>(sender());
+    QAction *action = qobject_cast<QAction *>(sender());
     if (action != NULL) {
         const QString suggestion = action->property(PropertyIdSuggestion).toString();
         entryId->setText(suggestion);
     }
 }
 
+void ReferenceWidget::entryIdManuallyChanged()
+{
+    m_entryIdManuallySet = true;
+    gotModified();
+}
+
+void ReferenceWidget::setEntryIdByDefault()
+{
+    if (isReadOnly) {
+        /// Never set the suggestion automatically if in read-only mode
+        return;
+    }
+
+    if (m_entryIdManuallySet) {
+        /// If user changed entry id manually,
+        /// do not overwrite it by a default value
+        return;
+    }
+
+    static const IdSuggestions *idSuggestions = new IdSuggestions();
+    /// If there is a default suggestion format set ...
+    if (idSuggestions->hasDefaultFormat())  {
+        /// Collect information on the current entry as it is edited
+        QSharedPointer<Entry> guiDataEntry(new Entry());
+        m_applyElement->apply(guiDataEntry);
+        QSharedPointer<Entry> crossrefResolvedEntry(Entry::resolveCrossref(*guiDataEntry.data(), m_file));
+        /// Determine default suggestion based on current data
+        const QString defaultSuggestion = idSuggestions->defaultFormatId(*crossrefResolvedEntry.data());
+
+        if (!defaultSuggestion.isEmpty()) {
+            disconnect(entryId, SIGNAL(textChanged(QString)), this, SLOT(entryIdManuallyChanged()));
+            /// Apply default suggestion to widget
+            entryId->setText(defaultSuggestion);
+            connect(entryId, SIGNAL(textChanged(QString)), this, SLOT(entryIdManuallyChanged()));
+        }
+    }
+}
+
 FilesWidget::FilesWidget(QWidget *parent)
         : ElementWidget(parent)
 {
     QVBoxLayout *layout = new QVBoxLayout(this);
-    fileList = new FieldInput(KBibTeX::UrlList, KBibTeX::tfVerbatim, KBibTeX::tfVerbatim, this);
+    fileList = new FieldInput(KBibTeX::UrlList, KBibTeX::tfVerbatim /* eventually ignored, see constructor of UrlListEdit */, KBibTeX::tfVerbatim /* eventually ignored, see constructor of UrlListEdit */, this);
     fileList->setFieldKey(QLatin1String("^external"));
     layout->addWidget(fileList);
     connect(fileList, SIGNAL(modified()), this, SLOT(gotModified()));
@@ -559,7 +603,7 @@ bool FilesWidget::apply(QSharedPointer<Element> element) const
     for (Value::ConstIterator it = combinedValue.constBegin(); it != combinedValue.constEnd(); ++it) {
         const QSharedPointer<VerbatimText> verbatimText = (*it).dynamicCast<VerbatimText>();
         if (!verbatimText.isNull()) {
-            QString text = verbatimText->text();
+            const QString text = verbatimText->text();
             if (KBibTeX::urlRegExp.indexIn(text) > -1) {
                 /// add full URL
                 VerbatimText *newVT = new VerbatimText(KBibTeX::urlRegExp.cap(0));
@@ -804,12 +848,6 @@ void OtherFieldsWidget::actionOpen()
 
 void OtherFieldsWidget::createGUI()
 {
-    /// retrieve information from settings if labels should be
-    /// above widgets or on the left side
-    KSharedConfigPtr config(KSharedConfig::openConfig(QLatin1String("kbibtexrc")));
-    const QString configGroupName(QLatin1String("User Interface"));
-    KConfigGroup configGroup(config, configGroupName);
-
     QGridLayout *layout = new QGridLayout(this);
     /// set row and column stretches based on chosen layout
     layout->setColumnStretch(0, 0);
@@ -857,8 +895,8 @@ void OtherFieldsWidget::createGUI()
     buttonOpen->setEnabled(false);
     layout->addWidget(buttonOpen, 3, 2, 1, 1);
 
-    connect(otherFieldsList, SIGNAL(itemActivated(QTreeWidgetItem *, int)), this, SLOT(listElementExecuted(QTreeWidgetItem *, int)));
-    connect(otherFieldsList, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)), this, SLOT(listCurrentChanged(QTreeWidgetItem *, QTreeWidgetItem *)));
+    connect(otherFieldsList, SIGNAL(itemActivated(QTreeWidgetItem*,int)), this, SLOT(listElementExecuted(QTreeWidgetItem*,int)));
+    connect(otherFieldsList, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), this, SLOT(listCurrentChanged(QTreeWidgetItem*,QTreeWidgetItem*)));
     connect(otherFieldsList, SIGNAL(itemSelectionChanged()), this, SLOT(updateGUI()));
     connect(fieldName, SIGNAL(textEdited(QString)), this, SLOT(updateGUI()));
     connect(buttonAddApply, SIGNAL(clicked()), this, SLOT(actionAddApply()));
@@ -868,8 +906,8 @@ void OtherFieldsWidget::createGUI()
 
 void OtherFieldsWidget::updateList()
 {
-    QString selText = otherFieldsList->selectedItems().isEmpty() ? QString::null : otherFieldsList->selectedItems().first()->text(0);
-    QString curText = otherFieldsList->currentItem() == NULL ? QString::null : otherFieldsList->currentItem()->text(0);
+    const QString selText = otherFieldsList->selectedItems().isEmpty() ? QString() : otherFieldsList->selectedItems().first()->text(0);
+    const QString curText = otherFieldsList->currentItem() == NULL ? QString() : otherFieldsList->currentItem()->text(0);
     otherFieldsList->clear();
 
     for (Entry::ConstIterator it = internalEntry->constBegin(); it != internalEntry->constEnd(); ++it)
@@ -954,7 +992,7 @@ bool MacroWidget::canEdit(const Element *element)
 
 void MacroWidget::createGUI()
 {
-    QVBoxLayout *layout = new QVBoxLayout(this);
+    QBoxLayout *layout = new QHBoxLayout(this);
 
     QLabel *label = new QLabel(i18n("Value:"), this);
     layout->addWidget(label, 0);
@@ -1019,7 +1057,7 @@ bool PreambleWidget::canEdit(const Element *element)
 
 void PreambleWidget::createGUI()
 {
-    QVBoxLayout *layout = new QVBoxLayout(this);
+    QBoxLayout *layout = new QHBoxLayout(this);
 
     QLabel *label = new QLabel(i18n("Value:"), this);
     layout->addWidget(label, 0);
@@ -1046,7 +1084,7 @@ protected:
         FileExporterBibTeX exporter;
         const File *file = importer.fromString(event->mimeData()->text());
         if (file->count() == 1)
-            document()->setPlainText(exporter.toString(file->first()));
+            document()->setPlainText(exporter.toString(file->first(), file));
         else
             KTextEdit::dropEvent(event);
     }
@@ -1065,9 +1103,9 @@ SourceWidget::~SourceWidget()
 
 bool SourceWidget::apply(QSharedPointer<Element> element) const
 {
-    if (isReadOnly) return false; /// never save data if in read-only mode
+    if (isReadOnly) return false; ///< never save data if in read-only mode
 
-    QString text = sourceEdit->document()->toPlainText();
+    const QString text = sourceEdit->document()->toPlainText();
     FileImporterBibTeX importer;
     File *file = importer.fromString(text);
     if (file == NULL) return false;
@@ -1091,10 +1129,12 @@ bool SourceWidget::apply(QSharedPointer<Element> element) const
                 if (!readPreamble.isNull() && !preamble.isNull()) {
                     preamble->operator =(*readPreamble.data());
                     result = true;
-                }
+                } else
+                    kWarning() << "Do not know how to apply source code";
             }
         }
-    }
+    } else
+        kDebug() << "Expected exactly 1 BibTeX element in source code, but found" << file->count() << "instead";
 
     delete file;
     return result;
@@ -1108,15 +1148,15 @@ bool SourceWidget::reset(QSharedPointer<const Element> element)
 
     FileExporterBibTeX exporter;
     exporter.setEncoding(QLatin1String("utf-8"));
-    const QString exportedText = exporter.toString(element);
-    if (!exportedText.isNull()) {
+    const QString exportedText = exporter.toString(element, m_file);
+    if (!exportedText.isEmpty()) {
         originalText = exportedText;
         sourceEdit->document()->setPlainText(originalText);
     }
 
     connect(sourceEdit, SIGNAL(textChanged()), this, SLOT(gotModified()));
 
-    return !exportedText.isNull();
+    return !exportedText.isEmpty();
 }
 
 void SourceWidget::setReadOnly(bool isReadOnly)
