@@ -1,25 +1,22 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
 
-#ifndef KBIBTEX_GUI_DIALOGS_ELEMENTSWIDGETS_H
-#define KBIBTEX_GUI_DIALOGS_ELEMENTSWIDGETS_H
+#ifndef KBIBTEX_GUI_ELEMENTWIDGETS_H
+#define KBIBTEX_GUI_ELEMENTWIDGETS_H
 
 #include "kbibtexgui_export.h"
 
@@ -50,7 +47,7 @@ class ElementWidget : public QWidget
     Q_OBJECT
 
 public:
-    ElementWidget(QWidget *parent);
+    explicit ElementWidget(QWidget *parent);
     virtual bool apply(QSharedPointer<Element> element) const = 0;
     virtual bool reset(QSharedPointer<const Element> element) = 0;
     virtual void setReadOnly(bool isReadOnly) {
@@ -98,7 +95,7 @@ private:
     QMap<QString, FieldInput *> bibtexKeyToWidget;
 
     void createGUI();
-    void layoutGUI(bool forceVisible, const QString &entryType = QString::null);
+    void layoutGUI(bool forceVisible, const QString &entryType = QString());
 
 public:
     EntryConfiguredWidget(QSharedPointer<EntryTabLayout> &entryTabLayout, QWidget *parent);
@@ -128,12 +125,12 @@ private:
     void createGUI();
 
 public:
-    ReferenceWidget(QWidget *parent);
+    explicit ReferenceWidget(QWidget *parent);
 
     bool apply(QSharedPointer<Element> element) const;
     bool reset(QSharedPointer<const Element> element);
     void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {};
+    void showReqOptWidgets(bool, const QString &) {}
     void setApplyElementInterface(ElementEditor::ApplyElementInterface *applyElement) {
         m_applyElement = applyElement;
     }
@@ -146,13 +143,18 @@ public:
 
     bool canEdit(const Element *element);
 
+public slots:
+    void setEntryIdByDefault();
+
 private:
     ElementEditor::ApplyElementInterface *m_applyElement;
+    bool m_entryIdManuallySet;
     QSharedPointer<Element> m_element;
 
 private slots:
     void prepareSuggestionsMenu();
     void insertSuggestionFromAction();
+    void entryIdManuallyChanged();
 
 signals:
     void entryTypeChanged();
@@ -164,12 +166,15 @@ private:
     FieldInput *fileList;
 
 public:
-    FilesWidget(QWidget *parent);
+    explicit FilesWidget(QWidget *parent);
 
     bool apply(QSharedPointer<Element> element) const;
     bool reset(QSharedPointer<const Element> element);
     void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {};
+    void showReqOptWidgets(bool, const QString &) {}
+    void setApplyElementInterface(ElementEditor::ApplyElementInterface *applyElement) {
+        m_applyElement = applyElement;
+    }
 
     QString label();
     KIcon icon();
@@ -177,6 +182,9 @@ public:
     virtual void setFile(const File *file);
 
     bool canEdit(const Element *element);
+
+private:
+    ElementEditor::ApplyElementInterface *m_applyElement;
 };
 
 class OtherFieldsWidget : public ElementWidget
@@ -206,7 +214,7 @@ public:
     bool apply(QSharedPointer<Element> element) const;
     bool reset(QSharedPointer<const Element> element);
     void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {};
+    void showReqOptWidgets(bool, const QString &) {}
     QString label();
     KIcon icon();
 
@@ -229,13 +237,13 @@ private:
     void createGUI();
 
 public:
-    MacroWidget(QWidget *parent);
+    explicit MacroWidget(QWidget *parent);
     ~MacroWidget();
 
     bool apply(QSharedPointer<Element> element) const;
     bool reset(QSharedPointer<const Element> element);
     void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {};
+    void showReqOptWidgets(bool, const QString &) {}
     QString label();
     KIcon icon();
 
@@ -250,12 +258,12 @@ private:
     void createGUI();
 
 public:
-    PreambleWidget(QWidget *parent);
+    explicit PreambleWidget(QWidget *parent);
 
     bool apply(QSharedPointer<Element> element) const;
     bool reset(QSharedPointer<const Element> element);
     void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {};
+    void showReqOptWidgets(bool, const QString &) {}
     QString label();
     KIcon icon();
 
@@ -274,13 +282,13 @@ private:
     void createGUI();
 
 public:
-    SourceWidget(QWidget *parent);
+    explicit SourceWidget(QWidget *parent);
     ~SourceWidget();
 
     bool apply(QSharedPointer<Element> element) const;
     bool reset(QSharedPointer<const Element> element);
     void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {};
+    void showReqOptWidgets(bool, const QString &) {}
     QString label();
     KIcon icon();
 
@@ -293,4 +301,4 @@ private:
     KPushButton *m_buttonRestore;
 };
 
-#endif // KBIBTEX_GUI_DIALOGS_ELEMENTSWIDGETS_H
+#endif // KBIBTEX_GUI_ELEMENTWIDGETS_H
