@@ -1,22 +1,22 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "findpdfui.h"
+#include "findpdfui_p.h"
 
 #include <QGridLayout>
 #include <QListView>
@@ -43,9 +43,8 @@
 #include <KFileDialog>
 #include <KIO/NetAccess>
 
-#include <fieldlistedit.h>
-#include "findpdfui.h"
 #include "fileinfo.h"
+#include "fieldlistedit.h"
 
 class PDFListModel;
 
@@ -107,13 +106,13 @@ QList<QWidget *> PDFItemDelegate::createItemWidgets() const
     previewLabel->setBackgroundRole(QPalette::NoRole);
     previewLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
     list << previewLabel;
-    Q_ASSERT_X(list.count() == posLabelUrl + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posLabelUrl + 1");
+    Q_ASSERT_X(list.count() == posLabelPreview + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posLabelPreview + 1");
 
     /// add a push button to view the PDF file
     KPushButton *pushButton = new KPushButton(KIcon("application-pdf"), i18n("View"));
     list << pushButton;
     connect(pushButton, SIGNAL(clicked()), this, SLOT(slotViewPDF()));
-    Q_ASSERT_X(list.count() == posLabelUrl + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posLabelUrl + 1");
+    Q_ASSERT_X(list.count() == posViewButton + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posViewButton + 1");
 
     /// a button group to choose what to do with this particular PDF file
     QButtonGroup *bg = new QButtonGroup();
@@ -123,21 +122,21 @@ QList<QWidget *> PDFItemDelegate::createItemWidgets() const
     bg->addButton(radioButton);
     list << radioButton;
     connect(radioButton, SIGNAL(toggled(bool)), this, SLOT(slotRadioNoDownloadToggled(bool)));
-    Q_ASSERT_X(list.count() == posLabelUrl + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posLabelUrl + 1");
+    Q_ASSERT_X(list.count() == posRadioNoDownload + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posRadioNoDownload + 1");
 
     /// download this file and store it locally, user will be asked for "Save As"
     radioButton = new QRadioButton(i18n("Download"));
     bg->addButton(radioButton);
     list << radioButton;
     connect(radioButton, SIGNAL(toggled(bool)), this, SLOT(slotRadioDownloadToggled(bool)));
-    Q_ASSERT_X(list.count() == posLabelUrl + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posLabelUrl + 1");
+    Q_ASSERT_X(list.count() == posRadioDownload + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posRadioDownload + 1");
 
     /// paste URL into BibTeX entry, no local copy is stored
     radioButton = new QRadioButton(i18n("Use URL only"));
     bg->addButton(radioButton);
     list << radioButton;
     connect(radioButton, SIGNAL(toggled(bool)), this, SLOT(slotRadioURLonlyToggled(bool)));
-    Q_ASSERT_X(list.count() == posLabelUrl + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posLabelUrl + 1");
+    Q_ASSERT_X(list.count() == posRadioURLonly + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posRadioURLonly + 1");
 
     return list;
 }
@@ -315,9 +314,7 @@ bool PDFListModel::setData(const QModelIndex &index, const QVariant &value, int
     if (index != QModelIndex() && index.row() < m_resultList.count() && role == DownloadModeRole) {
         bool ok = false;
         FindPDF::DownloadMode downloadMode = (FindPDF::DownloadMode)value.toInt(&ok);
-        kDebug() << "FindPDF::DownloadMode=" << downloadMode << ok;
         if (ok) {
-            kDebug()    << "Setting row " << index.row();
             m_resultList[index.row()].downloadMode = downloadMode;
             return true;
         }
@@ -339,46 +336,86 @@ QVariant PDFListModel::headerData(int section, Qt::Orientation orientation, int
 }
 
 
-FindPDFUI::FindPDFUI(Entry &entry, QWidget *parent)
-        : QWidget(parent), m_findpdf(new FindPDF(this))
+class FindPDFUI::Private
 {
-    createGUI();
+private:
+    FindPDFUI *p;
 
-    m_labelMessage->show();
-    m_labelMessage->setText(i18n("Starting to search ..."));
+public:
+    QListView *listViewResult;
+    QLabel *labelMessage;
+    QList<FindPDF::ResultItem> resultList;
+    FindPDF *findpdf;
 
-    connect(m_findpdf, SIGNAL(finished()), this, SLOT(searchFinished()));
-    connect(m_findpdf, SIGNAL(progress(int, int)), this, SLOT(searchProgress(int, int)));
-    m_findpdf->search(entry);
+    Private(FindPDFUI *parent)
+            : p(parent), findpdf(new FindPDF(parent))
+    {
+        /// nothing
+    }
+
+    void createGUI()
+    {
+        QGridLayout *layout = new QGridLayout(p);
+
+        const int minWidth = p->fontMetrics().height() * 40;
+        const int minHeight = p->fontMetrics().height() * 20;
+        p->setMinimumSize(minWidth, minHeight);
+
+        listViewResult = new QListView(p);
+        layout->addWidget(listViewResult, 0, 0);
+        listViewResult->setEnabled(false);
+        listViewResult->hide();
+
+        labelMessage = new QLabel(p);
+        layout->addWidget(labelMessage, 1, 0);
+        labelMessage->setMinimumSize(minWidth, minHeight);
+        labelMessage->setAlignment(Qt::AlignVCenter | Qt::AlignCenter);
+
+        static_cast<QWidget *>(p->parent())->setCursor(Qt::WaitCursor);
+    }
+};
+
+
+FindPDFUI::FindPDFUI(Entry &entry, QWidget *parent)
+        : QWidget(parent), d(new Private(this))
+{
+    d->createGUI();
+
+    d->labelMessage->show();
+    d->labelMessage->setText(i18n("Starting to search..."));
+
+    connect(d->findpdf, SIGNAL(finished()), this, SLOT(searchFinished()));
+    connect(d->findpdf, SIGNAL(progress(int,int,int)), this, SLOT(searchProgress(int,int,int)));
+    d->findpdf->search(entry);
 }
 
 FindPDFUI::~FindPDFUI()
 {
-    for (QList<FindPDF::ResultItem>::Iterator it = m_resultList.begin(); it != m_resultList.end();) {
+    for (QList<FindPDF::ResultItem>::Iterator it = d->resultList.begin(); it != d->resultList.end();) {
         delete it->tempFilename;
-        it = m_resultList.erase(it);
+        it = d->resultList.erase(it);
     }
 }
 
 void FindPDFUI::interactiveFindPDF(Entry &entry, const File &bibtexFile, QWidget *parent)
 {
     QPointer<KDialog> dlg = new KDialog(parent);
-    FindPDFUI widget(entry, dlg);
+    QPointer<FindPDFUI> widget = new FindPDFUI(entry, dlg);
     dlg->setCaption(i18n("Find PDF"));
-    dlg->setMainWidget(&widget);
+    dlg->setMainWidget(widget);
     dlg->enableButtonOk(false);
 
-    connect(&widget, SIGNAL(resultAvailable(bool)), dlg, SLOT(enableButtonOk(bool)));
+    connect(widget, SIGNAL(resultAvailable(bool)), dlg, SLOT(enableButtonOk(bool)));
 
     if (dlg->exec() == KDialog::Accepted) {
-        widget.apply(entry, bibtexFile);
+        widget->apply(entry, bibtexFile);
     }
     delete dlg;
 }
 
 void FindPDFUI::apply(Entry &entry, const File &bibtexFile)
 {
-    QAbstractItemModel *model = m_listViewResult->model();
+    QAbstractItemModel *model = d->listViewResult->model();
     for (int i = 0; i < model->rowCount(); ++i) {
         bool ok = false;
         FindPDF::DownloadMode downloadMode = (FindPDF::DownloadMode)model->data(model->index(i, 0), DownloadModeRole).toInt(&ok);
@@ -441,45 +478,24 @@ void FindPDFUI::apply(Entry &entry, const File &bibtexFile)
     }
 }
 
-void FindPDFUI::createGUI()
-{
-    QGridLayout *layout = new QGridLayout(this);
-
-    const int minWidth = fontMetrics().height() * 40;
-    const int minHeight = fontMetrics().height() * 20;
-    setMinimumSize(minWidth, minHeight);
-
-    m_listViewResult = new QListView(this);
-    layout->addWidget(m_listViewResult, 0, 0);
-    m_listViewResult->setEnabled(false);
-    m_listViewResult->hide();
-
-    m_labelMessage = new QLabel(this);
-    layout->addWidget(m_labelMessage, 1, 0);
-    m_labelMessage->setMinimumSize(minWidth, minHeight);
-    m_labelMessage->setAlignment(Qt::AlignVCenter | Qt::AlignCenter);
-
-    static_cast<QWidget *>(parent())->setCursor(Qt::WaitCursor);
-}
-
 void FindPDFUI::searchFinished()
 {
-    m_labelMessage->hide();
-    m_listViewResult->show();
+    d->labelMessage->hide();
+    d->listViewResult->show();
 
-    m_resultList = m_findpdf->results();
-    m_listViewResult->setModel(new PDFListModel(m_resultList, m_listViewResult));
-    m_listViewResult->setItemDelegate(new PDFItemDelegate(m_listViewResult, m_listViewResult));
-    m_listViewResult->setEnabled(true);
-    m_listViewResult->reset();
+    d->resultList = d->findpdf->results();
+    d->listViewResult->setModel(new PDFListModel(d->resultList, d->listViewResult));
+    d->listViewResult->setItemDelegate(new PDFItemDelegate(d->listViewResult, d->listViewResult));
+    d->listViewResult->setEnabled(true);
+    d->listViewResult->reset();
 
     static_cast<QWidget *>(parent())->unsetCursor();
     emit resultAvailable(true);
 }
 
-void FindPDFUI::searchProgress(int visitedPages, int foundDocuments)
+void FindPDFUI::searchProgress(int visitedPages, int runningJobs, int foundDocuments)
 {
-    m_listViewResult->hide();
-    m_labelMessage->show();
-    m_labelMessage->setText(i18n("Searching ...\nNumber of visited pages: %1\nNumber of found documents: %2", visitedPages, foundDocuments));
+    d->listViewResult->hide();
+    d->labelMessage->show();
+    d->labelMessage->setText(i18n("<qt>Number of visited pages: <b>%1</b><br/>Number of running downloads: <b>%2</b><br/>Number of found documents: <b>%3</b></qt>", visitedPages, runningJobs, foundDocuments));
 }
