@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "fieldlineedit.h"
 
 #include <typeinfo>
 
@@ -39,15 +38,15 @@
 #include <KSharedConfig>
 #include <KConfigGroup>
 
-#include <fileinfo.h>
-#include <file.h>
-#include <entry.h>
-#include <value.h>
-#include <fileimporterbibtex.h>
-#include <fileexporterbibtex.h>
-#include <bibtexfields.h>
-#include <encoderlatex.h>
-#include "fieldlineedit.h"
+#include "fileinfo.h"
+#include "file.h"
+#include "entry.h"
+#include "value.h"
+#include "fileimporterbibtex.h"
+#include "fileexporterbibtex.h"
+#include "bibtexfields.h"
+#include "encoderlatex.h"
+#include "preferences.h"
 
 class FieldLineEdit::FieldLineEditPrivate
 {
@@ -72,7 +71,7 @@ public:
 
     FieldLineEditPrivate(KBibTeX::TypeFlag ptf, KBibTeX::TypeFlags tf, FieldLineEdit *p)
             : parent(p), preferredTypeFlag(ptf), typeFlags(tf), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), configGroupNameGeneral(QLatin1String("General")), file(NULL) {
-        menuTypes = new QMenu(i18nc("Value types like Source, Verbatim, Person, PlainText, ...", "Types"), parent);
+        menuTypes = new QMenu(parent);
         menuTypesSignalMapper = new QSignalMapper(parent);
         setupMenu();
         connect(menuTypesSignalMapper, SIGNAL(mapped(int)), parent, SLOT(slotTypeChanged(int)));
@@ -90,7 +89,7 @@ public:
         updateGUI(typeFlag);
 
         KConfigGroup configGroup(config, configGroupNameGeneral);
-        personNameFormatting = configGroup.readEntry(Person::keyPersonNameFormatting, Person::defaultPersonNameFormatting);
+        personNameFormatting = configGroup.readEntry(Preferences::keyPersonNameFormatting, Preferences::defaultPersonNameFormatting);
     }
 
     ~FieldLineEditPrivate() {
@@ -157,12 +156,12 @@ public:
 
     bool apply(Value &value) const {
         value.clear();
-        const QString text = parent->text().simplified();
+        /// Remove unnecessary white space from input
+        /// Exception: source and verbatim content is kept unmodified
+        const QString text = typeFlag == KBibTeX::tfSource || typeFlag == KBibTeX::tfVerbatim ? parent->text() : parent->text().simplified();
 
         EncoderLaTeX *encoder = EncoderLaTeX::instance();
         const QString encodedText = encoder->decode(text);
-        if (encodedText != text)
-            parent->setText(encodedText);
 
         if (encodedText.isEmpty())
             return true;
@@ -173,7 +172,9 @@ public:
             value.append(QSharedPointer<MacroKey>(new MacroKey(encodedText)));
             return true;
         } else if (typeFlag == KBibTeX::tfPerson) {
-            value.append(FileImporterBibTeX::personFromString(encodedText));
+            QSharedPointer<Person> person = FileImporterBibTeX::personFromString(encodedText);
+            if (!person.isNull())
+                value.append(person);
             return true;
         } else if (typeFlag == KBibTeX::tfKeyword) {
             QList<QSharedPointer<Keyword> > keywords = FileImporterBibTeX::splitKeywords(encodedText);
@@ -313,7 +314,7 @@ public:
 
         bool result = true;
         EncoderLaTeX *enc = EncoderLaTeX::instance();
-        QString rawText = QString::null;
+        QString rawText;
         const QSharedPointer<ValueItem> first = value.first();
 
         const QSharedPointer<PlainText> plainText = first.dynamicCast<PlainText>();
@@ -383,7 +384,7 @@ public:
 
     void updateURL(const QString &text) {
         QList<KUrl> urls;
-        FileInfo::urlsInText(text, FileInfo::TestExistanceYes, file != NULL && file->property(File::Url).toUrl().isValid() ? KUrl(file->property(File::Url).toUrl()).directory() : QString::null, urls);
+        FileInfo::urlsInText(text, FileInfo::TestExistenceYes, file != NULL && file->property(File::Url).toUrl().isValid() ? KUrl(file->property(File::Url).toUrl()).directory() : QString(), urls);
         if (!urls.isEmpty() && urls.first().isValid())
             urlToOpen = urls.first();
         else
@@ -480,7 +481,7 @@ void FieldLineEdit::dropEvent(QDropEvent *event)
     if (clipboardText.isEmpty()) return;
 
     const File *file = NULL;
-    if (!d->fieldKey.isEmpty() && clipboardText.startsWith("@")) {
+    if (!d->fieldKey.isEmpty() && clipboardText.startsWith(QLatin1String("@"))) {
         FileImporterBibTeX importer;
         file = importer.fromString(clipboardText);
         const QSharedPointer<Entry> entry = (file != NULL && file->count() == 1) ? file->first().dynamicCast<Entry>() : QSharedPointer<Entry>();
