@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "fieldlistedit.h"
 
 #include <typeinfo>
 
@@ -30,6 +29,7 @@
 #include <QDropEvent>
 #include <QUrl>
 #include <QTimer>
+#include <QAction>
 
 #include <KMessageBox>
 #include <KLocale>
@@ -38,14 +38,17 @@
 #include <KInputDialog>
 #include <KIO/NetAccess>
 #include <KGlobalSettings>
+#include <KMenu>
+#include <KDebug>
 
-#include <fileinfo.h>
-#include <file.h>
-#include <entry.h>
-#include <fileimporterbibtex.h>
-#include <fileexporterbibtex.h>
-#include <fieldlineedit.h>
-#include "fieldlistedit.h"
+#include "fileinfo.h"
+#include "file.h"
+#include "entry.h"
+#include "fileimporterbibtex.h"
+#include "fileexporterbibtex.h"
+#include "fieldlineedit.h"
+#include "associatedfiles.h"
+#include "associatedfilesui.h"
 
 class FieldListEdit::FieldListEditProtected
 {
@@ -105,16 +108,14 @@ public:
         addLineButton = new KPushButton(KIcon("list-add"), i18n("Add"), pushButtonContainer);
         addLineButton->setObjectName(QLatin1String("addButton"));
         connect(addLineButton, SIGNAL(clicked()), p, SLOT(lineAdd()));
+        connect(addLineButton, SIGNAL(clicked()), p, SIGNAL(modified()));
         pushButtonContainerLayout->addWidget(addLineButton);
 
         layout->addStretch(100);
 
-        connect(smRemove, SIGNAL(mapped(QWidget *)), p, SLOT(lineRemove(QWidget *)));
-        connect(smRemove, SIGNAL(mapped(QWidget *)), p, SIGNAL(modified()));
-        connect(smGoDown, SIGNAL(mapped(QWidget *)), p, SLOT(lineGoDown(QWidget *)));
-        connect(smGoDown, SIGNAL(mapped(QWidget *)), p, SIGNAL(modified()));
-        connect(smGoUp, SIGNAL(mapped(QWidget *)), p, SLOT(lineGoUp(QWidget *)));
-        connect(smGoDown, SIGNAL(mapped(QWidget *)), p, SIGNAL(modified()));
+        connect(smRemove, SIGNAL(mapped(QWidget*)), p, SLOT(lineRemove(QWidget*)));
+        connect(smGoDown, SIGNAL(mapped(QWidget*)), p, SLOT(lineGoDown(QWidget*)));
+        connect(smGoUp, SIGNAL(mapped(QWidget*)), p, SLOT(lineGoUp(QWidget*)));
 
         scrollArea->setBackgroundRole(QPalette::Base);
         scrollArea->ensureWidgetVisible(container);
@@ -194,7 +195,7 @@ public:
 
     void goDownFieldLineEdit(FieldLineEdit *fieldLineEdit) {
         int idx = lineEditList.indexOf(fieldLineEdit);
-        if (idx < lineEditList.count() - lineEditList.size()) {
+        if (idx < lineEditList.count() - 1) {
             layout->removeWidget(fieldLineEdit);
             lineEditList.removeOne(fieldLineEdit);
             lineEditList.insert(idx + 1, fieldLineEdit);
@@ -319,14 +320,14 @@ void FieldListEdit::dropEvent(QDropEvent *event)
     if (clipboardText.isEmpty()) return;
 
     const File *file = NULL;
-    if (!d->fieldKey.isEmpty() && clipboardText.startsWith("@")) {
+    if (!d->fieldKey.isEmpty() && clipboardText.startsWith(QLatin1String("@"))) {
         FileImporterBibTeX importer;
         file = importer.fromString(clipboardText);
         const QSharedPointer<Entry> entry = (file != NULL && file->count() == 1) ? file->first().dynamicCast<Entry>() : QSharedPointer<Entry>();
 
-        if (!entry.isNull() && d->fieldKey == QLatin1String("^external")) {
+        if (file != NULL && !entry.isNull() && d->fieldKey == QLatin1String("^external")) {
             /// handle "external" list differently
-            QList<KUrl> urlList = FileInfo::entryUrls(entry.data(), KUrl(file->property(File::Url).toUrl()), FileInfo::TestExistanceNo);
+            QList<KUrl> urlList = FileInfo::entryUrls(entry.data(), KUrl(file->property(File::Url).toUrl()), FileInfo::TestExistenceNo);
             Value v;
             foreach(const KUrl &url, urlList) {
                 v.append(QSharedPointer<VerbatimText>(new VerbatimText(url.pathOrUrl())));
@@ -366,7 +367,6 @@ void FieldListEdit::lineAdd()
     QSize size(d->container->width(), d->recommendedHeight());
     d->container->resize(size);
     newEdit->setFocus(Qt::ShortcutFocusReason);
-    emit modified();
 }
 
 void FieldListEdit::lineRemove(QWidget *widget)
@@ -375,19 +375,21 @@ void FieldListEdit::lineRemove(QWidget *widget)
     d->removeFieldLineEdit(fieldLineEdit);
     QSize size(d->container->width(), d->recommendedHeight());
     d->container->resize(size);
+    emit modified();
 }
 
 void FieldListEdit::lineGoDown(QWidget *widget)
 {
     FieldLineEdit *fieldLineEdit = static_cast<FieldLineEdit *>(widget);
     d->goDownFieldLineEdit(fieldLineEdit);
+    emit modified();
 }
 
 void FieldListEdit::lineGoUp(QWidget *widget)
 {
     FieldLineEdit *fieldLineEdit = static_cast<FieldLineEdit *>(widget);
     d->goUpFieldLineEdit(fieldLineEdit);
-
+    emit modified();
 }
 
 PersonListEdit::PersonListEdit(KBibTeX::TypeFlag preferredTypeFlag, KBibTeX::TypeFlags typeFlags, QWidget *parent)
@@ -444,7 +446,7 @@ void PersonListEdit::slotAddNamesFromClipboard()
     if (text.isEmpty())
         text = clipboard->text(QClipboard::Selection);
     if (!text.isEmpty()) {
-        QList<QSharedPointer<Person> > personList = FileImporterBibTeX::splitNames(text);
+        const QList<QSharedPointer<Person> > personList = FileImporterBibTeX::splitNames(text);
         foreach(QSharedPointer<Person> person, personList) {
             Value *value = new Value();
             value->append(person);
@@ -460,20 +462,18 @@ UrlListEdit::UrlListEdit(QWidget *parent)
         : FieldListEdit(KBibTeX::tfVerbatim, KBibTeX::tfVerbatim, parent)
 {
     m_signalMapperSaveLocallyButtonClicked = new QSignalMapper(this);
-    connect(m_signalMapperSaveLocallyButtonClicked, SIGNAL(mapped(QWidget *)), this, SLOT(slotSaveLocally(QWidget *)));
+    connect(m_signalMapperSaveLocallyButtonClicked, SIGNAL(mapped(QWidget*)), this, SLOT(slotSaveLocally(QWidget*)));
     m_signalMapperFieldLineEditTextChanged = new QSignalMapper(this);
-    connect(m_signalMapperFieldLineEditTextChanged, SIGNAL(mapped(QWidget *)), this, SLOT(textChanged(QWidget *)));
+    connect(m_signalMapperFieldLineEditTextChanged, SIGNAL(mapped(QWidget*)), this, SLOT(textChanged(QWidget*)));
 
-    /// Button to add a reference (i.e. only the filename or URL) to an entry
-    m_addReferenceToFile = new KPushButton(KIcon("emblem-symbolic-link"), i18n("Add reference to file ..."), this);
-    addButton(m_addReferenceToFile);
-    connect(m_addReferenceToFile, SIGNAL(clicked()), this, SLOT(slotAddReferenceToFile()));
+    m_buttonAddFile = new KPushButton(KIcon("list-add"), i18n("Add file..."), this);
+    addButton(m_buttonAddFile);
+    KMenu *menuAddFile = new KMenu(m_buttonAddFile);
+    m_buttonAddFile->setMenu(menuAddFile);
+    connect(m_buttonAddFile, SIGNAL(clicked()), m_buttonAddFile, SLOT(showMenu()));
 
-    /// Button to copy a file near the BibTeX file (e.g. same folder) and then
-    /// add the copy's relative filename to the entry
-    m_copyFile = new KPushButton(KIcon("document-save-all"), i18n("Insert file ..."), this);
-    addButton(m_copyFile);
-    connect(m_copyFile, SIGNAL(clicked()), this, SLOT(slotCopyFile()));
+    menuAddFile->addAction(KIcon("emblem-symbolic-link"), i18n("Add reference ..."), this, SLOT(slotAddReference()));
+    menuAddFile->addAction(KIcon("emblem-symbolic-link"), i18n("Add reference from clipboard"), this, SLOT(slotAddReferenceFromClipboard()));
 }
 
 UrlListEdit::~UrlListEdit()
@@ -482,14 +482,30 @@ UrlListEdit::~UrlListEdit()
     delete m_signalMapperFieldLineEditTextChanged;
 }
 
-void UrlListEdit::slotAddReferenceToFile()
+void UrlListEdit::slotAddReference()
 {
     QUrl bibtexUrl(d->file != NULL ? d->file->property(File::Url, QVariant()).toUrl() : QUrl());
-    QFileInfo bibtexUrlInfo = bibtexUrl.isEmpty() ? QFileInfo() : QFileInfo(bibtexUrl.path());
+    if (!bibtexUrl.isEmpty()) {
+        const QFileInfo fi(bibtexUrl.path());
+        bibtexUrl.setPath(fi.absolutePath());
+    }
+    const KUrl documentUrl = KFileDialog::getOpenUrl(bibtexUrl, QString(), this, i18n("File to Associate"));
+    if (!documentUrl.isEmpty())
+        addReference(documentUrl);
+}
 
-    const QString filename = KFileDialog::getOpenFileName(KUrl(bibtexUrlInfo.absolutePath()), QString(), this, i18n("Select File"));
-    if (!filename.isEmpty()) {
-        const QString visibleFilename = askRelativeOrStaticFilename(this, filename, bibtexUrl);
+void UrlListEdit::slotAddReferenceFromClipboard()
+{
+    const KUrl url = KUrl::fromUserInput(QApplication::clipboard()->text());
+    if (!url.isEmpty())
+        addReference(url);
+}
+
+void UrlListEdit::addReference(const QUrl &url) {
+    const Entry *entry = dynamic_cast<const Entry *>(m_element);
+    const QString entryId = entry != NULL ? entry->id() : QString();
+    const QString visibleFilename = AssociatedFilesUI::associateUrl(url, entryId, d->file, this);
+    if (!visibleFilename.isEmpty()) {
         Value *value = new Value();
         value->append(QSharedPointer<VerbatimText>(new VerbatimText(visibleFilename)));
         lineAdd(value);
@@ -497,59 +513,10 @@ void UrlListEdit::slotAddReferenceToFile()
     }
 }
 
-void UrlListEdit::slotCopyFile()
-{
-    QUrl bibtexUrl(d->file != NULL ? d->file->property(File::Url, QVariant()).toUrl() : QUrl());
-    if (!bibtexUrl.isValid() || bibtexUrl.isEmpty()) {
-        KMessageBox::information(this, i18n("The current BibTeX document has to be saved first before associating other files with this document."));
-        return;
-    }
-    /// Gather information on BibTeX file's URL
-    QFileInfo bibtexUrlInfo = QFileInfo(bibtexUrl.path());
-
-    /// Ask user which file to associate with the BibTeX document
-    // TODO allow a similar operation by drag-and-drop of e.g. PDF files
-    const QString sourceFilename = KFileDialog::getOpenFileName(KUrl(), QString(), this, i18n("Select Source File"));
-    if (!sourceFilename.isEmpty()) {
-        /// Build a proposal for the new filename relative to the BibTeX document
-        // TODO make this more configurable, e.g. via templates
-        const Entry *entry = dynamic_cast<const Entry *>(m_element);
-        const QString entryKey = entry != NULL ? entry->id() : QLatin1String("");
-        int p = sourceFilename.lastIndexOf(QLatin1Char('.'));
-        const QString extension = p > 0 && !entryKey.isEmpty() ? sourceFilename.mid(p) : QString();
-        QString destinationFilename = bibtexUrlInfo.absolutePath() + QDir::separator() + entryKey + extension;
-
-        /// Ask user for confirmation on the new filename
-        destinationFilename = KFileDialog::getSaveFileName(KUrl(destinationFilename), QString(), this, i18n("Select Destination"));
-        if (!destinationFilename.isEmpty()) {
-            /// Memorize the absolute filename for the copy operation further below
-            const QString absoluteDestinationFilename = destinationFilename;
-
-            /// Determine the relative filename (relative to the BibTeX file)
-            QFileInfo destinationFilenameInfo(destinationFilename);
-            if (destinationFilenameInfo.absolutePath() == bibtexUrlInfo.absolutePath() || destinationFilenameInfo.absolutePath().startsWith(bibtexUrlInfo.absolutePath() + QDir::separator())) {
-                QString relativePath = destinationFilenameInfo.absolutePath().mid(bibtexUrlInfo.absolutePath().length() + 1);
-                destinationFilename = relativePath + (relativePath.isEmpty() ? QLatin1String("") : QString(QDir::separator())) + destinationFilenameInfo.fileName();
-            }
-
-            /// If file with same destination filename exists, delete it
-            KIO::NetAccess::del(KUrl(absoluteDestinationFilename), this);
-            /// Copy associated document to destination using the absolute filename
-            if (KIO::NetAccess::file_copy(KUrl(sourceFilename), KUrl(absoluteDestinationFilename), this)) {
-                /// Only if copy operation succeeded, add reference to copied file into BibTeX entry
-                Value *value = new Value();
-                value->append(QSharedPointer<VerbatimText>(new VerbatimText(destinationFilename)));
-                lineAdd(value);
-                emit modified();
-            }
-        }
-    }
-}
-
 void UrlListEdit::slotSaveLocally(QWidget *widget)
 {
     /// Determine FieldLineEdit widget
-    FieldLineEdit *fieldLineEdit = dynamic_cast<FieldLineEdit *>(widget);
+    FieldLineEdit *fieldLineEdit = qobject_cast<FieldLineEdit *>(widget);
     /// Build Url from line edit's content
     const KUrl url(fieldLineEdit->text());
 
@@ -581,7 +548,6 @@ void UrlListEdit::slotSaveLocally(QWidget *widget)
                 Value *value = new Value();
                 value->append(QSharedPointer<VerbatimText>(new VerbatimText(visibleFilename)));
                 lineAdd(value);
-                emit modified();
             }
             setEnabled(true);
             unsetCursor();
@@ -592,11 +558,11 @@ void UrlListEdit::slotSaveLocally(QWidget *widget)
 void UrlListEdit::textChanged(QWidget *widget)
 {
     /// Determine associated KPushButton "Save locally"
-    KPushButton *buttonSaveLocally = dynamic_cast<KPushButton *>(widget);
+    KPushButton *buttonSaveLocally = qobject_cast<KPushButton *>(widget);
     if (buttonSaveLocally == NULL) return; ///< should never happen!
 
     /// Assume a FieldLineEdit was the sender of this signal
-    FieldLineEdit *fieldLineEdit = dynamic_cast<FieldLineEdit *>(m_signalMapperFieldLineEditTextChanged->mapping(widget));
+    FieldLineEdit *fieldLineEdit = qobject_cast<FieldLineEdit *>(m_signalMapperFieldLineEditTextChanged->mapping(widget));
     if (fieldLineEdit == NULL) return; ///< should never happen!
 
     /// Create URL from new text to make some tests on it
@@ -609,7 +575,7 @@ void UrlListEdit::textChanged(QWidget *widget)
     /// Enable button only if Url is valid and points to a remote
     /// DjVu, PDF, or PostScript file
     // TODO more file types?
-    bool canBeSaved = url.isValid() && !urlIsLocal(url) && (newText.endsWith(QLatin1String(".djvu")) || newText.endsWith(QLatin1String(".pdf")) || newText.endsWith(QLatin1String(".ps"))) && !urlIsLocal(url);
+    bool canBeSaved = url.isValid() && !urlIsLocal(url) && (newText.endsWith(QLatin1String(".djvu")) || newText.endsWith(QLatin1String(".pdf")) || newText.endsWith(QLatin1String(".ps")));
     buttonSaveLocally->setEnabled(canBeSaved);
     buttonSaveLocally->setToolTip(canBeSaved ? i18n("Save file '%1' locally", url.pathOrUrl()) : QLatin1String(""));
 }
@@ -630,9 +596,10 @@ QString UrlListEdit::askRelativeOrStaticFilename(QWidget *parent, const QString
 
 bool UrlListEdit::urlIsLocal(const QUrl &url)
 {
+    // FIXME same function as in AssociateFiles; move to common code base?
     const QString scheme = url.scheme();
     /// Test various schemes such as "http", "https", "ftp", ...
-    return !scheme.startsWith(QLatin1String("http")) && !scheme.startsWith(QLatin1String("ftp")) && !scheme.startsWith(QLatin1String("webdav")) && scheme != QLatin1String("smb");
+    return !scheme.startsWith(QLatin1String("http")) && !scheme.startsWith(QLatin1String("ftp")) && !scheme.startsWith(QLatin1String("sftp")) && !scheme.startsWith(QLatin1String("fish")) && !scheme.startsWith(QLatin1String("webdav")) && scheme != QLatin1String("smb");
 }
 
 FieldLineEdit *UrlListEdit::addFieldLineEdit()
@@ -659,8 +626,7 @@ FieldLineEdit *UrlListEdit::addFieldLineEdit()
 void UrlListEdit::setReadOnly(bool isReadOnly)
 {
     FieldListEdit::setReadOnly(isReadOnly);
-    m_addReferenceToFile->setEnabled(!isReadOnly);
-    m_copyFile->setEnabled(!isReadOnly);
+    m_buttonAddFile->setEnabled(!isReadOnly);
 }
 
 
@@ -714,10 +680,10 @@ void KeywordListEdit::slotAddKeywordsFromClipboard()
 {
     QClipboard *clipboard = QApplication::clipboard();
     QString text = clipboard->text(QClipboard::Clipboard);
-    if (text.isEmpty())
+    if (text.isEmpty()) ///< use "mouse" clipboard as fallback
         text = clipboard->text(QClipboard::Selection);
     if (!text.isEmpty()) {
-        QList<QSharedPointer<Keyword> > keywords = FileImporterBibTeX::splitKeywords(text);
+        const QList<QSharedPointer<Keyword> > keywords = FileImporterBibTeX::splitKeywords(text);
         for (QList<QSharedPointer<Keyword> >::ConstIterator it = keywords.constBegin(); it != keywords.constEnd(); ++it) {
             Value *value = new Value();
             value->append(*it);
