@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "settingsfileexporterwidget.h"
 
 #include <QFormLayout>
 #include <QCheckBox>
@@ -31,11 +30,12 @@
 #include <KFileDialog>
 #include <KUrlRequester>
 
-#include <fileexporter.h>
-#include <clipboard.h>
+#include "guihelper.h"
+#include "italictextitemmodel.h"
+#include "fileexporter.h"
+#include "clipboard.h"
 #include "preferences.h"
 #include "lyx.h"
-#include "settingsfileexporterwidget.h"
 
 class SettingsFileExporterWidget::SettingsFileExporterWidgetPrivate
 {
@@ -63,7 +63,8 @@ public:
     void loadState() {
         KConfigGroup configGroup(config, Preferences::groupGeneral);
         const QString copyReferenceCommand = configGroup.readEntry(Clipboard::keyCopyReferenceCommand, Clipboard::defaultCopyReferenceCommand);
-        p->selectValue(comboBoxCopyReferenceCmd, copyReferenceCommand.isEmpty() ? QString("") : copyReferenceCommand, Qt::UserRole);
+        int row = GUIHelper::selectValue(comboBoxCopyReferenceCmd->model(), copyReferenceCommand.isEmpty() ? QString("") : copyReferenceCommand, ItalicTextItemModel::IdentifierRole);
+        comboBoxCopyReferenceCmd->setCurrentIndex(row);
 
         const int index = qMax(0, comboBoxBackupScope->findData(configGroup.readEntry(Preferences::keyBackupScope, (int)Preferences::defaultBackupScope)));
         comboBoxBackupScope->setCurrentIndex(index);
@@ -77,7 +78,7 @@ public:
 
     void saveState() {
         KConfigGroup configGroup(config, Preferences::groupGeneral);
-        const QString copyReferenceCommand = comboBoxCopyReferenceCmd->itemData(comboBoxCopyReferenceCmd->currentIndex()).toString();
+        const QString copyReferenceCommand = comboBoxCopyReferenceCmd->itemData(comboBoxCopyReferenceCmd->currentIndex(), ItalicTextItemModel::IdentifierRole).toString();
         configGroup.writeEntry(Clipboard::keyCopyReferenceCommand, copyReferenceCommand);
 
         configGroup.writeEntry(Preferences::keyBackupScope, comboBoxBackupScope->itemData(comboBoxBackupScope->currentIndex()).toInt());
@@ -91,7 +92,8 @@ public:
     }
 
     void resetToDefaults() {
-        p->selectValue(comboBoxCopyReferenceCmd, QString(""), Qt::UserRole);
+        int row = GUIHelper::selectValue(comboBoxCopyReferenceCmd->model(), QString(""), Qt::UserRole);
+        comboBoxCopyReferenceCmd->setCurrentIndex(row);
 
         const int index = qMax(0, comboBoxBackupScope->findData(Preferences::defaultBackupScope));
         comboBoxBackupScope->setCurrentIndex(index);
@@ -108,7 +110,7 @@ public:
 
         comboBoxCopyReferenceCmd = new KComboBox(false, p);
         comboBoxCopyReferenceCmd->setObjectName("comboBoxCopyReferenceCmd");
-        layout->addRow(i18n("Command for \"Copy Reference\":"), comboBoxCopyReferenceCmd);
+        layout->addRow(i18n("Command for 'Copy Reference':"), comboBoxCopyReferenceCmd);
         ItalicTextItemModel *itim = new ItalicTextItemModel();
         itim->addItem(i18n("No command"), QString(""));
         const QStringList citeCommands = QStringList() << QLatin1String("cite") << QLatin1String("citealt") << QLatin1String("citeauthor") << QLatin1String("citeauthor*") << QLatin1String("citeyear") << QLatin1String("citeyearpar") << QLatin1String("shortcite") << QLatin1String("citet") << QLatin1String("citet*") << QLatin1String("citep") << QLatin1String("citep*"); // TODO more
