@@ -1,32 +1,31 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "settingsidsuggestionseditor.h"
 
 #include <QGridLayout>
 #include <QFormLayout>
 #include <QScrollArea>
 #include <QLabel>
-#include <QGroupBox>
 #include <QSpinBox>
 #include <QCheckBox>
 #include <QSignalMapper>
 #include <QMenu>
+#include <QtCore/QPointer>
 
 #include <KLineEdit>
 #include <KComboBox>
@@ -34,106 +33,124 @@
 #include <KPushButton>
 #include <KAction>
 
-#include "settingsidsuggestionseditor.h"
+#include <qxtspanslider.h>
 
-class TokenWidget : public QGroupBox
+
+TokenWidget::TokenWidget(QWidget *parent)
+        : QGroupBox(parent)
 {
-protected:
-    QGridLayout *gridLayout;
-    QFormLayout *formLayout;
+    gridLayout = new QGridLayout(this);
+    formLayout = new QFormLayout();
+    gridLayout->addLayout(formLayout, 0, 0, 4, 1);
+    gridLayout->setColumnStretch(0, 100);
+}
 
-public:
-    TokenWidget(QWidget *parent)
-            : QGroupBox(parent) {
-        gridLayout = new QGridLayout(this);
-        formLayout = new QFormLayout();
-        gridLayout->addLayout(formLayout, 0, 0, 4, 1);
-        gridLayout->setColumnStretch(0, 100);
-    }
-
-    void addButtons(KPushButton *buttonUp, KPushButton *buttonDown, KPushButton *buttonRemove) {
-        gridLayout->setColumnMinimumWidth(1, 32);
-        gridLayout->setColumnStretch(1, 1);
-        gridLayout->setColumnStretch(2, 1);
-
-        gridLayout->addWidget(buttonUp, 0, 2, 1, 1);
-        buttonUp->setParent(this);
-        gridLayout->addWidget(buttonDown, 1, 2, 1, 1);
-        buttonDown->setParent(this);
-        gridLayout->addWidget(buttonRemove, 2, 2, 1, 1);
-        buttonRemove->setParent(this);
-    }
-
-    virtual QString toString() const = 0;
-};
-
-class AuthorWidget : public TokenWidget
+void TokenWidget::addButtons(KPushButton *buttonUp, KPushButton *buttonDown, KPushButton *buttonRemove)
 {
-private:
-    KComboBox *comboBoxWhichAuthor;
-    KComboBox *comboBoxChangeCase;
-    KLineEdit *lineEditTextInBetween;
-    QSpinBox *spinBoxLength;
+    gridLayout->setColumnMinimumWidth(1, 32);
+    gridLayout->setColumnStretch(1, 1);
+    gridLayout->setColumnStretch(2, 1);
 
-public:
-    AuthorWidget(const struct IdSuggestions::IdSuggestionTokenInfo &info, IdSuggestions::Authors author, IdSuggestionsEditWidget *isew, QWidget *parent)
-            : TokenWidget(parent) {
-        setTitle(i18n("Authors"));
+    gridLayout->addWidget(buttonUp, 0, 2, 1, 1);
+    buttonUp->setParent(this);
+    gridLayout->addWidget(buttonDown, 1, 2, 1, 1);
+    buttonDown->setParent(this);
+    gridLayout->addWidget(buttonRemove, 2, 2, 1, 1);
+    buttonRemove->setParent(this);
+}
 
-        comboBoxWhichAuthor = new KComboBox(false, this);
-        comboBoxWhichAuthor->addItem(i18n("All authors"), IdSuggestions::aAll);
-        comboBoxWhichAuthor->addItem(i18n("Only first author"), IdSuggestions::aOnlyFirst);
-        comboBoxWhichAuthor->addItem(i18n("All except first author"), IdSuggestions::aNotFirst);
-        formLayout->addRow(i18n("Authors:"), comboBoxWhichAuthor);
-        comboBoxWhichAuthor->setCurrentIndex(comboBoxWhichAuthor->findData(author));
 
-        comboBoxChangeCase = new KComboBox(false, this);
-        comboBoxChangeCase->addItem(i18n("No change"), IdSuggestions::ccNoChange);
-        comboBoxChangeCase->addItem(i18n("To upper case"), IdSuggestions::ccToUpper);
-        comboBoxChangeCase->addItem(i18n("To lower case"), IdSuggestions::ccToLower);
-        comboBoxChangeCase->addItem(i18n("To CamelCase"), IdSuggestions::ccToCamelCase);
-        formLayout->addRow(i18n("Change casing:"), comboBoxChangeCase);
-        comboBoxChangeCase->setCurrentIndex((int)info.caseChange); /// enum has numbers assigned to cases and combo box has same indices
+AuthorWidget::AuthorWidget(const struct IdSuggestions::IdSuggestionTokenInfo &info, IdSuggestionsEditWidget *isew, QWidget *parent)
+        : TokenWidget(parent)
+{
+    setTitle(i18n("Authors"));
 
-        lineEditTextInBetween = new KLineEdit(this);
-        formLayout->addRow(i18n("Text in between:"), lineEditTextInBetween);
-        lineEditTextInBetween->setText(info.inBetween);
+    QBoxLayout *boxLayout = new QVBoxLayout();
+    boxLayout->setMargin(0);
+    formLayout->addRow(i18n("Author Range:"), boxLayout);
+    spanSliderAuthor = new QxtSpanSlider(Qt::Horizontal, this);
+    boxLayout->addWidget(spanSliderAuthor);
+    spanSliderAuthor->setRange(0, 9);
+    spanSliderAuthor->setHandleMovementMode(QxtSpanSlider::NoCrossing);
+    spanSliderAuthor->setLowerValue(info.startWord);
+    spanSliderAuthor->setUpperValue(qMin(spanSliderAuthor->maximum(), info.endWord));
 
-        spinBoxLength = new QSpinBox(this);
-        formLayout->addRow(i18n("Only first characters:"), spinBoxLength);
-        spinBoxLength->setSpecialValueText(i18n("No limitation"));
-        spinBoxLength->setMinimum(0);
-        spinBoxLength->setMaximum(9);
-        spinBoxLength->setValue(info.len == 0 || info.len > 9 ? 0 : info.len);
+    checkBoxLastAuthor = new QCheckBox(i18n("... and last author"), this);
+    boxLayout->addWidget(checkBoxLastAuthor);
 
-        connect(comboBoxWhichAuthor, SIGNAL(currentIndexChanged(int)), isew, SLOT(updatePreview()));
-        connect(comboBoxChangeCase, SIGNAL(currentIndexChanged(int)), isew, SLOT(updatePreview()));
-        connect(lineEditTextInBetween, SIGNAL(textEdited(QString)), isew, SLOT(updatePreview()));
-        connect(spinBoxLength, SIGNAL(valueChanged(int)), isew, SLOT(updatePreview()));
-    }
+    labelAuthorRange = new QLabel(this);
+    boxLayout->addWidget(labelAuthorRange);
+    const int maxWidth = qMax(labelAuthorRange->fontMetrics().width(i18n("From first author to author %1 and last author", 88)), labelAuthorRange->fontMetrics().width(i18n("From author %1 to author %2 and last author", 88, 88)));
+    labelAuthorRange->setMinimumWidth(maxWidth);
 
-    QString toString() const {
-        IdSuggestions::Authors author = (IdSuggestions::Authors)comboBoxWhichAuthor->itemData(comboBoxWhichAuthor->currentIndex()).toInt();
-        QString result = author == IdSuggestions::aAll ? QLatin1String("A") : (author == IdSuggestions::aOnlyFirst ? QLatin1String("a") : QLatin1String("z"));
+    comboBoxChangeCase = new KComboBox(false, this);
+    comboBoxChangeCase->addItem(i18n("No change"), IdSuggestions::ccNoChange);
+    comboBoxChangeCase->addItem(i18n("To upper case"), IdSuggestions::ccToUpper);
+    comboBoxChangeCase->addItem(i18n("To lower case"), IdSuggestions::ccToLower);
+    comboBoxChangeCase->addItem(i18n("To CamelCase"), IdSuggestions::ccToCamelCase);
+    formLayout->addRow(i18n("Change casing:"), comboBoxChangeCase);
+    comboBoxChangeCase->setCurrentIndex((int)info.caseChange); /// enum has numbers assigned to cases and combo box has same indices
 
-        if (spinBoxLength->value() > 0)
-            result.append(QString::number(spinBoxLength->value()));
+    lineEditTextInBetween = new KLineEdit(this);
+    formLayout->addRow(i18n("Text in between:"), lineEditTextInBetween);
+    lineEditTextInBetween->setText(info.inBetween);
 
-        IdSuggestions::CaseChange caseChange = (IdSuggestions::CaseChange)comboBoxChangeCase->itemData(comboBoxChangeCase->currentIndex()).toInt();
-        if (caseChange == IdSuggestions::ccToLower)
-            result.append(QLatin1String("l"));
-        else if (caseChange == IdSuggestions::ccToUpper)
-            result.append(QLatin1String("u"));
-        else if (caseChange == IdSuggestions::ccToCamelCase)
-            result.append(QLatin1String("c"));
+    spinBoxLength = new QSpinBox(this);
+    formLayout->addRow(i18n("Only first characters:"), spinBoxLength);
+    spinBoxLength->setSpecialValueText(i18n("No limitation"));
+    spinBoxLength->setMinimum(0);
+    spinBoxLength->setMaximum(9);
+    spinBoxLength->setValue(info.len == 0 || info.len > 9 ? 0 : info.len);
 
-        const QString text = lineEditTextInBetween->text();
-        if (!text.isEmpty())
-            result.append(QLatin1String("\"")).append(text);
+    connect(spanSliderAuthor, SIGNAL(lowerValueChanged(int)), isew, SLOT(updatePreview()));
+    connect(spanSliderAuthor, SIGNAL(upperValueChanged(int)), isew, SLOT(updatePreview()));
+    connect(spanSliderAuthor, SIGNAL(lowerValueChanged(int)), this, SLOT(updateRangeLabel()));
+    connect(spanSliderAuthor, SIGNAL(upperValueChanged(int)), this, SLOT(updateRangeLabel()));
+    connect(checkBoxLastAuthor, SIGNAL(toggled(bool)), isew, SLOT(updatePreview()));
+    connect(checkBoxLastAuthor, SIGNAL(toggled(bool)), this, SLOT(updateRangeLabel()));
+    connect(comboBoxChangeCase, SIGNAL(currentIndexChanged(int)), isew, SLOT(updatePreview()));
+    connect(lineEditTextInBetween, SIGNAL(textEdited(QString)), isew, SLOT(updatePreview()));
+    connect(spinBoxLength, SIGNAL(valueChanged(int)), isew, SLOT(updatePreview()));
 
-        return result;
-    }
-};
+    updateRangeLabel();
+}
+
+QString AuthorWidget::toString() const
+{
+    QString result = QLatin1String("A");
+
+    if (spinBoxLength->value() > 0)
+        result.append(QString::number(spinBoxLength->value()));
+
+    IdSuggestions::CaseChange caseChange = (IdSuggestions::CaseChange)comboBoxChangeCase->currentIndex();
+    if (caseChange == IdSuggestions::ccToLower)
+        result.append(QLatin1String("l"));
+    else if (caseChange == IdSuggestions::ccToUpper)
+        result.append(QLatin1String("u"));
+    else if (caseChange == IdSuggestions::ccToCamelCase)
+        result.append(QLatin1String("c"));
+
+    if (spanSliderAuthor->lowerValue() > spanSliderAuthor->minimum() || spanSliderAuthor->upperValue() < spanSliderAuthor->maximum())
+        result.append(QString(QLatin1String("w%1%2")).arg(spanSliderAuthor->lowerValue()).arg(spanSliderAuthor->upperValue() < spanSliderAuthor->maximum() ? QString::number(spanSliderAuthor->upperValue()) : QLatin1String("I")));
+    if (checkBoxLastAuthor->isChecked())
+        result.append(QLatin1String("L"));
+
+    const QString text = lineEditTextInBetween->text();
+    if (!text.isEmpty())
+        result.append(QLatin1String("\"")).append(text);
+
+    return result;
+}
+
+void AuthorWidget::updateRangeLabel()
+{
+    const int lower = spanSliderAuthor->lowerValue();
+    const int upper = spanSliderAuthor->upperValue();
+    const int min = spanSliderAuthor->minimum();
+    const int max = spanSliderAuthor->maximum();
+
+    labelAuthorRange->setText(IdSuggestions::formatAuthorRange(lower == min ? 0 : lower, upper == max ? 0x00ffffff : upper, checkBoxLastAuthor->isChecked()));
+}
 
 class YearWidget : public TokenWidget
 {
@@ -162,22 +179,130 @@ public:
     }
 };
 
-class TitleWidget : public TokenWidget
+TitleWidget::TitleWidget(const struct IdSuggestions::IdSuggestionTokenInfo &info, bool removeSmallWords, IdSuggestionsEditWidget *isew, QWidget *parent)
+        : TokenWidget(parent)
+{
+    setTitle(i18n("Title"));
+
+    QBoxLayout *boxLayout = new QVBoxLayout();
+    boxLayout->setMargin(0);
+    formLayout->addRow(i18n("Word Range:"), boxLayout);
+    spanSliderWords = new QxtSpanSlider(Qt::Horizontal, this);
+    boxLayout->addWidget(spanSliderWords);
+    spanSliderWords->setRange(0, 9);
+    spanSliderWords->setHandleMovementMode(QxtSpanSlider::NoCrossing);
+    if (info.startWord > 0 || info.endWord < 0xffff) {
+        spanSliderWords->setLowerValue(info.startWord);
+        spanSliderWords->setUpperValue(qMin(spanSliderWords->maximum(), info.endWord));
+    } else {
+        spanSliderWords->setLowerValue(0);
+        spanSliderWords->setUpperValue(spanSliderWords->maximum());
+    }
+
+    labelWordsRange = new QLabel(this);
+    boxLayout->addWidget(labelWordsRange);
+    const int a = qMax(labelWordsRange->fontMetrics().width(i18n("From first to last word")), labelWordsRange->fontMetrics().width(i18n("From word %1 to last word", 88)));
+    const int b = qMax(labelWordsRange->fontMetrics().width(i18n("From first word to word %1", 88)), labelWordsRange->fontMetrics().width(i18n("From word %1 to word %2", 88, 88)));
+    labelWordsRange->setMinimumWidth(qMax(a, b));
+
+    checkBoxRemoveSmallWords = new QCheckBox(i18n("Remove"), this);
+    formLayout->addRow(i18n("Small words:"), checkBoxRemoveSmallWords);
+    checkBoxRemoveSmallWords->setChecked(removeSmallWords);
+
+    comboBoxChangeCase = new KComboBox(false, this);
+    comboBoxChangeCase->addItem(i18n("No change"), IdSuggestions::ccNoChange);
+    comboBoxChangeCase->addItem(i18n("To upper case"), IdSuggestions::ccToUpper);
+    comboBoxChangeCase->addItem(i18n("To lower case"), IdSuggestions::ccToLower);
+    comboBoxChangeCase->addItem(i18n("To CamelCase"), IdSuggestions::ccToCamelCase);
+    formLayout->addRow(i18n("Change casing:"), comboBoxChangeCase);
+    comboBoxChangeCase->setCurrentIndex((int)info.caseChange); /// enum has numbers assigned to cases and combo box has same indices
+
+    lineEditTextInBetween = new KLineEdit(this);
+    formLayout->addRow(i18n("Text in between:"), lineEditTextInBetween);
+    lineEditTextInBetween->setText(info.inBetween);
+
+    spinBoxLength = new QSpinBox(this);
+    formLayout->addRow(i18n("Only first characters:"), spinBoxLength);
+    spinBoxLength->setSpecialValueText(i18n("No limitation"));
+    spinBoxLength->setMinimum(0);
+    spinBoxLength->setMaximum(9);
+    spinBoxLength->setValue(info.len == 0 || info.len > 9 ? 0 : info.len);
+
+    connect(spanSliderWords, SIGNAL(lowerValueChanged(int)), isew, SLOT(updatePreview()));
+    connect(spanSliderWords, SIGNAL(upperValueChanged(int)), isew, SLOT(updatePreview()));
+    connect(spanSliderWords, SIGNAL(lowerValueChanged(int)), this, SLOT(updateRangeLabel()));
+    connect(spanSliderWords, SIGNAL(upperValueChanged(int)), this, SLOT(updateRangeLabel()));
+    connect(checkBoxRemoveSmallWords, SIGNAL(toggled(bool)), isew, SLOT(updatePreview()));
+    connect(comboBoxChangeCase, SIGNAL(currentIndexChanged(int)), isew, SLOT(updatePreview()));
+    connect(lineEditTextInBetween, SIGNAL(textEdited(QString)), isew, SLOT(updatePreview()));
+    connect(spinBoxLength, SIGNAL(valueChanged(int)), isew, SLOT(updatePreview()));
+
+    updateRangeLabel();
+}
+
+QString TitleWidget::toString() const
+{
+    QString result = checkBoxRemoveSmallWords->isChecked() ? QLatin1String("T") : QLatin1String("t");
+
+    if (spinBoxLength->value() > 0)
+        result.append(QString::number(spinBoxLength->value()));
+
+    IdSuggestions::CaseChange caseChange = (IdSuggestions::CaseChange)comboBoxChangeCase->currentIndex();
+    if (caseChange == IdSuggestions::ccToLower)
+        result.append(QLatin1String("l"));
+    else if (caseChange == IdSuggestions::ccToUpper)
+        result.append(QLatin1String("u"));
+    else if (caseChange == IdSuggestions::ccToCamelCase)
+        result.append(QLatin1String("c"));
+
+    if (spanSliderWords->lowerValue() > spanSliderWords->minimum() || spanSliderWords->upperValue() < spanSliderWords->maximum())
+        result.append(QString(QLatin1String("w%1%2")).arg(spanSliderWords->lowerValue()).arg(spanSliderWords->upperValue() < spanSliderWords->maximum() ? QString::number(spanSliderWords->upperValue()) : QLatin1String("I")));
+
+    const QString text = lineEditTextInBetween->text();
+    if (!text.isEmpty())
+        result.append(QLatin1String("\"")).append(text);
+
+    return result;
+}
+
+void TitleWidget::updateRangeLabel()
+{
+    const int lower = spanSliderWords->lowerValue();
+    const int upper = spanSliderWords->upperValue();
+    const int min = spanSliderWords->minimum();
+    const int max = spanSliderWords->maximum();
+
+    if (lower == min && upper == min)
+        labelWordsRange->setText(i18n("First word only"));
+    else if (lower == min + 1 && upper == max)
+        labelWordsRange->setText(i18n("All but first word"));
+    else if (lower == min && upper == max)
+        labelWordsRange->setText(i18n("From first to last word"));
+    else if (lower > min && upper == max)
+        labelWordsRange->setText(i18n("From word %1 to last word", lower + 1));
+    else if (lower == min && upper < max)
+        labelWordsRange->setText(i18n("From first word to word %1", upper + 1));
+    else
+        labelWordsRange->setText(i18n("From word %1 to word %2", lower + 1, upper + 1));
+}
+
+/**
+ * @author Thomas Fischer
+ */
+class JournalWidget : public TokenWidget
 {
 private:
-    QCheckBox *checkBoxRemoveSmallWords;
     KComboBox *comboBoxChangeCase;
-    KLineEdit *lineEditTextInBetween;
     QSpinBox *spinBoxLength;
 
 public:
-    TitleWidget(const struct IdSuggestions::IdSuggestionTokenInfo &info, bool removeSmallWords, IdSuggestionsEditWidget *isew, QWidget *parent)
-            : TokenWidget(parent) {
-        setTitle(i18n("Title"));
+    JournalWidget(const struct IdSuggestions::IdSuggestionTokenInfo &info, IdSuggestionsEditWidget *isew, QWidget *parent)
+            : TokenWidget(parent)
+    {
+        setTitle(i18n("Journal"));
 
-        checkBoxRemoveSmallWords = new QCheckBox(i18n("Remove"), this);
-        formLayout->addRow(i18n("Small words:"), checkBoxRemoveSmallWords);
-        checkBoxRemoveSmallWords->setChecked(removeSmallWords);
+        QBoxLayout *boxLayout = new QVBoxLayout();
+        boxLayout->setMargin(0);
 
         comboBoxChangeCase = new KComboBox(false, this);
         comboBoxChangeCase->addItem(i18n("No change"), IdSuggestions::ccNoChange);
@@ -187,10 +312,6 @@ public:
         formLayout->addRow(i18n("Change casing:"), comboBoxChangeCase);
         comboBoxChangeCase->setCurrentIndex((int)info.caseChange); /// enum has numbers assigned to cases and combo box has same indices
 
-        lineEditTextInBetween = new KLineEdit(this);
-        formLayout->addRow(i18n("Text in between:"), lineEditTextInBetween);
-        lineEditTextInBetween->setText(info.inBetween);
-
         spinBoxLength = new QSpinBox(this);
         formLayout->addRow(i18n("Only first characters:"), spinBoxLength);
         spinBoxLength->setSpecialValueText(i18n("No limitation"));
@@ -198,19 +319,18 @@ public:
         spinBoxLength->setMaximum(9);
         spinBoxLength->setValue(info.len == 0 || info.len > 9 ? 0 : info.len);
 
-        connect(checkBoxRemoveSmallWords, SIGNAL(toggled(bool)), isew, SLOT(updatePreview()));
         connect(comboBoxChangeCase, SIGNAL(currentIndexChanged(int)), isew, SLOT(updatePreview()));
-        connect(lineEditTextInBetween, SIGNAL(textEdited(QString)), isew, SLOT(updatePreview()));
         connect(spinBoxLength, SIGNAL(valueChanged(int)), isew, SLOT(updatePreview()));
     }
 
-    QString toString() const {
-        QString result = checkBoxRemoveSmallWords->isChecked() ? QLatin1String("T") : QLatin1String("t");
+    QString toString() const
+    {
+        QString result = QLatin1String("j");
 
         if (spinBoxLength->value() > 0)
             result.append(QString::number(spinBoxLength->value()));
 
-        IdSuggestions::CaseChange caseChange = (IdSuggestions::CaseChange)comboBoxChangeCase->itemData(comboBoxChangeCase->currentIndex()).toInt();
+        IdSuggestions::CaseChange caseChange = (IdSuggestions::CaseChange)comboBoxChangeCase->currentIndex();
         if (caseChange == IdSuggestions::ccToLower)
             result.append(QLatin1String("l"));
         else if (caseChange == IdSuggestions::ccToUpper)
@@ -218,10 +338,6 @@ public:
         else if (caseChange == IdSuggestions::ccToCamelCase)
             result.append(QLatin1String("c"));
 
-        const QString text = lineEditTextInBetween->text();
-        if (!text.isEmpty())
-            result.append(QLatin1String("\"")).append(text);
-
         return result;
     }
 };
@@ -254,7 +370,7 @@ class IdSuggestionsEditWidget::IdSuggestionsEditWidgetPrivate
 private:
     IdSuggestionsEditWidget *p;
 public:
-    enum TokenType {ttTitle = 0, ttAuthor = 1, ttYear = 2, ttText = 3};
+    enum TokenType {ttTitle = 0, ttAuthor = 1, ttYear = 2, ttJournal = 3, ttText = 4};
 
     QWidget *container;
     QBoxLayout *containerLayout;
@@ -305,6 +421,8 @@ public:
         signalMapperAddMenu->setMapping(action, -ttAuthor);
         action = menuAddToken->addAction(i18n("Year"), signalMapperAddMenu, SLOT(map()));
         signalMapperAddMenu->setMapping(action, -ttYear);
+        action = menuAddToken->addAction(i18n("Journal"), signalMapperAddMenu, SLOT(map()));
+        signalMapperAddMenu->setMapping(action, -ttJournal);
         action = menuAddToken->addAction(i18n("Text"), signalMapperAddMenu, SLOT(map()));
         signalMapperAddMenu->setMapping(action, -ttText);
         connect(signalMapperAddMenu, SIGNAL(mapped(int)), p, SLOT(addToken(int)));
@@ -318,16 +436,18 @@ public:
         signalMapperAddMenu->setMapping(action, ttAuthor);
         action = menuAddToken->addAction(i18n("Year"), signalMapperAddMenu, SLOT(map()));
         signalMapperAddMenu->setMapping(action, ttYear);
+        action = menuAddToken->addAction(i18n("Journal"), signalMapperAddMenu, SLOT(map()));
+        signalMapperAddMenu->setMapping(action, ttJournal);
         action = menuAddToken->addAction(i18n("Text"), signalMapperAddMenu, SLOT(map()));
         signalMapperAddMenu->setMapping(action, ttText);
         connect(signalMapperAddMenu, SIGNAL(mapped(int)), p, SLOT(addToken(int)));
 
         signalMapperMoveUp = new QSignalMapper(p);
-        connect(signalMapperMoveUp, SIGNAL(mapped(QWidget *)), p, SLOT(moveUpToken(QWidget *)));
+        connect(signalMapperMoveUp, SIGNAL(mapped(QWidget*)), p, SLOT(moveUpToken(QWidget*)));
         signalMapperMoveDown = new QSignalMapper(p);
-        connect(signalMapperMoveDown, SIGNAL(mapped(QWidget *)), p, SLOT(moveDownToken(QWidget *)));
+        connect(signalMapperMoveDown, SIGNAL(mapped(QWidget*)), p, SLOT(moveDownToken(QWidget*)));
         signalMapperRemove = new QSignalMapper(p);
-        connect(signalMapperRemove, SIGNAL(mapped(QWidget *)), p, SLOT(removeToken(QWidget *)));
+        connect(signalMapperRemove, SIGNAL(mapped(QWidget*)), p, SLOT(removeToken(QWidget*)));
 
     }
 
@@ -354,6 +474,9 @@ public:
             struct IdSuggestions::IdSuggestionTokenInfo info;
             info.inBetween = QString();
             info.len = -1;
+            info.startWord = 0;
+            info.endWord = 0x00ffffff;
+            info.lastWord = false;
             info.caseChange = IdSuggestions::ccNoChange;
             tokenWidget = new TitleWidget(info, true, p, container);
             widgetList << tokenWidget;
@@ -364,8 +487,11 @@ public:
             struct IdSuggestions::IdSuggestionTokenInfo info;
             info.inBetween = QString();
             info.len = -1;
+            info.startWord = 0;
+            info.endWord = 0x00ffffff;
+            info.lastWord = false;
             info.caseChange = IdSuggestions::ccNoChange;
-            tokenWidget = new AuthorWidget(info, aAll, p, container);
+            tokenWidget = new AuthorWidget(info, p, container);
             widgetList << tokenWidget;
             containerLayout->insertWidget(pos, tokenWidget, 1);
         }
@@ -375,6 +501,19 @@ public:
             widgetList << tokenWidget;
             containerLayout->insertWidget(pos, tokenWidget, 1);
             break;
+        case ttJournal: {
+            struct IdSuggestions::IdSuggestionTokenInfo info;
+            info.inBetween = QString();
+            info.len = 1;
+            info.startWord = 0;
+            info.endWord = 0x00ffffff;
+            info.lastWord = false;
+            info.caseChange = IdSuggestions::ccNoChange;
+            tokenWidget = new JournalWidget(info, p, container);
+            widgetList << tokenWidget;
+            containerLayout->insertWidget(pos, tokenWidget, 1);
+        }
+        break;
         case ttText:
             tokenWidget = new TextWidget(QString(), p, container);
             widgetList << tokenWidget;
@@ -388,14 +527,20 @@ public:
         while (!widgetList.isEmpty())
             delete widgetList.takeFirst();
 
-        QStringList tokenList = formatString.split(QLatin1Char('|'), QString::SkipEmptyParts);
+        QStringList tokenList = formatString.split(QLatin1String("|"), QString::SkipEmptyParts);
         foreach(const QString &token, tokenList) {
             TokenWidget *tokenWidget = NULL;
 
             if (token[0] == 'a' || token[0] == 'A' || token[0] == 'z') {
-                IdSuggestions::Authors author = token[0] == 'a' ? aOnlyFirst : (token[0] == 'A' ? aAll : aNotFirst);
                 struct IdSuggestions::IdSuggestionTokenInfo info = p->evalToken(token.mid(1));
-                tokenWidget = new AuthorWidget(info, author, p, container);
+                /// Support deprecated 'a' and 'z' cases
+                if (token[0] == 'a')
+                    info.startWord = info.endWord = 0;
+                else if (token[0] == 'z') {
+                    info.startWord = 1;
+                    info.endWord = 0x00ffffff;
+                }
+                tokenWidget = new AuthorWidget(info, p, container);
                 widgetList << tokenWidget;
                 containerLayout->insertWidget(containerLayout->count() - 2, tokenWidget, 1);
             } else if (token[0] == 'y') {
@@ -411,6 +556,11 @@ public:
                 tokenWidget = new TitleWidget(info, token[0] == 'T', p, container);
                 widgetList << tokenWidget;
                 containerLayout->insertWidget(containerLayout->count() - 2, tokenWidget, 1);
+            } else if (token[0] == 'j') {
+                struct IdSuggestions::IdSuggestionTokenInfo info = p->evalToken(token.mid(1));
+                tokenWidget = new JournalWidget(info, p, container);
+                widgetList << tokenWidget;
+                containerLayout->insertWidget(containerLayout->count() - 2, tokenWidget, 1);
             } else if (token[0] == '"') {
                 tokenWidget = new TextWidget(token.mid(1), p, container);
                 widgetList << tokenWidget;
@@ -524,13 +674,14 @@ IdSuggestionsEditDialog::~IdSuggestionsEditDialog()
 QString IdSuggestionsEditDialog::editSuggestion(const Entry *previewEntry, const QString &suggestion, QWidget *parent)
 {
     QPointer<IdSuggestionsEditDialog> dlg = new IdSuggestionsEditDialog(parent);
-    IdSuggestionsEditWidget widget(previewEntry, dlg);
-    dlg->setMainWidget(&widget);
+    IdSuggestionsEditWidget *widget = new IdSuggestionsEditWidget(previewEntry, dlg);
+    dlg->setMainWidget(widget);
 
-    widget.setFormatString(suggestion);
+    widget->setFormatString(suggestion);
     if (dlg->exec() == Accepted) {
+        const QString formatString = widget->formatString();
         delete dlg;
-        return widget.formatString();
+        return formatString;
     }
 
     delete dlg;
