@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "valuelistmodel.h"
 
 #include <typeinfo>
 
@@ -39,11 +38,11 @@
 #include <QLayout>
 #include <QHeaderView>
 
-#include <fieldlineedit.h>
-#include <bibtexfields.h>
-#include <entry.h>
-#include <preferences.h>
-#include "valuelistmodel.h"
+#include "fieldlineedit.h"
+#include "bibtexfields.h"
+#include "entry.h"
+#include "preferences.h"
+#include "filemodel.h"
 
 const int CountRole = Qt::UserRole + 611;
 
@@ -191,16 +190,9 @@ static QRegExp ignoredInSorting("[{}\\\\]+");
 ValueListModel::ValueListModel(const File *bibtexFile, const QString &fieldName, QObject *parent)
         : QAbstractTableModel(parent), file(bibtexFile), fName(fieldName.toLower()), showCountColumn(true), sortBy(SortByText)
 {
-    /// load mapping from color value to label
-    KSharedConfigPtr config(KSharedConfig::openConfig(QLatin1String("kbibtexrc")));
-    KConfigGroup configGroup(config, Preferences::groupColor);
-    QStringList colorCodes = configGroup.readEntry(Preferences::keyColorCodes, Preferences::defaultColorCodes);
-    QStringList colorLabels = configGroup.readEntry(Preferences::keyColorLabels, Preferences::defaultcolorLabels);
-    for (QStringList::ConstIterator itc = colorCodes.constBegin(), itl = colorLabels.constBegin(); itc != colorCodes.constEnd() && itl != colorLabels.constEnd(); ++itc, ++itl) {
-        colorToLabel.insert(*itc, *itl);
-    }
-
+    readConfiguration();
     updateValues();
+    NotificationHub::registerNotificationListener(this, NotificationHub::EventConfigurationChanged);
 }
 
 int ValueListModel::rowCount(const QModelIndex &parent) const
@@ -230,7 +222,7 @@ QVariant ValueListModel::data(const QModelIndex &index, int role) const
         } else
             return QVariant(values[index.row()].count);
     } else if (role == SortRole) {
-        QString buffer = values[index.row()].sortBy.isNull() ? values[index.row()].text : values[index.row()].sortBy;
+        QString buffer = values[index.row()].sortBy.isEmpty() ? values[index.row()].text : values[index.row()].sortBy;
         buffer = buffer.remove(ignoredInSorting).toLower();
 
         if ((showCountColumn && index.column() == 1) || (!showCountColumn && sortBy == SortByCount)) {
@@ -326,6 +318,27 @@ void ValueListModel::setSortBy(SortBy sortBy)
     reset();
 }
 
+void ValueListModel::notificationEvent(int eventId)
+{
+    if (eventId == NotificationHub::EventConfigurationChanged) {
+        readConfiguration();
+        reset();
+    }
+}
+
+void ValueListModel::readConfiguration()
+{
+    /// load mapping from color value to label
+    KSharedConfigPtr config(KSharedConfig::openConfig(QLatin1String("kbibtexrc")));
+    KConfigGroup configGroup(config, Preferences::groupColor);
+    QStringList colorCodes = configGroup.readEntry(Preferences::keyColorCodes, Preferences::defaultColorCodes);
+    QStringList colorLabels = configGroup.readEntry(Preferences::keyColorLabels, Preferences::defaultcolorLabels);
+    colorToLabel.clear();
+    for (QStringList::ConstIterator itc = colorCodes.constBegin(), itl = colorLabels.constBegin(); itc != colorCodes.constEnd() && itl != colorLabels.constEnd(); ++itc, ++itl) {
+        colorToLabel.insert(*itc, i18n((*itl).toUtf8().constData()));
+    }
+}
+
 void ValueListModel::updateValues()
 {
     values.clear();
@@ -350,7 +363,7 @@ void ValueListModel::updateValues()
 void ValueListModel::insertValue(const Value &value)
 {
     foreach(QSharedPointer<ValueItem> item, value) {
-        const QString text = PlainTextValue::text(*item, file);
+        const QString text = PlainTextValue::text(*item);
         if (text.isEmpty()) continue; ///< skip empty values
 
         int index = indexOf(text);
@@ -456,7 +469,7 @@ bool ValueListModel::searchAndReplaceValueInModel(const QModelIndex &index, cons
         values[row].text = newText;
         values[row].value = newValue;
         const QSharedPointer<Person> person = newValue.first().dynamicCast<Person>();
-        values[row].sortBy = person.isNull() ? QString::null : person->lastName() + QLatin1String(" ") + person->firstName();
+        values[row].sortBy = person.isNull() ? QString() : person->lastName() + QLatin1String(" ") + person->firstName();
     } else {
         /// The user-entered text existed before
 
@@ -563,4 +576,3 @@ void ValueListModel::removeValueFromModel(const QModelIndex &index)
     values.remove(lastRow);
     endRemoveRows();
 }
-
