@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2013 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "filterbar.h"
 
 #include <QLayout>
 #include <QLabel>
@@ -30,7 +29,6 @@
 #include <KStandardDirs>
 #include <KIcon>
 
-#include "filterbar.h"
 #include "bibtexfields.h"
 #include "delayedexecutiontimer.h"
 
@@ -62,37 +60,35 @@ public:
         delete delayedTimer;
     }
 
-    SortFilterBibTeXFileModel::FilterQuery filter() {
-        SortFilterBibTeXFileModel::FilterQuery result;
-        result.combination = comboBoxCombination->currentIndex() == 0 ? SortFilterBibTeXFileModel::AnyTerm : SortFilterBibTeXFileModel::EveryTerm;
+    SortFilterFileModel::FilterQuery filter() {
+        SortFilterFileModel::FilterQuery result;
+        result.combination = comboBoxCombination->currentIndex() == 0 ? SortFilterFileModel::AnyTerm : SortFilterFileModel::EveryTerm;
         result.terms.clear();
         if (comboBoxCombination->currentIndex() == 2) /// exact phrase
             result.terms << comboBoxFilterText->lineEdit()->text();
         else /// any or every word
             result.terms = comboBoxFilterText->lineEdit()->text().split(QRegExp(QLatin1String("\\s+")), QString::SkipEmptyParts);
-        result.field = comboBoxField->currentIndex() == 0 ? QString::null : comboBoxField->itemData(comboBoxField->currentIndex(), Qt::UserRole).toString();
+        result.field = comboBoxField->currentIndex() == 0 ? QString() : comboBoxField->itemData(comboBoxField->currentIndex(), Qt::UserRole).toString();
         result.searchPDFfiles = buttonSearchPDFfiles->isChecked();
 
         return result;
     }
 
-    void setFilter(SortFilterBibTeXFileModel::FilterQuery fq) {
+    void setFilter(SortFilterFileModel::FilterQuery fq) {
         /// Avoid triggering loops of activation
         comboBoxCombination->blockSignals(true);
         /// Set check state for action for either "any word",
         /// "every word", or "exact phrase", respectively
-        const int combinationIndex = fq.combination == SortFilterBibTeXFileModel::AnyTerm ? 0 : (fq.terms.count() < 2 ? 2 : 1);
+        const int combinationIndex = fq.combination == SortFilterFileModel::AnyTerm ? 0 : (fq.terms.count() < 2 ? 2 : 1);
         comboBoxCombination->setCurrentIndex(combinationIndex);
         /// Reset activation block
         comboBoxCombination->blockSignals(false);
 
-
         /// Avoid triggering loops of activation
         comboBoxField->blockSignals(true);
         /// Find and check action that corresponds to field name ("author", ...)
         const QString lower = fq.field.toLower();
         for (int idx = comboBoxField->count() - 1; idx >= 0; --idx) {
-            const QString lower = fq.field.toLower();
             if (comboBoxField->itemData(idx, Qt::UserRole).toString().toLower() == lower) {
                 comboBoxField->setCurrentIndex(idx);
                 break;
@@ -176,35 +172,32 @@ public:
 FilterBar::FilterBar(QWidget *parent)
         : QWidget(parent), d(new FilterBarPrivate(this))
 {
-    QGridLayout *layout = new QGridLayout(this);
+    QBoxLayout *layout = new QHBoxLayout(this);
     layout->setMargin(0);
-    layout->setRowStretch(0, 1);
-    layout->setRowStretch(1, 0);
-    layout->setRowStretch(2, 1);
 
     QLabel *label = new QLabel(i18n("Filter:"), this);
-    layout->addWidget(label, 1, 0);
+    layout->addWidget(label, 0);
 
     d->comboBoxFilterText = new KComboBox(true, this);
     label->setBuddy(d->comboBoxFilterText);
-    setFocusProxy(d->comboBoxFilterText);
-    layout->addWidget(d->comboBoxFilterText, 1, 1);
+    layout->addWidget(d->comboBoxFilterText, 5);
     d->comboBoxFilterText->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
     d->comboBoxFilterText->setEditable(true);
     QFontMetrics metrics(d->comboBoxFilterText->font());
     d->comboBoxFilterText->setMinimumWidth(metrics.width(QLatin1String("AIWaiw")) * 7);
     KLineEdit *lineEdit = static_cast<KLineEdit *>(d->comboBoxFilterText->lineEdit());
     lineEdit->setClearButtonShown(true);
+    lineEdit->setClickMessage(i18n("Filter bibliographic entries"));
 
     d->comboBoxCombination = new KComboBox(false, this);
-    layout->addWidget(d->comboBoxCombination, 1, 2);
+    layout->addWidget(d->comboBoxCombination, 1);
     d->comboBoxCombination->addItem(i18n("any word")); /// AnyWord=0
     d->comboBoxCombination->addItem(i18n("every word")); /// EveryWord=1
     d->comboBoxCombination->addItem(i18n("exact phrase")); /// ExactPhrase=2
     d->comboBoxCombination->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
 
     d->comboBoxField = new KComboBox(false, this);
-    layout->addWidget(d->comboBoxField, 1, 3);
+    layout->addWidget(d->comboBoxField, 1);
     d->comboBoxField->addItem(i18n("any field"), QVariant());
     d->comboBoxField->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
 
@@ -217,12 +210,12 @@ FilterBar::FilterBar(QWidget *parent)
     d->buttonSearchPDFfiles->setIcon(KIcon("application-pdf"));
     d->buttonSearchPDFfiles->setToolTip(i18n("Include PDF files in full-text search"));
     d->buttonSearchPDFfiles->setCheckable(true);
-    layout->addWidget(d->buttonSearchPDFfiles, 1, 4);
+    layout->addWidget(d->buttonSearchPDFfiles, 0);
 
     d->buttonClearAll = new KPushButton(this);
     d->buttonClearAll->setIcon(KIcon("edit-clear-locationbar-rtl"));
     d->buttonClearAll->setToolTip(i18n("Reset filter criteria"));
-    layout->addWidget(d->buttonClearAll, 1, 5);
+    layout->addWidget(d->buttonClearAll, 0);
 
     connect(d->comboBoxFilterText->lineEdit(), SIGNAL(textChanged(QString)), d->delayedTimer, SLOT(trigger()));
     connect(d->comboBoxFilterText->lineEdit(), SIGNAL(returnPressed()), this, SLOT(userPressedEnter()));
@@ -246,6 +239,8 @@ FilterBar::FilterBar(QWidget *parent)
 
     d->restoreState();
 
+    setFocusProxy(d->comboBoxFilterText);
+
     QTimer::singleShot(250, this, SLOT(buttonHeight()));
 }
 
@@ -254,17 +249,22 @@ FilterBar::~FilterBar()
     delete d;
 }
 
-void FilterBar::setFilter(SortFilterBibTeXFileModel::FilterQuery fq)
+void FilterBar::setFilter(SortFilterFileModel::FilterQuery fq)
 {
     d->setFilter(fq);
     emit filterChanged(fq);
 }
 
-SortFilterBibTeXFileModel::FilterQuery FilterBar::filter()
+SortFilterFileModel::FilterQuery FilterBar::filter()
 {
     return d->filter();
 }
 
+void FilterBar::setClickMessage(const QString &msg) {
+    KLineEdit *lineEdit = static_cast<KLineEdit *>(d->comboBoxFilterText->lineEdit());
+    lineEdit->setClickMessage(msg);
+}
+
 void FilterBar::comboboxStatusChanged()
 {
     d->buttonSearchPDFfiles->setEnabled(d->comboBoxField->currentIndex() == 0);
