@@ -1,42 +1,45 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "menulineedit.h"
 
 #include <QLayout>
 #include <QApplication>
 #include <QMenu>
 #include <QTimer>
 
-#include <KTextEdit>
 #include <KPushButton>
 #include <KLineEdit>
+#include <KTextEdit>
+#include <KConfigGroup>
 
-#include "menulineedit.h"
+#include "notificationhub.h"
 
-class MenuLineEdit::MenuLineEditPrivate
+const int MenuLineEdit::MenuLineConfigurationChangedEvent = NotificationHub::EventUserDefined + 1861;
+const QString MenuLineEdit::keyLimitKeyboardTabStops = QLatin1String("LimitKeyboardTabStops");
+
+class MenuLineEdit::MenuLineEditPrivate : public NotificationListener
 {
 private:
     MenuLineEdit *p;
     bool isMultiLine;
     bool m_isReadOnly;
     QHBoxLayout *hLayout;
-    const QString transparentStyleSheet, normalStyleSheet;
+    static const QString transparentStyleSheet, normalStyleSheet;
     bool makeInnerWidgetsTransparent;
 
 public:
@@ -45,17 +48,10 @@ public:
     KTextEdit *m_multiLineEditText;
 
     MenuLineEditPrivate(bool isMultiLine, MenuLineEdit *parent)
-            : p(parent), m_isReadOnly(false),
-          // FIXME much here is hard-coded. do it better?
-          transparentStyleSheet(
-              QLatin1String("KTextEdit { border-style: none; background-color: transparent; }")
-              + QLatin1String("KLineEdit { border-style: none; background-color: transparent; }")
-              + QLatin1String("KPushButton { border-style: none; background-color: transparent; padding: 0px; margin-left: 2px; margin-right:2px; text-align: left; }")
-          ), normalStyleSheet(
-              QLatin1String("KPushButton { padding:4px; margin:0px;  text-align: left; }")
-              + QLatin1String("QPushButton::menu-indicator {subcontrol-position: right center; subcontrol-origin: content;}")
-          ), makeInnerWidgetsTransparent(false), m_singleLineEditText(NULL), m_multiLineEditText(NULL) {
+            : p(parent), m_isReadOnly(false), makeInnerWidgetsTransparent(false), m_singleLineEditText(NULL), m_multiLineEditText(NULL) {
         this->isMultiLine = isMultiLine;
+        /// listen to configuration change events specifically concerning a MenuLineEdit widget
+        NotificationHub::registerNotificationListener(this, MenuLineEdit::MenuLineConfigurationChangedEvent);
     }
 
     ~MenuLineEditPrivate() {
@@ -65,6 +61,25 @@ public:
         }
     }
 
+    virtual void notificationEvent(int eventId) {
+        if (eventId == MenuLineEdit::MenuLineConfigurationChangedEvent) {
+            /// load setting limitKeyboardTabStops
+            KSharedConfigPtr config(KSharedConfig::openConfig(QLatin1String("kbibtexrc")));
+            static QString const configGroupName = QLatin1String("User Interface");
+            KConfigGroup configGroup(config, configGroupName);
+            const bool limitKeyboardTabStops = configGroup.readEntry(MenuLineEdit::keyLimitKeyboardTabStops, false);
+
+            /// check each widget inside MenuLineEdit
+            for (int i = hLayout->count() - 1; i >= 0; --i) {
+                QWidget *w = hLayout->itemAt(i)->widget();
+                if (w != NULL && w != m_singleLineEditText && w != m_multiLineEditText) {
+                    /// for all widgets except the main editing widget: change tab focus policy
+                    w->setFocusPolicy(limitKeyboardTabStops ? Qt::ClickFocus : Qt::StrongFocus);
+                }
+            }
+        }
+    }
+
     void setupUI() {
         p->setObjectName("FieldLineEdit");
 
@@ -169,6 +184,9 @@ public:
     }
 };
 
+const QString MenuLineEdit::MenuLineEditPrivate::transparentStyleSheet = QLatin1String("KTextEdit { border-style: none; background-color: transparent; }\nKLineEdit { border-style: none; background-color: transparent; }\nKPushButton { border-style: none; background-color: transparent; padding: 0px; margin-left: 2px; margin-right:2px; text-align: left; }");
+const QString MenuLineEdit::MenuLineEditPrivate::normalStyleSheet = QLatin1String("KPushButton { padding:4px; margin:0px;  text-align: left; }\nQPushButton::menu-indicator {subcontrol-position: right center; subcontrol-origin: content;}");
+
 MenuLineEdit::MenuLineEdit(bool isMultiLine, QWidget *parent)
         : QFrame(parent), d(new MenuLineEditPrivate(isMultiLine, this))
 {
