@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "internalnetworkaccessmanager.h"
 
 #include <ctime>
 
@@ -28,14 +27,13 @@
 #include <QNetworkRequest>
 #include <QNetworkReply>
 #include <QtGlobal>
-#include <QNetworkProxy>
+#include <QTimer>
 
+#include <KDebug>
 #include <KUrl>
 #include <KApplication>
 #include <KProtocolManager>
 
-#include "internalnetworkaccessmanager.h"
-
 /**
  * @author Thomas Fischer <fischer@unix-ag.uni-kl.de>
  */
@@ -54,7 +52,7 @@ public:
             const QString path = cookieContent.cap(3);
             QUrl cookieUrl = url;
             QList<QNetworkCookie> cookies = cookiesForUrl(cookieUrl);
-            cookies.append(QNetworkCookie(key.toAscii(), value.toAscii()));
+            cookies.append(QNetworkCookie(key.toLatin1(), value.toLatin1()));
             setCookiesFromUrl(cookies, cookieUrl);
         }
     }
@@ -101,7 +99,7 @@ const QStringList userAgentList = QStringList()
                                   << QLatin1String("Mozilla/5.0 (SymbianOS/9.1; U; [en]; Series60/3.0 NokiaE60/4.06.0) AppleWebKit/413 (KHTML, like Gecko) Safari/413")
                                   << QLatin1String("Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.133 Safari/534.16");
 
-QString InternalNetworkAccessManager::userAgentString = QString::null;
+QString InternalNetworkAccessManager::userAgentString;
 InternalNetworkAccessManager *InternalNetworkAccessManager::instance = NULL;
 
 InternalNetworkAccessManager::InternalNetworkAccessManager(QObject *parent)
@@ -135,7 +133,7 @@ QNetworkReply *InternalNetworkAccessManager::get(QNetworkRequest &request, const
     if (!proxyHostName.isEmpty() && proxyHostName != QLatin1String("DIRECT")) {
         /// Extract both hostname and port number for proxy
         proxyHostName = proxyHostName.mid(proxyHostName.indexOf(QLatin1String("://")) + 3);
-        QStringList proxyComponents = proxyHostName.split(QLatin1Char(':'), QString::SkipEmptyParts);
+        QStringList proxyComponents = proxyHostName.split(QLatin1String(":"), QString::SkipEmptyParts);
         if (proxyComponents.length() == 1) {
             /// Proxy configuration is missing a port number,
             /// using 8080 as default
@@ -150,12 +148,13 @@ QNetworkReply *InternalNetworkAccessManager::get(QNetworkRequest &request, const
         setProxy(QNetworkProxy());
     }
 
-    request.setRawHeader(QString("Accept").toAscii(), QString("text/*, */*;q=0.7").toAscii());
-    request.setRawHeader(QString("Accept-Charset").toAscii(), QString("utf-8, us-ascii, ISO-8859-1;q=0.7, ISO-8859-15;q=0.7, windows-1252;q=0.3").toAscii());
-    request.setRawHeader(QString("Accept-Language").toAscii(), QString("en-US, en;q=0.9").toAscii());
-    request.setRawHeader("User-Agent", userAgent().toAscii());
+    if (!request.hasRawHeader(QString("Accept").toLatin1()))
+        request.setRawHeader(QString("Accept").toLatin1(), QString("text/*, */*;q=0.7").toLatin1());
+    request.setRawHeader(QString("Accept-Charset").toLatin1(), QString("utf-8, us-ascii, ISO-8859-1;q=0.7, ISO-8859-15;q=0.7, windows-1252;q=0.3").toLatin1());
+    request.setRawHeader(QString("Accept-Language").toLatin1(), QString("en-US, en;q=0.9").toLatin1());
+    request.setRawHeader("User-Agent", userAgent().toLatin1());
     if (oldUrl.isValid())
-        request.setRawHeader("Referer", oldUrl.toString().toAscii());
+        request.setRawHeader("Referer", oldUrl.toString().toLatin1());
     QNetworkReply *reply = QNetworkAccessManager::get(request);
     return reply;
 }
@@ -173,3 +172,34 @@ QString InternalNetworkAccessManager::userAgent()
     }
     return userAgentString;
 }
+
+void InternalNetworkAccessManager::setNetworkReplyTimeout(QNetworkReply *reply, int timeOutSec)
+{
+    QTimer *timer = new QTimer(reply);
+    connect(timer, SIGNAL(timeout()), this, SLOT(networkReplyTimeout()));
+    m_mapTimerToReply.insert(timer, reply);
+    timer->start(timeOutSec * 1000);
+    kDebug() << "Setting timeout of" << timer->interval() << "seconds to " << reply->url().toString();
+    connect(reply, SIGNAL(finished()), this, SLOT(networkReplyFinished()));
+}
+
+void InternalNetworkAccessManager::networkReplyTimeout()
+{
+    QTimer *timer = static_cast<QTimer *>(sender());
+    timer->stop();
+    QNetworkReply *reply = m_mapTimerToReply[timer];
+    if (reply != NULL) {
+        kDebug() << "Timeout on reply to " << reply->url().toString();
+        reply->close();
+        m_mapTimerToReply.remove(timer);
+    }
+}
+void InternalNetworkAccessManager::networkReplyFinished()
+{
+    QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
+    QTimer *timer = m_mapTimerToReply.key(reply, NULL);
+    if (timer != NULL) {
+        timer->stop();
+        m_mapTimerToReply.remove(timer);
+    }
+}
