@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "onlinesearchabstract.h"
 
 #include <QFileInfo>
 #include <QNetworkReply>
@@ -32,10 +31,9 @@
 #include <KMessageBox>
 #include <KPassivePopup>
 
-#include <encoderlatex.h>
-#include <internalnetworkaccessmanager.h>
+#include "encoderlatex.h"
+#include "internalnetworkaccessmanager.h"
 #include "kbibtexnamespace.h"
-#include "onlinesearchabstract.h"
 
 const QString OnlineSearchAbstract::queryKeyFreeText = QLatin1String("free");
 const QString OnlineSearchAbstract::queryKeyTitle = QLatin1String("title");
@@ -47,6 +45,7 @@ const int OnlineSearchAbstract::resultCancelled = 0; /// may get redefined in th
 const int OnlineSearchAbstract::resultUnspecifiedError = 1;
 const int OnlineSearchAbstract::resultAuthorizationRequired = 2;
 const int OnlineSearchAbstract::resultNetworkError = 3;
+const int OnlineSearchAbstract::resultInvalidArguments = 4;
 
 const char *OnlineSearchAbstract::httpUnsafeChars = "%:/=+$?&\0";
 
@@ -66,15 +65,15 @@ QStringList OnlineSearchQueryFormAbstract::authorLastNames(const Entry &entry)
 }
 
 OnlineSearchAbstract::OnlineSearchAbstract(QWidget *parent)
-        : QObject(parent), m_name(QString::null)
+        : QObject(parent), m_name(QString())
 {
     m_parent = parent;
 }
 
 KIcon OnlineSearchAbstract::icon(QListWidgetItem *listWidgetItem)
 {
-    static const QRegExp invalidChars("[^-a-z0-9_]", Qt::CaseInsensitive);
-    const QString fileNameStem = KStandardDirs::locateLocal("cache", "favicons/") + QString(favIconUrl()).remove(invalidChars);
+    static const QRegExp invalidChars(QLatin1String("[^-a-z0-9_]"), Qt::CaseInsensitive);
+    const QString fileNameStem = KStandardDirs::locateLocal("cache", QLatin1String("favicons/")) + QString(favIconUrl()).remove(invalidChars);
     const QStringList fileNameExtensions = QStringList() << QLatin1String(".ico") << QLatin1String(".png") << QString();
 
     foreach(const QString &extension, fileNameExtensions) {
@@ -95,7 +94,7 @@ KIcon OnlineSearchAbstract::icon(QListWidgetItem *listWidgetItem)
 QString OnlineSearchAbstract::name()
 {
     static const QRegExp invalidChars("[^-a-z0-9]", Qt::CaseInsensitive);
-    if (m_name.isNull())
+    if (m_name.isEmpty())
         m_name = label().remove(invalidChars);
     return m_name;
 }
@@ -217,7 +216,7 @@ QString OnlineSearchAbstract::encodeURL(QString rawText)
         rawText = rawText.replace(QChar(*cur), '%' + QString::number(*cur, 16));
         ++cur;
     }
-    rawText = rawText.replace(" ", "+");
+    rawText = rawText.replace(QLatin1Char(' '), QLatin1Char('+'));
     return rawText;
 }
 
@@ -230,7 +229,7 @@ QString OnlineSearchAbstract::decodeURL(QString rawText)
         if (ok)
             rawText = rawText.replace(mimeRegExp.cap(0), c);
     }
-    rawText = rawText.replace("&amp;", "&").replace("+", " ");
+    rawText = rawText.replace(QLatin1String("&amp;"), QLatin1String("&")).replace(QLatin1Char('+'), QLatin1String(" "));
     return rawText;
 }
 
@@ -296,15 +295,15 @@ QMap<QString, QString> OnlineSearchAbstract::formParameters(const QString &htmlT
     p = htmlText.indexOf(selectTagBegin, startPos, Qt::CaseInsensitive);
     while (p > startPos && p < endPos) {
         /// get "name" attribute from "select" tag
-        QString selectName = selectNameRegExp.indexIn(htmlText, p) == p ? selectNameRegExp.cap(1) : QString::null;
+        const QString selectName = selectNameRegExp.indexIn(htmlText, p) == p ? selectNameRegExp.cap(1) : QString();
 
         /// "select" tag contains one or several "option" tags, search all
         int popt = htmlText.indexOf(optionTagBegin, p, Qt::CaseInsensitive);
         int endSelect = htmlText.indexOf(selectTagEnd, p, Qt::CaseInsensitive);
         while (popt > p && popt < endSelect) {
             /// get "value" attribute from "option" tag
-            QString optionValue = optionValueRegExp.indexIn(htmlText, popt) == popt ? optionValueRegExp.cap(1) : QString::null;
-            if (!selectName.isNull() && !optionValue.isNull()) {
+            const QString optionValue = optionValueRegExp.indexIn(htmlText, popt) == popt ? optionValueRegExp.cap(1) : QString();
+            if (!selectName.isEmpty() && !optionValue.isEmpty()) {
                 /// if this "option" tag is "selected", store value
                 if (htmlText.indexOf(optionSelectedRegExp, popt) == popt) {
                     result[selectName] = optionValue;
@@ -320,42 +319,18 @@ QMap<QString, QString> OnlineSearchAbstract::formParameters(const QString &htmlT
     return result;
 }
 
-
-void OnlineSearchAbstract::setNetworkReplyTimeout(QNetworkReply *reply, int timeOutSec)
-{
-    QTimer *timer = new QTimer(reply);
-    connect(timer, SIGNAL(timeout()), this, SLOT(networkReplyTimeout()));
-    m_mapTimerToReply.insert(timer, reply);
-    timer->start(timeOutSec * 1000);
-    connect(reply, SIGNAL(finished()), this, SLOT(networkReplyFinished()));
-}
-
-void OnlineSearchAbstract::networkReplyTimeout()
-{
-    QTimer *timer = static_cast<QTimer *>(sender());
-    QNetworkReply *reply = m_mapTimerToReply[timer];
-    if (reply != NULL) {
-        kDebug() << "Timout on reply to " << reply->url().toString();
-        reply->close();
-        m_mapTimerToReply.remove(timer);
-    }
-}
-void OnlineSearchAbstract::networkReplyFinished()
-{
-    QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
-    QTimer *timer = m_mapTimerToReply.key(reply, NULL);
-    if (timer != NULL) {
-        m_mapTimerToReply.remove(timer);
-        timer->stop();
-    }
-}
-
 void OnlineSearchAbstract::iconDownloadFinished()
 {
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
     if (reply->error() == QNetworkReply::NoError) {
         const QByteArray iconData = reply->readAll();
+        if (iconData.size() < 10) {
+            /// Unlikely that an icon's data is less than 10 bytes,
+            /// must be an error.
+            qWarning() << "Received invalid icon data from " << reply->url().toString();
+            return;
+        }
 
         QString extension;
         if (iconData[1] == 'P' && iconData[2] == 'N' && iconData[3] == 'G') {
@@ -405,3 +380,78 @@ void OnlineSearchAbstract::delayedStoppedSearchTimer()
     emit progress(1, 1);
     emit stoppedSearch(m_delayedStoppedSearchReturnCode);
 }
+
+void OnlineSearchAbstract::sanitizeEntry(QSharedPointer<Entry> entry)
+{
+    if (entry.isNull()) return;
+
+    /// Sometimes, there is no identifier, so set a random one
+    if (entry->id().isEmpty())
+        entry->setId(QString(QLatin1String("entry-%1")).arg(QString::number(qrand(), 36)));
+
+    const QString ftIssue = QLatin1String("issue");
+    if (entry->contains(ftIssue)) {
+        /// ACM's Digital Library uses "issue" instead of "number" -> fix that
+        Value v = entry->value(ftIssue);
+        entry->remove(ftIssue);
+        entry->insert(Entry::ftNumber, v);
+    }
+
+    /// If entry contains a description field but no abstract,
+    /// rename description field to abstract
+    const QString ftDescription = QLatin1String("description");
+    if (!entry->contains(Entry::ftAbstract) && entry->contains(ftDescription)) {
+        Value v = entry->value(ftDescription);
+        entry->remove(ftDescription);
+        entry->insert(Entry::ftAbstract, v);
+    }
+
+    /// Remove "dblp" artifacts in abstracts and keywords
+    if (entry->contains(Entry::ftAbstract)) {
+        const QString abstract = PlainTextValue::text(entry->value(Entry::ftAbstract));
+        if (abstract == QLatin1String("dblp"))
+            entry->remove(Entry::ftAbstract);
+    }
+    if (entry->contains(Entry::ftKeywords)) {
+        const QString keywords = PlainTextValue::text(entry->value(Entry::ftKeywords));
+        if (keywords == QLatin1String("dblp"))
+            entry->remove(Entry::ftKeywords);
+    }
+
+    if (entry->contains(Entry::ftMonth)) {
+        /// Fix strigns for months: "September" -> "sep"
+        const QString monthStr = PlainTextValue::text(entry->value(Entry::ftMonth));
+
+        static const QRegExp longMonth = QRegExp(QLatin1String("(jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)[a-z]+"), Qt::CaseInsensitive);
+        if (monthStr.indexOf(longMonth) == 0 && monthStr == longMonth.cap(0)) {
+            /// String used for month is actually a full name, therefore replace it
+            entry->remove(Entry::ftMonth);
+
+            /// Regular expression checked for valid three-letter abbreviations
+            /// that month names start with. Use those three letters for a macro key
+            /// Example: "September" -> sep
+            Value v;
+            v.append(QSharedPointer<MacroKey>(new MacroKey(longMonth.cap(1).toLower())));
+            entry->insert(Entry::ftMonth, v);
+        }
+    }
+
+    /// Referenced strings or entries do not exist in the search result
+    /// and BibTeX breaks if it finds a reference to a non-existing string or entry
+    entry->remove(Entry::ftCrossRef);
+}
+
+bool OnlineSearchAbstract::publishEntry(QSharedPointer<Entry> entry)
+{
+    if (entry.isNull()) return false;
+
+    Value v;
+    v.append(QSharedPointer<VerbatimText>(new VerbatimText(label())));
+    entry->insert("x-fetchedfrom", v);
+
+    sanitizeEntry(entry);
+
+    emit foundEntry(entry);
+
+    return true;
+}
