@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "onlinesearcharxiv.h"
 
 #include <QGridLayout>
 #include <QLabel>
@@ -31,9 +30,8 @@
 #include <KConfigGroup>
 
 #include "fileimporterbibtex.h"
-#include "onlinesearcharxiv.h"
-#include <internalnetworkaccessmanager.h>
 #include "xsltransform.h"
+#include "internalnetworkaccessmanager.h"
 
 
 class OnlineSearchArXiv::OnlineSearchQueryFormArXiv : public OnlineSearchQueryFormAbstract
@@ -84,7 +82,7 @@ public:
     }
 
     void copyFromEntry(const Entry &entry) {
-        lineEditFreeText->setText(authorLastNames(entry).join(" ") + " " + PlainTextValue::text(entry[Entry::ftTitle]));
+        lineEditFreeText->setText(authorLastNames(entry).join(QLatin1String(" ")) + QLatin1Char(' ') + PlainTextValue::text(entry[Entry::ftTitle]));
     }
 
     void saveState() {
@@ -108,7 +106,10 @@ public:
 
     OnlineSearchArXivPrivate(OnlineSearchArXiv *parent)
             : p(parent), form(NULL), arXivQueryBaseUrl("http://export.arxiv.org/api/query?") {
-        xslt = XSLTransform::createXSLTransform(KStandardDirs::locate("data", "kbibtex/arxiv2bibtex.xsl"));
+        const QString xsltFilename = QLatin1String("kbibtex/arxiv2bibtex.xsl");
+        xslt = XSLTransform::createXSLTransform(KStandardDirs::locate("data", xsltFilename));
+        if (xslt == NULL)
+            kWarning() << "Could not create XSLT transformation for" << xsltFilename;
     }
 
     ~OnlineSearchArXivPrivate() {
@@ -119,8 +120,8 @@ public:
         /// format search terms
         QStringList queryFragments;
 
-        foreach(const QString &queryFragment, p->splitRespectingQuotationMarks(form->lineEditFreeText->text()))
-        queryFragments.append(p->encodeURL(queryFragment));
+        foreach (const QString &queryFragment, p->splitRespectingQuotationMarks(form->lineEditFreeText->text()))
+            queryFragments.append(p->encodeURL(queryFragment));
         return KUrl(QString("%1search_query=all:\"%3\"&start=0&max_results=%2").arg(arXivQueryBaseUrl).arg(form->numResultsField->value()).arg(queryFragments.join("\"+AND+all:\""))); ///< join search terms with an AND operation
     }
 
@@ -128,8 +129,8 @@ public:
         /// format search terms
         QStringList queryFragments;
         for (QMap<QString, QString>::ConstIterator it = query.constBegin(); it != query.constEnd(); ++it)
-            foreach(const QString &queryFragment, p->splitRespectingQuotationMarks(it.value()))
-            queryFragments.append(p->encodeURL(queryFragment));
+            foreach (const QString &queryFragment, p->splitRespectingQuotationMarks(it.value()))
+                queryFragments.append(p->encodeURL(queryFragment));
         return KUrl(QString("%1search_query=all:\"%3\"&start=0&max_results=%2").arg(arXivQueryBaseUrl).arg(numResults).arg(queryFragments.join("\"+AND+all:\""))); ///< join search terms with an AND operation
     }
 
@@ -577,13 +578,20 @@ OnlineSearchArXiv::~OnlineSearchArXiv()
 
 void OnlineSearchArXiv::startSearch()
 {
+    if (d->xslt == NULL) {
+        /// Don't allow searches if xslt is not defined
+        kWarning() << "Cannot allow searching" << label() << "if XSL Transformation not available";
+        delayedStoppedSearch(resultUnspecifiedError);
+        return;
+    }
+
     d->curStep = 0;
     d->numSteps = 1;
     m_hasBeenCanceled = false;
 
     QNetworkRequest request(d->buildQueryUrl());
     QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    setNetworkReplyTimeout(reply);
+    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
     connect(reply, SIGNAL(finished()), this, SLOT(downloadDone()));
 
     emit progress(0, d->numSteps);
@@ -593,13 +601,20 @@ void OnlineSearchArXiv::startSearch()
 
 void OnlineSearchArXiv::startSearch(const QMap<QString, QString> &query, int numResults)
 {
+    if (d->xslt == NULL) {
+        /// Don't allow searches if xslt is not defined
+        kWarning() << "Cannot allow searching" << label() << "if XSL Transformation not available";
+        delayedStoppedSearch(resultUnspecifiedError);
+        return;
+    }
+
     d->curStep = 0;
     d->numSteps = 1;
     m_hasBeenCanceled = false;
 
     QNetworkRequest request(d->buildQueryUrl(query, numResults));
     QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    setNetworkReplyTimeout(reply);
+    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
     connect(reply, SIGNAL(finished()), this, SLOT(downloadDone()));
 
     emit progress(0, d->numSteps);
@@ -651,15 +666,7 @@ void OnlineSearchArXiv::downloadDone()
         if (bibtexFile != NULL) {
             for (File::ConstIterator it = bibtexFile->constBegin(); it != bibtexFile->constEnd(); ++it) {
                 QSharedPointer<Entry> entry = (*it).dynamicCast<Entry>();
-                if (!entry.isNull()) {
-                    d->interpreteJournal(*entry);
-                    Value v;
-                    v.append(QSharedPointer<VerbatimText>(new VerbatimText(label())));
-                    entry->insert("x-fetchedfrom", v);
-                    emit foundEntry(entry);
-                    hasEntries = true;
-                }
-
+                hasEntries |= publishEntry(entry);
             }
 
             if (!hasEntries)
@@ -675,3 +682,10 @@ void OnlineSearchArXiv::downloadDone()
     } else
         kDebug() << "url was" << reply->url().toString();
 }
+
+void OnlineSearchArXiv::sanitizeEntry(QSharedPointer<Entry> entry)
+{
+    OnlineSearchAbstract::sanitizeEntry(entry);
+
+    d->interpreteJournal(*entry);
+}
