@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "onlinesearchbibsonomy.h"
 
 #include <QBuffer>
 #include <QLayout>
@@ -32,11 +31,10 @@
 #include <KMessageBox>
 #include <KConfigGroup>
 
-#include <fileimporterbibtex.h>
-#include <file.h>
-#include <entry.h>
-#include <internalnetworkaccessmanager.h>
-#include "onlinesearchbibsonomy.h"
+#include "fileimporterbibtex.h"
+#include "file.h"
+#include "entry.h"
+#include "internalnetworkaccessmanager.h"
 
 class OnlineSearchBibsonomy::OnlineSearchQueryFormBibsonomy : public OnlineSearchQueryFormAbstract
 {
@@ -97,7 +95,7 @@ public:
 
     void copyFromEntry(const Entry &entry) {
         comboBoxSearchWhere->setCurrentIndex(comboBoxSearchWhere->count() - 1);
-        lineEditSearchTerm->setText(authorLastNames(entry).join(" ") + " " + PlainTextValue::text(entry[Entry::ftTitle]));
+        lineEditSearchTerm->setText(authorLastNames(entry).join(QLatin1String(" ")) + QLatin1Char(' ') + PlainTextValue::text(entry[Entry::ftTitle]));
     }
 
     void saveState() {
@@ -130,7 +128,6 @@ public:
         }
 
         QString queryString = p->encodeURL(form->lineEditSearchTerm->text());
-        // FIXME: Number of results doesn't seem to be supported by BibSonomy
         return KUrl("http://www.bibsonomy.org/bib/" + form->comboBoxSearchWhere->itemData(form->comboBoxSearchWhere->currentIndex()).toString() + "/" + queryString + QString("?items=%1").arg(form->numResultsField->value()));
     }
 
@@ -142,11 +139,11 @@ public:
         bool hasAuthor = !query[queryKeyAuthor].isEmpty();
         bool hasYear = !query[queryKeyYear].isEmpty();
 
-        QString searchType = "search";
+        QString searchType = QLatin1String("search");
         if (hasAuthor && !hasFreeText && !hasTitle && !hasYear) {
             /// if only the author field is used, a special author search
             /// on BibSonomy can be used
-            searchType = "author";
+            searchType = QLatin1String("author");
         }
 
         QStringList queryFragments;
@@ -154,8 +151,8 @@ public:
             queryFragments << p->encodeURL(it.value());
         }
 
-        QString queryString = queryFragments.join("%20");
-        url.append(searchType + "/" + queryString + QString("?items=%1").arg(numResults));
+        QString queryString = queryFragments.join(QLatin1String("%20"));
+        url.append(searchType + QLatin1Char('/') + queryString + QString(QLatin1String("?items=%1")).arg(numResults));
 
         return KUrl(url);
     }
@@ -191,7 +188,7 @@ void OnlineSearchBibsonomy::startSearch(const QMap<QString, QString> &query, int
 
     QNetworkRequest request(d->buildQueryUrl(query, numResults));
     QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    setNetworkReplyTimeout(reply);
+    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
     connect(reply, SIGNAL(finished()), this, SLOT(downloadDone()));
 
     emit progress(0, d->numSteps);
@@ -205,7 +202,7 @@ void OnlineSearchBibsonomy::startSearch()
 
     QNetworkRequest request(d->buildQueryUrl());
     QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    setNetworkReplyTimeout(reply);
+    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
     connect(reply, SIGNAL(finished()), this, SLOT(downloadDone()));
 
     emit progress(0, d->numSteps);
@@ -258,15 +255,7 @@ void OnlineSearchBibsonomy::downloadDone()
             if (bibtexFile != NULL) {
                 for (File::ConstIterator it = bibtexFile->constBegin(); it != bibtexFile->constEnd(); ++it) {
                     QSharedPointer<Entry> entry = (*it).dynamicCast<Entry>();
-                    if (!entry.isNull()) {
-                        Value v;
-                        v.append(QSharedPointer<VerbatimText>(new VerbatimText(label())));
-                        entry->insert("x-fetchedfrom", v);
-                        d->sanitizeEntry(entry);
-                        emit foundEntry(entry);
-                        hasEntries = true;
-                    }
-
+                    hasEntries |= publishEntry(entry);
                 }
 
                 if (!hasEntries)
