@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "onlinesearchgooglescholar.h"
 
 #include <QSpinBox>
 #include <QLayout>
@@ -31,15 +30,14 @@
 #include <KLineEdit>
 #include <KIcon>
 
-#include <fileimporterbibtex.h>
-#include <internalnetworkaccessmanager.h>
-#include "onlinesearchgooglescholar.h"
+#include "fileimporterbibtex.h"
+#include "internalnetworkaccessmanager.h"
 
 
 class OnlineSearchGoogleScholar::OnlineSearchGoogleScholarPrivate
 {
 private:
-    OnlineSearchGoogleScholar *p;
+    // UNUSED OnlineSearchGoogleScholar *p;
 
 public:
     int numResults;
@@ -53,8 +51,9 @@ public:
     FileImporterBibTeX importer;
     int numSteps, curStep;
 
-    OnlineSearchGoogleScholarPrivate(OnlineSearchGoogleScholar *parent)
-            : p(parent) {
+    OnlineSearchGoogleScholarPrivate(OnlineSearchGoogleScholar */* UNUSED parent*/)
+    // : UNUSED p(parent)
+    {
         startPageUrl = QLatin1String("http://scholar.google.com/");
         configPageUrl = QLatin1String("http://%1/scholar_settings");
         setConfigPageUrl = QLatin1String("http://%1/scholar_setprefs");
@@ -86,7 +85,7 @@ public:
             }
         }
 
-        return QString::null;
+        return QString();
     }
 
     QString mainUrlForBibTeXEntry(const QString &htmlText, int bibLinkPos) {
@@ -108,7 +107,7 @@ public:
             }
         }
 
-        return QString::null;
+        return QString();
     }
 };
 
@@ -154,7 +153,7 @@ void OnlineSearchGoogleScholar::startSearch(const QMap<QString, QString> &query,
     KUrl url(d->startPageUrl);
     QNetworkRequest request(url);
     QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    setNetworkReplyTimeout(reply);
+    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
     connect(reply, SIGNAL(finished()), this, SLOT(doneFetchingStartPage()));
 
     emit progress(0, d->numSteps);
@@ -173,7 +172,7 @@ void OnlineSearchGoogleScholar::doneFetchingStartPage()
             ++d->numSteps;
             QNetworkRequest request(newDomainUrl);
             QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-            setNetworkReplyTimeout(reply);
+            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
             connect(reply, SIGNAL(finished()), this, SLOT(doneFetchingStartPage()));
         } else {
             /// landed on country-specific domain
@@ -183,7 +182,7 @@ void OnlineSearchGoogleScholar::doneFetchingStartPage()
 
             QNetworkRequest request(url);
             QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply->url());
-            setNetworkReplyTimeout(newReply);
+            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
             connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingConfigPage()));
         }
     } else
@@ -197,12 +196,13 @@ void OnlineSearchGoogleScholar::doneFetchingConfigPage()
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
     if (handleErrors(reply)) {
-        QMap<QString, QString> inputMap = formParameters(reply->readAll(), "<form ");
-        inputMap["hl"] = "en";
-        inputMap["scis"] = "yes";
-        inputMap["scisf"] = "4";
-        inputMap["num"] = QString::number(d->numResults);
-        inputMap["submit"] = "";
+        const QString htmlText = QString::fromUtf8(reply->readAll().data());
+        QMap<QString, QString> inputMap = formParameters(htmlText, "<form ");
+        inputMap[QLatin1String("hl")] = QLatin1String("en");
+        inputMap[QLatin1String("scis")] = QLatin1String("yes");
+        inputMap[QLatin1String("scisf")] = QLatin1String("4");
+        inputMap[QLatin1String("num")] = QString::number(d->numResults);
+        inputMap[QLatin1String("submit")] = QLatin1String("");
 
         KUrl url(d->setConfigPageUrl.arg(reply->url().host()));
         for (QMap<QString, QString>::ConstIterator it = inputMap.constBegin(); it != inputMap.constEnd(); ++it)
@@ -210,7 +210,7 @@ void OnlineSearchGoogleScholar::doneFetchingConfigPage()
 
         QNetworkRequest request(url);
         QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply);
-        setNetworkReplyTimeout(newReply);
+        InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
         connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingSetConfigPage()));
     } else
         kDebug() << "url was" << reply->url().toString();
@@ -224,17 +224,17 @@ void OnlineSearchGoogleScholar::doneFetchingSetConfigPage()
 
     if (handleErrors(reply)) {
         KUrl url(QString(d->queryPageUrl).arg(reply->url().host()));
-        url.addEncodedQueryItem(QString("as_q").toAscii(), d->queryFreetext.toAscii());
-        url.addEncodedQueryItem(QString("as_sauthors").toAscii(), d->queryAuthor.toAscii());
-        url.addEncodedQueryItem(QString("as_ylo").toAscii(), d->queryYear.toAscii());
-        url.addEncodedQueryItem(QString("as_yhi").toAscii(), d->queryYear.toAscii());
-        url.addEncodedQueryItem(QString("as_vis").toAscii(), "1"); ///< include citations
+        url.addEncodedQueryItem(QString("as_q").toLatin1(), d->queryFreetext.toLatin1());
+        url.addEncodedQueryItem(QString("as_sauthors").toLatin1(), d->queryAuthor.toLatin1());
+        url.addEncodedQueryItem(QString("as_ylo").toLatin1(), d->queryYear.toLatin1());
+        url.addEncodedQueryItem(QString("as_yhi").toLatin1(), d->queryYear.toLatin1());
+        url.addEncodedQueryItem(QString("as_vis").toLatin1(), "1"); ///< include citations
         url.addQueryItem("num", QString::number(d->numResults));
         url.addQueryItem("btnG", "Search Scholar");
 
         QNetworkRequest request(url);
         QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply);
-        setNetworkReplyTimeout(newReply);
+        InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
         connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingQueryPage()));
     } else
         kDebug() << "url was" << reply->url().toString();
@@ -247,9 +247,7 @@ void OnlineSearchGoogleScholar::doneFetchingQueryPage()
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
     if (handleErrors(reply)) {
-        QString htmlText = reply->readAll();
-
-        dumpToFile("googlescholar.html", htmlText);
+        const QString htmlText = QString::fromUtf8(reply->readAll().data());
 
         static const QRegExp linkToBib("/scholar.bib\\?[^\" >]+");
         int pos = 0;
@@ -267,7 +265,7 @@ void OnlineSearchGoogleScholar::doneFetchingQueryPage()
 
         if (!d->listBibTeXurls.isEmpty()) {
             const QString bibtexUrl = d->listBibTeXurls.constBegin().key();
-            const QStringList urls = d->listBibTeXurls.constBegin().value().split(QLatin1Char('|'), QString::KeepEmptyParts);
+            const QStringList urls = d->listBibTeXurls.constBegin().value().split(QLatin1String("|"), QString::KeepEmptyParts);
             const QString primaryUrl = urls.first();
             const QString documentUrl = urls.last();
             QNetworkRequest request(bibtexUrl);
@@ -280,7 +278,7 @@ void OnlineSearchGoogleScholar::doneFetchingQueryPage()
                 /// Store URL to document as a property of the request/reply
                 newReply->setProperty("documenturl", QVariant::fromValue<QString>(documentUrl));
             }
-            setNetworkReplyTimeout(newReply);
+            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
             connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingBibTeX()));
             d->listBibTeXurls.erase(d->listBibTeXurls.begin());
         } else {
@@ -334,19 +332,19 @@ void OnlineSearchGoogleScholar::doneFetchingBibTeX()
         }
 
         if (!hasEntry) {
-            kWarning() << "Searching" << label() << "resulted in invalid BibTeX data:" << QString(reply->readAll());
+            kWarning() << "Searching" << label() << "resulted in invalid BibTeX data:" << rawText;
             emit stoppedSearch(resultUnspecifiedError);
             return;
         }
 
         if (!d->listBibTeXurls.isEmpty()) {
             const QString bibtexUrl = d->listBibTeXurls.constBegin().key();
-            const QStringList urls = d->listBibTeXurls.constBegin().value().split(QLatin1Char('|'), QString::KeepEmptyParts);
+            const QStringList urls = d->listBibTeXurls.constBegin().value().split(QLatin1String("|"), QString::KeepEmptyParts);
             const QString primaryUrl = urls.first();
             const QString documentUrl = urls.last();
             QNetworkRequest request(bibtexUrl);
             QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply);
-            setNetworkReplyTimeout(newReply);
+            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
             if (!primaryUrl.isEmpty()) {
                 /// Store primary URL as a property of the request/reply
                 newReply->setProperty("primaryurl", QVariant::fromValue<QString>(primaryUrl));
