@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "onlinesearchingentaconnect.h"
 
 #include <QBuffer>
 #include <QLabel>
@@ -30,11 +29,10 @@
 #include <KLineEdit>
 #include <KConfigGroup>
 
-#include <fileimporterbibtex.h>
-#include <file.h>
-#include <entry.h>
-#include <internalnetworkaccessmanager.h>
-#include "onlinesearchingentaconnect.h"
+#include "file.h"
+#include "entry.h"
+#include "fileimporterbibtex.h"
+#include "internalnetworkaccessmanager.h"
 
 class OnlineSearchIngentaConnect::OnlineSearchQueryFormIngentaConnect : public OnlineSearchQueryFormAbstract
 {
@@ -291,17 +289,6 @@ public:
         queryUrl.addQueryItem(QLatin1String("sortField"), QLatin1String("default"));
         return queryUrl;
     }
-
-    void sanitizeEntry(QSharedPointer<Entry> entry) {
-        /// if entry contains a description field but no abstract,
-        /// rename description field to abstract
-        const QString ftDescription = QLatin1String("description");
-        if (!entry->contains(Entry::ftAbstract) && entry->contains(ftDescription)) {
-            Value v = entry->value(ftDescription);
-            entry->insert(Entry::ftAbstract, v);
-            entry->remove(ftDescription);
-        }
-    }
 };
 
 OnlineSearchIngentaConnect::OnlineSearchIngentaConnect(QWidget *parent)
@@ -321,7 +308,7 @@ void OnlineSearchIngentaConnect::startSearch(const QMap<QString, QString> &query
 
     QNetworkRequest request(d->buildQueryUrl(query, numResults));
     QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    setNetworkReplyTimeout(reply);
+    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
     connect(reply, SIGNAL(finished()), this, SLOT(downloadDone()));
 
     emit progress(0, 1);
@@ -333,7 +320,7 @@ void OnlineSearchIngentaConnect::startSearch()
 
     QNetworkRequest request(d->buildQueryUrl());
     QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    setNetworkReplyTimeout(reply);
+    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
     connect(reply, SIGNAL(finished()), this, SLOT(downloadDone()));
 
     emit progress(0, 1);
@@ -386,15 +373,7 @@ void OnlineSearchIngentaConnect::downloadDone()
             if (bibtexFile != NULL) {
                 for (File::ConstIterator it = bibtexFile->constBegin(); it != bibtexFile->constEnd(); ++it) {
                     QSharedPointer<Entry> entry = (*it).dynamicCast<Entry>();
-                    if (!entry.isNull()) {
-                        Value v;
-                        v.append(QSharedPointer<VerbatimText>(new VerbatimText(label())));
-                        entry->insert("x-fetchedfrom", v);
-                        d->sanitizeEntry(entry);
-                        emit foundEntry(entry);
-                        hasEntries = true;
-                    }
-
+                    hasEntries |= publishEntry(entry);
                 }
 
                 if (!hasEntries)
