@@ -1,27 +1,24 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
-
-#include <KLocale>
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
 
 #include "onlinesearchinspirehep.h"
 
+#include <KLocale>
+
 OnlineSearchInspireHep::OnlineSearchInspireHep(QWidget *parent)
         : OnlineSearchSimpleBibTeXDownload(parent)
 {
@@ -30,7 +27,7 @@ OnlineSearchInspireHep::OnlineSearchInspireHep(QWidget *parent)
 
 QString OnlineSearchInspireHep::label() const
 {
-    return i18n("Inpsire High-Energy Physics Literature Database");
+    return i18n("Inspire High-Energy Physics Literature Database");
 }
 
 OnlineSearchQueryFormAbstract *OnlineSearchInspireHep::customWidget(QWidget *parent)
@@ -82,9 +79,9 @@ KUrl OnlineSearchInspireHep::buildQueryUrl(const QMap<QString, QString> &query,
     urlText.append(QString(QLatin1String("&rg=%1")).arg(numResults));
     /// Append actual query
     urlText.append(QLatin1String("&p="));
-    urlText.append(queryFragments.join(" and "));
+    urlText.append(queryFragments.join(QLatin1String(" and ")));
     /// URL-encode text
-    urlText = urlText.replace(" ", "%20").replace("\"", "%22");
+    urlText = urlText.replace(QLatin1Char(' '), QLatin1String("%20")).replace(QLatin1Char('"'), QLatin1String("%22"));
 
     return KUrl(urlText);
 }
