@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "onlinesearchjstor.h"
 
 #include <QNetworkRequest>
 #include <QNetworkReply>
@@ -24,82 +23,24 @@
 #include <KLocale>
 #include <KDebug>
 
-#include <internalnetworkaccessmanager.h>
+#include "internalnetworkaccessmanager.h"
 #include "iocommon.h"
 #include "fileimporterbibtex.h"
-#include "onlinesearchjstor.h"
 
 class OnlineSearchJStor::OnlineSearchJStorPrivate
 {
 private:
-    OnlineSearchJStor *p;
+    // UNUSED OnlineSearchJStor *p;
 
 public:
     int numFoundResults, curStep, numSteps;
     static const QString jstorBaseUrl;
     KUrl queryUrl;
 
-    OnlineSearchJStorPrivate(OnlineSearchJStor *parent)
-            : p(parent) {
-        // nothing
-    }
-
-    void sanitizeEntry(QSharedPointer<Entry> entry) {
-        if (KBibTeX::doiRegExp.indexIn(entry->id()) == 0) {
-            /// entry ID is a DOI
-            Value v;
-            v.append(QSharedPointer<VerbatimText>(new VerbatimText(KBibTeX::doiRegExp.cap(0))));
-            entry->insert(Entry::ftDOI, v);
-        }
-
-        QString url = PlainTextValue::text(entry->value(Entry::ftUrl));
-        if (url.startsWith("http://www.jstor.org/stable/")) {
-            /// use JSTOR's own stable IDs for entry ID
-            entry->setId("jstor" + url.mid(28).replace(QLatin1Char(','), QString()));
-        }
-
-        const QString formattedDateKey = "jstor_formatteddate";
-        const QString formattedDate = PlainTextValue::text(entry->value(formattedDateKey));
-
-        /// first, try to guess month by inspecting the beginning
-        /// the jstor_formatteddate field
-        const QString formattedDateLower = formattedDate.toLower();
-        int i;
-        for (i = 0; i < 12; ++i)
-            if (formattedDateLower.startsWith(MonthsTriple[i])) break;
-        entry->remove(formattedDateKey);
-        if (i < 12) {
-            Value v;
-            v.append(QSharedPointer<MacroKey>(new MacroKey(MonthsTriple[i])));
-            entry->insert(Entry::ftMonth, v);
-        }
-        /// guessing failed, therefore extract first part if it exists
-        else if ((i = formattedDate.indexOf(",")) >= 0) {
-            /// text may be a season ("Winter")
-            Value v;
-            v.append(QSharedPointer<PlainText>(new PlainText(formattedDate.left(i))));
-            entry->insert(Entry::ftMonth, v);
-        } else {
-            /// this case happens if the field only contains a year
-            kDebug() << "Cannot extract month/season from date" << formattedDate;
-        }
-
-        /// page field may start with "pp. ", remove that
-        QString pages = PlainTextValue::text(entry->value(Entry::ftPages)).toLower();
-        if (pages.startsWith("pp. ")) {
-            pages = pages.mid(4);
-            entry->remove(Entry::ftPages);
-            Value v;
-            v.append(QSharedPointer<PlainText>(new PlainText(pages)));
-            entry->insert(Entry::ftPages, v);
-        }
-
-        for (QMap<QString, Value>::Iterator it = entry->begin(); it != entry->end();) {
-            if (PlainTextValue::text(it.value()).isEmpty())
-                it = entry->erase(it);
-            else
-                ++it;
-        }
+    OnlineSearchJStorPrivate(OnlineSearchJStor */* UNUSED parent*/)
+    // : UNUSED p(parent)
+    {
+        /// nothing
     }
 };
 
@@ -108,7 +49,7 @@ const QString OnlineSearchJStor::OnlineSearchJStorPrivate::jstorBaseUrl = QLatin
 OnlineSearchJStor::OnlineSearchJStor(QWidget *parent)
         : OnlineSearchAbstract(parent), d(new OnlineSearchJStorPrivate(this))
 {
-    // nothing
+    /// nothing
 }
 
 OnlineSearchJStor::~OnlineSearchJStor()
@@ -162,7 +103,7 @@ void OnlineSearchJStor::startSearch(const QMap<QString, QString> &query, int num
 
     QNetworkRequest request(d->jstorBaseUrl);
     QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    setNetworkReplyTimeout(reply);
+    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
     connect(reply, SIGNAL(finished()), this, SLOT(doneFetchingStartPage()));
     emit progress(d->curStep, d->numSteps);
 }
@@ -212,12 +153,12 @@ void OnlineSearchJStor::doneFetchingStartPage()
             /// redirection to another url
             QNetworkRequest request(redirUrl);
             QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply->url());
-            setNetworkReplyTimeout(newReply);
+            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
             connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingStartPage()));
         } else {
             QNetworkRequest request(d->queryUrl);
             QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request);
-            setNetworkReplyTimeout(newReply);
+            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
             connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingResultPage()));
         }
     } else
@@ -258,7 +199,7 @@ void OnlineSearchJStor::doneFetchingResultPage()
 
             QNetworkRequest request(bibTeXUrl);
             QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request);
-            setNetworkReplyTimeout(newReply);
+            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
             connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingBibTeXCode()));
         }
     } else
@@ -281,16 +222,9 @@ void OnlineSearchJStor::doneFetchingBibTeXCode()
         if (bibtexFile != NULL) {
             for (File::ConstIterator it = bibtexFile->constBegin(); it != bibtexFile->constEnd(); ++it) {
                 QSharedPointer<Entry> entry = (*it).dynamicCast<Entry>();
-                if (!entry.isNull() && !entry->id().isEmpty() /** skip invalid entries with empty ID */) {
-                    Value v;
-                    v.append(QSharedPointer<VerbatimText>(new VerbatimText(label())));
-                    entry->insert("x-fetchedfrom", v);
-
-                    d->sanitizeEntry(entry);
-                    emit foundEntry(entry);
-
+                if (publishEntry(entry))
                     ++d->numFoundResults;
-                }
+
             }
             delete bibtexFile;
         }
@@ -301,3 +235,68 @@ void OnlineSearchJStor::doneFetchingBibTeXCode()
         kDebug() << "url was" << reply->url().toString();
 }
 
+void OnlineSearchJStor::sanitizeEntry(QSharedPointer<Entry> entry)
+{
+    OnlineSearchAbstract::sanitizeEntry(entry);
+
+    if (KBibTeX::doiRegExp.indexIn(entry->id()) == 0) {
+        /// entry ID is a DOI
+        Value v;
+        v.append(QSharedPointer<VerbatimText>(new VerbatimText(KBibTeX::doiRegExp.cap(0))));
+        entry->insert(Entry::ftDOI, v);
+    }
+
+    QString url = PlainTextValue::text(entry->value(Entry::ftUrl));
+    if (url.startsWith(QLatin1String("http://www.jstor.org/stable/"))) {
+        /// use JSTOR's own stable ID for entry ID
+        entry->setId("jstor" + url.mid(28).replace(QLatin1Char(','), QString()));
+        /// store JSTOR's own stable ID
+        Value v;
+        v.append(QSharedPointer<VerbatimText>(new VerbatimText(url.mid(28))));
+        entry->insert(QLatin1String("jstor_id"), v);
+    }
+
+    const QString formattedDateKey = "jstor_formatteddate";
+    const QString formattedDate = PlainTextValue::text(entry->value(formattedDateKey));
+
+    /// first, try to guess month by inspecting the beginning
+    /// the jstor_formatteddate field
+    const QString formattedDateLower = formattedDate.toLower();
+    int i;
+    for (i = 0; i < 12; ++i)
+        if (formattedDateLower.startsWith(MonthsTriple[i])) break;
+    entry->remove(formattedDateKey);
+    if (i < 12) {
+        Value v;
+        v.append(QSharedPointer<MacroKey>(new MacroKey(MonthsTriple[i])));
+        entry->insert(Entry::ftMonth, v);
+    }
+    /// guessing failed, therefore extract first part if it exists
+    else if ((i = formattedDate.indexOf(",")) >= 0) {
+        /// text may be a season ("Winter")
+        Value v;
+        v.append(QSharedPointer<PlainText>(new PlainText(formattedDate.left(i))));
+        entry->insert(Entry::ftMonth, v);
+    } else {
+        /// this case happens if the field only contains a year
+        kDebug() << "Cannot extract month/season from date" << formattedDate;
+    }
+
+    /// page field may start with "pp. ", remove that
+    QString pages = PlainTextValue::text(entry->value(Entry::ftPages)).toLower();
+    if (pages.startsWith(QLatin1String("pp. "))) {
+        pages = pages.mid(4);
+        entry->remove(Entry::ftPages);
+        Value v;
+        v.append(QSharedPointer<PlainText>(new PlainText(pages)));
+        entry->insert(Entry::ftPages, v);
+    }
+
+    for (QMap<QString, Value>::Iterator it = entry->begin(); it != entry->end();) {
+        if (PlainTextValue::text(it.value()).isEmpty())
+            it = entry->erase(it);
+        else
+            ++it;
+    }
+}
+
