@@ -1,37 +1,35 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "onlinesearchsciencedirect.h"
 
 #include <QNetworkReply>
 
 #include <KDebug>
 #include <KLocale>
 
-#include <encoderlatex.h>
+#include "encoderlatex.h"
 #include "fileimporterbibtex.h"
-#include "onlinesearchsciencedirect.h"
-#include <internalnetworkaccessmanager.h>
+#include "internalnetworkaccessmanager.h"
 
 class OnlineSearchScienceDirect::OnlineSearchScienceDirectPrivate
 {
 private:
-    OnlineSearchScienceDirect *p;
+    // UNUSED OnlineSearchScienceDirect *p;
 
 public:
     QString queryFreetext, queryAuthor;
@@ -42,9 +40,9 @@ public:
     int runningJobs;
     int numSteps, curStep;
 
-    OnlineSearchScienceDirectPrivate(OnlineSearchScienceDirect *parent)
-            : p(parent), scienceDirectBaseUrl(QLatin1String("http://www.sciencedirect.com/")) {
-        // nothing
+    OnlineSearchScienceDirectPrivate(OnlineSearchScienceDirect */* UNUSED parent*/)
+        : /* UNUSED p(parent), */scienceDirectBaseUrl(QLatin1String("http://www.sciencedirect.com/")) {
+        /// nothing
     }
 
     void sanitizeBibTeXCode(QString &code) {
@@ -54,6 +52,19 @@ public:
         while ((p = quotationMarks.indexIn(code, p + 2)) >= 0) {
             code = code.left(p - 1) + quotationMarks.cap(1) + '\\' + '"' + quotationMarks.cap(2) + code.mid(p + quotationMarks.cap(0).length());
         }
+        /// Remove "<!-- Tag Not Handled -->" and other XML tags from keywords
+        int i1 = -1;
+        while ((i1 = code.indexOf(QLatin1String("keywords = "), i1 + 1)) > 0) {
+            const int i2 = code.indexOf(QLatin1String("\n"), i1);
+            int t1 = -1;
+            while ((t1 = code.indexOf(QLatin1Char('<'), i1 + 7)) > 0) {
+                const int t2 = code.indexOf(QLatin1Char('>'), t1);
+                if (t2 > 0 && t1 < i2 && t2 < i2)
+                    code = code.left(t1) + code.mid(t2 + 1);
+                else
+                    break;
+            }
+        }
     }
 };
 
@@ -83,7 +94,7 @@ void OnlineSearchScienceDirect::startSearch(const QMap<QString, QString> &query,
     d->bibTeXUrls.clear();
     d->currentSearchPosition = 0;
     d->curStep = 0;
-    d->numSteps = 2 + 3 * numResults;
+    d->numSteps = 2 + 2 * numResults;
 
     d->queryFreetext = query[queryKeyFreeText] + ' ' + query[queryKeyTitle] + ' ' + query[queryKeyYear];
     d->queryAuthor = query[queryKeyAuthor];
@@ -92,7 +103,7 @@ void OnlineSearchScienceDirect::startSearch(const QMap<QString, QString> &query,
     ++d->runningJobs;
     QNetworkRequest request(d->scienceDirectBaseUrl);
     QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    setNetworkReplyTimeout(reply);
+    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
     connect(reply, SIGNAL(finished()), this, SLOT(doneFetchingStartPage()));
 
     emit progress(0, d->numSteps);
@@ -134,7 +145,7 @@ void OnlineSearchScienceDirect::doneFetchingStartPage()
     KUrl redirUrl;
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
     if (handleErrors(reply, redirUrl)) {
-        const QString htmlText = reply->readAll();
+        const QString htmlText = QString::fromUtf8(reply->readAll().data());
 
         if (redirUrl.isValid()) {
             ++d->numSteps;
@@ -143,21 +154,21 @@ void OnlineSearchScienceDirect::doneFetchingStartPage()
             /// redirection to another url
             QNetworkRequest request(redirUrl);
             QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply->url());
-            setNetworkReplyTimeout(newReply);
+            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
             connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingStartPage()));
         } else {
             InternalNetworkAccessManager::self()->mergeHtmlHeadCookies(htmlText, reply->url());
 
-            KUrl url(d->scienceDirectBaseUrl + "science");
-            QMap<QString, QString> inputMap = formParameters(htmlText, QLatin1String("<form name=\"qkSrch\""));
-            inputMap["qs_all"] = d->queryFreetext.simplified();
-            inputMap["qs_author"] = d->queryAuthor.simplified();
-            inputMap["resultsPerPage"] = QString::number(d->numExpectedResults);
-            inputMap["_ob"] = "QuickSearchURL";
-            inputMap["_method"] = "submitForm";
-            inputMap["sdSearch"] = "Search";
+            KUrl url(d->scienceDirectBaseUrl + QLatin1String("science"));
+            QMap<QString, QString> inputMap = formParameters(htmlText, QLatin1String("<form id=\"quickSearch\" name=\"qkSrch\" metho"));
+            inputMap[QLatin1String("qs_all")] = d->queryFreetext.simplified();
+            inputMap[QLatin1String("qs_author")] = d->queryAuthor.simplified();
+            inputMap[QLatin1String("resultsPerPage")] = QString::number(d->numExpectedResults);
+            inputMap[QLatin1String("_ob")] = QLatin1String("QuickSearchURL");
+            inputMap[QLatin1String("_method")] = QLatin1String("submitForm");
+            inputMap[QLatin1String("sdSearch")] = QLatin1String("Search");
 
-            static const QStringList orderOfParameters = QString("_ob|_method|_acct|_origin|_zone|md5|_eidkey|qs_issue|qs_pages|qs_title|qs_vol|sdSearch|qs_all|qs_author|resultsPerPage").split("|");
+            static const QStringList orderOfParameters = QString(QLatin1String("_ob|_method|_acct|_origin|_zone|md5|_eidkey|qs_issue|qs_pages|qs_title|qs_vol|sdSearch|qs_all|qs_author|resultsPerPage")).split(QLatin1String("|"));
             foreach(const QString &key, orderOfParameters) {
                 if (!inputMap.contains(key)) continue;
                 url.addQueryItem(key, inputMap[key]);
@@ -167,7 +178,7 @@ void OnlineSearchScienceDirect::doneFetchingStartPage()
             QNetworkRequest request(url);
             QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply);
             connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingResultPage()));
-            setNetworkReplyTimeout(newReply);
+            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
         }
     } else
         kDebug() << "url was" << reply->url().toString();
@@ -187,24 +198,30 @@ void OnlineSearchScienceDirect::doneFetchingResultPage()
             QNetworkRequest request(redirUrl);
             QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply);
             connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingResultPage()));
-            setNetworkReplyTimeout(newReply);
+            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
         } else {
             emit progress(++d->curStep, d->numSteps);
 
-            const QString htmlText = reply->readAll();
+            const QString htmlText = QString::fromUtf8(reply->readAll().data());
             InternalNetworkAccessManager::self()->mergeHtmlHeadCookies(htmlText, reply->url());
 
+            QSet<QString> knownUrls;
             int p = -1, p2 = -1;
-            while ((p = htmlText.indexOf("http://www.sciencedirect.com/science/article/pii/", p + 1)) >= 0 && (p2 = htmlText.indexOf("\"", p + 1)) >= 0)
+            while ((p = htmlText.indexOf("http://www.sciencedirect.com/science/article/pii/", p + 1)) >= 0 && (p2 = htmlText.indexOf(QRegExp("[\"/ #]"), p + 50)) >= 0) {
+                const QString urlText = htmlText.mid(p, p2 - p);
+                if (knownUrls.contains(urlText)) continue;
+                knownUrls.insert(urlText);
+
                 if (d->numFoundResults < d->numExpectedResults) {
                     ++d->numFoundResults;
                     ++d->runningJobs;
-                    KUrl url(htmlText.mid(p, p2 - p));
+                    KUrl url(urlText);
                     QNetworkRequest request(url);
                     QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply);
-                    setNetworkReplyTimeout(newReply);
+                    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
                     connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingAbstractPage()));
                 }
+            }
         }
 
         if (d->runningJobs <= 0) {
@@ -229,21 +246,24 @@ void OnlineSearchScienceDirect::doneFetchingAbstractPage()
             QNetworkRequest request(redirUrl);
             QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply);
             connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingAbstractPage()));
-            setNetworkReplyTimeout(newReply);
+            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
         } else {
             emit progress(++d->curStep, d->numSteps);
 
-            const QString htmlText = reply->readAll();
+            const QString htmlText = QString::fromUtf8(reply->readAll().data());
             InternalNetworkAccessManager::self()->mergeHtmlHeadCookies(htmlText, reply->url());
 
             int p1 = -1, p2 = -1;
-            if ((p1 = htmlText.indexOf("/science?_ob=DownloadURL&")) >= 0 && (p2 = htmlText.indexOf("\"", p1 + 1)) >= 0) {
+            if ((p1 = htmlText.indexOf("/science?_ob=DownloadURL&")) >= 0 && (p2 = htmlText.indexOf(QRegExp("[ \"<>]"), p1 + 1)) >= 0) {
                 KUrl url("http://www.sciencedirect.com" + htmlText.mid(p1, p2 - p1));
+                url.addQueryItem(QLatin1String("citation-type"), QLatin1String("BIBTEX"));
+                url.addQueryItem(QLatin1String("format"), QLatin1String("cite-abs"));
+                url.addQueryItem(QLatin1String("export"), QLatin1String("Export"));
                 ++d->runningJobs;
                 QNetworkRequest request(url);
                 QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply);
-                connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingExportCitationPage()));
-                setNetworkReplyTimeout(newReply);
+                connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingBibTeX()));
+                InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
             }
         }
 
@@ -255,56 +275,6 @@ void OnlineSearchScienceDirect::doneFetchingAbstractPage()
         kDebug() << "url was" << reply->url().toString();
 }
 
-void OnlineSearchScienceDirect::doneFetchingExportCitationPage()
-{
-    --d->runningJobs;
-    if (d->runningJobs < 0)
-        kWarning() << "In OnlineSearchScienceDirect::doneFetchingAbstractPage: Counting jobs failed (" << d->runningJobs << "< 0 )";
-
-    QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
-    if (handleErrors(reply)) {
-        KUrl redirUrl = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
-        if (!redirUrl.isEmpty()) {
-            ++d->runningJobs;
-            QNetworkRequest request(redirUrl);
-            QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply);
-            connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingExportCitationPage()));
-            setNetworkReplyTimeout(newReply);
-        } else {
-            emit progress(++d->curStep, d->numSteps);
-
-            const QString htmlText = reply->readAll();
-            InternalNetworkAccessManager::self()->mergeHtmlHeadCookies(htmlText, reply->url());
-
-            QMap<QString, QString> inputMap = formParameters(htmlText, QLatin1String("<form name=\"exportCite\""));
-            inputMap["format"] = "cite";
-            inputMap["citation-type"] = "BIBTEX";
-            inputMap["RETURN_URL"] = d->scienceDirectBaseUrl + "/science/home";
-            static const QStringList orderOfParameters = QString("_ob|_method|_acct|_userid|_docType|_eidkey|_ArticleListID|_uoikey|count|md5|JAVASCRIPT_ON|format|citation-type|Export|RETURN_URL").split("|");
-
-            QString body;
-            foreach(const QString &key, orderOfParameters) {
-                if (!inputMap.contains(key)) continue;
-                if (!body.isEmpty()) body += '&';
-                body += encodeURL(key) + '=' + encodeURL(inputMap[key]);
-            }
-
-            ++d->runningJobs;
-            QNetworkRequest request(KUrl(d->scienceDirectBaseUrl + "/science"));
-            request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
-            QNetworkReply *newReply = InternalNetworkAccessManager::self()->post(request, body.toUtf8());
-            setNetworkReplyTimeout(newReply);
-            connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingBibTeX()));
-        }
-
-        if (d->runningJobs <= 0) {
-            emit stoppedSearch(resultNoError);
-            emit progress(d->numSteps, d->numSteps);
-        }
-    } else
-        kDebug() << "url was" << reply->url().toString();
-}
-
 void OnlineSearchScienceDirect::doneFetchingBibTeX()
 {
     emit progress(++d->curStep, d->numSteps);
@@ -326,13 +296,8 @@ void OnlineSearchScienceDirect::doneFetchingBibTeX()
         if (bibtexFile != NULL) {
             for (File::ConstIterator it = bibtexFile->constBegin(); it != bibtexFile->constEnd(); ++it) {
                 QSharedPointer<Entry> entry = (*it).dynamicCast<Entry>();
-                if (!entry.isNull()) {
-                    hasEntry = true;
-                    Value v;
-                    v.append(QSharedPointer<VerbatimText>(new VerbatimText(label())));
-                    entry->insert("x-fetchedfrom", v);
-                    emit foundEntry(entry);
-                }
+                kDebug() << "entry's title=" << PlainTextValue::text(entry->value(Entry::ftTitle));
+                hasEntry |= publishEntry(entry);
             }
             delete bibtexFile;
         }
