@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "onlinesearchspringerlink.h"
 
 #include <QFile>
 #include <QFormLayout>
@@ -32,11 +31,10 @@
 #include <KConfigGroup>
 #include <KStandardDirs>
 
-#include <internalnetworkaccessmanager.h>
+#include "internalnetworkaccessmanager.h"
 #include "encoderlatex.h"
 #include "fileimporterbibtex.h"
 #include "xsltransform.h"
-#include "onlinesearchspringerlink.h"
 
 /**
  * @author Thomas Fischer <fischer@unix-ag.uni-kl.de>
@@ -149,7 +147,10 @@ public:
 
     OnlineSearchSpringerLinkPrivate(OnlineSearchSpringerLink *parent)
             : p(parent), springerMetadataKey(QLatin1String("7pphfmtb9rtwt3dw3e4hm7av")), form(NULL) {
-        xslt = XSLTransform::createXSLTransform(KStandardDirs::locate("data", "kbibtex/pam2bibtex.xsl"));
+        const QString xsltFilename = QLatin1String("kbibtex/pam2bibtex.xsl");
+        xslt = XSLTransform::createXSLTransform(KStandardDirs::locate("data", xsltFilename));
+        if (xslt == NULL)
+            kWarning() << "Could not create XSLT transformation for" << xsltFilename;
     }
 
     ~OnlineSearchSpringerLinkPrivate() {
@@ -164,17 +165,17 @@ public:
         QString queryString = form->lineEditFreeText->text();
 
         QStringList titleChunks = p->splitRespectingQuotationMarks(form->lineEditTitle->text());
-        foreach(const QString &titleChunk, titleChunks) {
+        foreach (const QString &titleChunk, titleChunks) {
             queryString += QString(QLatin1String(" title:%1")).arg(EncoderLaTeX::instance()->convertToPlainAscii(titleChunk));
         }
 
         titleChunks = p->splitRespectingQuotationMarks(form->lineEditBookTitle->text());
-        foreach(const QString &titleChunk, titleChunks) {
+        foreach (const QString &titleChunk, titleChunks) {
             queryString += QString(QLatin1String(" ( journal:%1 OR book:%1 )")).arg(EncoderLaTeX::instance()->convertToPlainAscii(titleChunk));
         }
 
         QStringList authors = p->splitRespectingQuotationMarks(form->lineEditAuthorEditor->text());
-        foreach(const QString &author, authors) {
+        foreach (const QString &author, authors) {
             queryString += QString(QLatin1String(" name:%1")).arg(EncoderLaTeX::instance()->convertToPlainAscii(author));
         }
 
@@ -194,12 +195,12 @@ public:
         QString queryString = query[queryKeyFreeText];
 
         QStringList titleChunks = p->splitRespectingQuotationMarks(query[queryKeyTitle]);
-        foreach(const QString &titleChunk, titleChunks) {
+        foreach (const QString &titleChunk, titleChunks) {
             queryString += QString(QLatin1String(" title:%1")).arg(EncoderLaTeX::instance()->convertToPlainAscii(titleChunk));
         }
 
         QStringList authors = p->splitRespectingQuotationMarks(query[queryKeyAuthor]);
-        foreach(const QString &author, authors) {
+        foreach (const QString &author, authors) {
             queryString += QString(QLatin1String(" name:%1")).arg(EncoderLaTeX::instance()->convertToPlainAscii(author));
         }
 
@@ -233,6 +234,13 @@ OnlineSearchSpringerLink::~OnlineSearchSpringerLink()
 
 void OnlineSearchSpringerLink::startSearch()
 {
+    if (d->xslt == NULL) {
+        /// Don't allow searches if xslt is not defined
+        kWarning() << "Cannot allow searching" << label() << "if XSL Transformation not available";
+        delayedStoppedSearch(resultUnspecifiedError);
+        return;
+    }
+
     m_hasBeenCanceled = false;
 
     KUrl springerLinkSearchUrl = d->buildQueryUrl();
@@ -240,7 +248,7 @@ void OnlineSearchSpringerLink::startSearch()
     emit progress(0, 1);
     QNetworkRequest request(springerLinkSearchUrl);
     QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    setNetworkReplyTimeout(reply);
+    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
     connect(reply, SIGNAL(finished()), this, SLOT(doneFetchingPAM()));
 
     if (d->form != NULL) d->form->saveState();
@@ -248,6 +256,13 @@ void OnlineSearchSpringerLink::startSearch()
 
 void OnlineSearchSpringerLink::startSearch(const QMap<QString, QString> &query, int numResults)
 {
+    if (d->xslt == NULL) {
+        /// Don't allow searches if xslt is not defined
+        kWarning() << "Cannot allow searching" << label() << "if XSL Transformation not available";
+        delayedStoppedSearch(resultUnspecifiedError);
+        return;
+    }
+
     m_hasBeenCanceled = false;
 
     KUrl springerLinkSearchUrl = d->buildQueryUrl(query);
@@ -256,7 +271,7 @@ void OnlineSearchSpringerLink::startSearch(const QMap<QString, QString> &query,
     emit progress(0, 1);
     QNetworkRequest request(springerLinkSearchUrl);
     QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    setNetworkReplyTimeout(reply);
+    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
     connect(reply, SIGNAL(finished()), this, SLOT(doneFetchingPAM()));
 }
 
@@ -302,16 +317,9 @@ void OnlineSearchSpringerLink::doneFetchingPAM()
 
         bool hasEntries = false;
         if (bibtexFile != NULL) {
-            foreach(const QSharedPointer<Element> &element, *bibtexFile) {
+            foreach (const QSharedPointer<Element> &element, *bibtexFile) {
                 QSharedPointer<Entry> entry = element.dynamicCast<Entry>();
-                if (!entry.isNull()) {
-                    Value v;
-                    v.append(QSharedPointer<VerbatimText>(new VerbatimText(label())));
-                    entry->insert("x-fetchedfrom", v);
-                    emit foundEntry(entry);
-                    hasEntries = true;
-                }
-
+                hasEntries |= publishEntry(entry);
             }
 
             if (!hasEntries)
