@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "findduplicates.h"
 
 #include <typeinfo>
 
@@ -29,7 +28,6 @@
 
 #include "file.h"
 #include "entry.h"
-#include "findduplicates.h"
 
 EntryClique::EntryClique()
 {
@@ -202,7 +200,7 @@ void EntryClique::insertKeyValueToValueMap(const QString &fieldName, const Value
 class FindDuplicates::FindDuplicatesPrivate
 {
 private:
-    FindDuplicates *p;
+    // UNUSED FindDuplicates *p;
     const unsigned int maxDistance;
     int **d;
     static const int dsize;
@@ -212,8 +210,8 @@ public:
     int sensitivity;
     QWidget *widget;
 
-    FindDuplicatesPrivate(FindDuplicates *parent, int sens, QWidget *w)
-            : p(parent), maxDistance(10000), gotCanceled(false), sensitivity(sens), widget(w) {
+    FindDuplicatesPrivate(FindDuplicates */* UNUSED parent*/, int sens, QWidget *w)
+        : /* UNUSED p(parent),*/ maxDistance(10000), gotCanceled(false), sensitivity(sens), widget(w) {
         d = new int*[dsize];
         for (int i = 0; i < dsize; ++i)
             d[i] = new int[dsize];
@@ -381,7 +379,7 @@ bool FindDuplicates::findDuplicateEntries(File *file, QList<EntryClique *> &entr
         KApplication::setOverrideCursor(Qt::WaitCursor);
         progressDlg = new KProgressDialog(d->widget, i18n("Finding Duplicates"));
         progressDlg->setModal(true);
-        progressDlg->setLabelText(i18n("Searching ..."));
+        progressDlg->setLabelText(i18n("Searching..."));
         progressDlg->setMinimumWidth(d->widget->fontMetrics().averageCharWidth() * 48);
         progressDlg->setAllowCancel(true);
         connect(progressDlg, SIGNAL(cancelClicked()), this, SLOT(gotCanceled()));
@@ -413,7 +411,7 @@ bool FindDuplicates::findDuplicateEntries(File *file, QList<EntryClique *> &entr
     if (d->widget != NULL) {
         progressDlg->progressBar()->setMaximum(maxProgress);
         progressDlg->show();
-        progressDlg->setLabelText(i18n("Searching ..."));
+        progressDlg->setLabelText(i18n("Searching..."));
     }
 
     emit maximumProgress(maxProgress);
@@ -508,13 +506,13 @@ void FindDuplicates::gotCanceled()
 class MergeDuplicates::MergeDuplicatesPrivate
 {
 private:
-    MergeDuplicates *p;
+    // UNUSED MergeDuplicates *p;
 
 public:
     QWidget *widget;
 
-    MergeDuplicatesPrivate(MergeDuplicates *parent, QWidget *w)
-            : p(parent), widget(w) {
+    MergeDuplicatesPrivate(MergeDuplicates */* UNUSED parent*/, QWidget *w)
+        : /* UNUSED p(parent),*/ widget(w) {
         // nothing
     }
 };
@@ -534,7 +532,7 @@ bool MergeDuplicates::mergeDuplicateEntries(const QList<EntryClique *> &entryCli
         /// which have been remove (not added) 10 lines below
         QSet<QString> coveredFields;
 
-        Entry *mergedEntry = new Entry(QString::null, QString::null);
+        Entry *mergedEntry = new Entry(QString(), QString());
         foreach(const QString &field, entryClique->fieldList()) {
             coveredFields << field;
             if (field == QLatin1String("^id"))
@@ -570,6 +568,7 @@ bool MergeDuplicates::mergeDuplicateEntries(const QList<EntryClique *> &entryCli
                         mergedEntry->insert(it.key(), it.value());
                         coveredFields << it.key();
                     }
+                // TODO needs to be rewritten if File shall be based on QVector instead of QList
                 file->removeOne(QSharedPointer<Entry>(entry));
             }
         }
@@ -590,8 +589,8 @@ bool MergeDuplicates::mergeDuplicateEntriesAuto(const QList<EntryClique *> &entr
      * Same procedure for each clique ...
      */
     foreach(EntryClique *entryClique, entryCliques) {
-        /// Create a new entry which will eventually replace the clique's Entrys
-        Entry *mergedEntry = new Entry(QString::null, QString::null);
+        /// Create a new entry which will eventually replace the clique's Entries
+        Entry *mergedEntry = new Entry(QString(), QString());
 
         /// Sort entries in clique according into sortedEntries
         /// according to values in field sortCriteriumField and
@@ -604,7 +603,7 @@ bool MergeDuplicates::mergeDuplicateEntriesAuto(const QList<EntryClique *> &entr
             else {
                 /// Get string, number and date based on current,
                 /// unsorted Entry
-                const QString a = PlainTextValue::text(entry->value(sortCriteriumField), file);
+                const QString a = PlainTextValue::text(entry->value(sortCriteriumField));
                 const int ai = a.toInt();
                 const QDate aDate = QDate::fromString(a, Qt::ISODate);
 
@@ -612,7 +611,7 @@ bool MergeDuplicates::mergeDuplicateEntriesAuto(const QList<EntryClique *> &entr
                 for (it = sortedEntries.begin(); it != sortedEntries.end(); ++it) {
                     /// Get string, number and date based on iterator's
                     /// current Entry in sortedEntries list
-                    const QString b = PlainTextValue::text((*it)->value(sortCriteriumField), file);
+                    const QString b = PlainTextValue::text((*it)->value(sortCriteriumField));
                     const int bi = b.toInt();
                     const QDate bDate = QDate::fromString(b, Qt::ISODate);
 
@@ -682,7 +681,7 @@ bool MergeDuplicates::mergeDuplicateEntriesAuto(const QList<EntryClique *> &entr
                     mergedEntry->insert(it.key(), it.value());
 
             /// Removed harvested Entries
-            // TODO does this still work?
+            // TODO needs to be rewritten if File shall be based on QVector instead of QList
             file->removeOne(QSharedPointer<Entry>(entry));
         }
 
