@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "documentpreview.h"
 
 #include <typeinfo>
 
@@ -29,7 +28,7 @@
 #include <QStackedWidget>
 #include <QDockWidget>
 #include <QMutex>
-#ifdef HAVE_QTWEBKIT
+#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
 #include <QWebView>
 #endif // HAVE_QTWEBKIT
 
@@ -44,7 +43,6 @@
 #include <KDebug>
 #include <kio/netaccess.h>
 #include <KPushButton>
-#include <KDebug>
 #include <kio/jobclasses.h>
 #include <kio/job.h>
 #include <kio/jobuidelegate.h>
@@ -53,12 +51,11 @@
 #include <KToolBar>
 #include <KActionCollection>
 
-#include <kbibtexnamespace.h>
-#include <element.h>
-#include <entry.h>
-#include <file.h>
-#include <fileinfo.h>
-#include "documentpreview.h"
+#include "kbibtexnamespace.h"
+#include "element.h"
+#include "entry.h"
+#include "file.h"
+#include "fileinfo.h"
 
 ImageLabel::ImageLabel(const QString &text, QWidget *parent, Qt::WindowFlags f)
         : QLabel(text, parent, f)
@@ -119,7 +116,7 @@ private:
     KMenuBar *menuBar;
     KToolBar *toolBar;
     KParts::ReadOnlyPart *okularPart;
-#ifdef HAVE_QTWEBKIT
+#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
     QWebView *htmlWidget;
 #else // HAVE_QTWEBKIT
     KParts::ReadOnlyPart *htmlPart;
@@ -165,7 +162,7 @@ public:
         QHBoxLayout *innerLayout = new QHBoxLayout();
         layout->addLayout(innerLayout, 0);
 
-        onlyLocalFilesButton = new KPushButton(KIcon("applications-internet"), QString::null, p);
+        onlyLocalFilesButton = new KPushButton(KIcon("applications-internet"), QString(), p);
         onlyLocalFilesButton->setToolTip(i18n("Toggle between local files only and all documents including remote ones"));
         innerLayout->addWidget(onlyLocalFilesButton, 0);
         onlyLocalFilesButton->setCheckable(true);
@@ -176,7 +173,7 @@ public:
         urlComboBox = new KComboBox(false, p);
         innerLayout->addWidget(urlComboBox, 1);
 
-        externalViewerButton = new KPushButton(KIcon("document-open"), QString::null, p);
+        externalViewerButton = new KPushButton(KIcon("document-open"), QString(), p);
         externalViewerButton->setToolTip(i18n("Open in external program"));
         innerLayout->addWidget(externalViewerButton, 0);
         sp = externalViewerButton->sizePolicy();
@@ -212,7 +209,7 @@ public:
         if (okularPart == NULL || swpOkular < 0) {
             kWarning() << "No Okular part for PDF or PostScript document preview available.";
         }
-#ifdef HAVE_QTWEBKIT
+#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
         kDebug() << "WebKit is available, using it instead of KHTML for HTML/Web preview.";
         htmlWidget = new QWebView(stackedWidget);
         swpHTML = stackedWidget->addWidget(htmlWidget);
@@ -231,7 +228,7 @@ public:
     }
 
     bool addUrl(const struct UrlInfo &urlInfo) {
-        bool isLocal = KBibTeX::isLocalOrRelative(urlInfo.url);
+        bool isLocal = isLocalOrRelative(urlInfo.url);
         anyLocal |= isLocal;
 
         if (!onlyLocalFilesButton->isChecked() && !isLocal) return true; ///< ignore URL if only local files are allowed
@@ -268,7 +265,7 @@ public:
         /// reset and clear all controls
         if (okularPart != NULL)
             okularPart->closeUrl();
-#ifdef HAVE_QTWEBKIT
+#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
         htmlWidget->stop();
 #else // HAVE_QTWEBKIT
         htmlPart->closeUrl();
@@ -277,7 +274,7 @@ public:
         urlComboBox->clear();
         cbxEntryToUrlInfo.clear();
         externalViewerButton->setEnabled(false);
-        showMessage(i18n("Refreshing ..."));
+        showMessage(i18n("Refreshing...")); // krazy:exclude=qmethods
 
         /// cancel/kill all running jobs
         for (QList<KIO::StatJob *>::ConstIterator it = runningJobs.constBegin(); it != runningJobs.constEnd(); ++it)
@@ -290,30 +287,29 @@ public:
 
         /// do not load external reference if widget is hidden
         if (isVisible()) {
-            QList<KUrl> urlList = FileInfo::entryUrls(entry.data(), baseUrl, FileInfo::TestExistanceYes);
+            QList<KUrl> urlList = FileInfo::entryUrls(entry.data(), baseUrl, FileInfo::TestExistenceYes);
 
             for (QList<KUrl>::ConstIterator it = urlList.constBegin(); it != urlList.constEnd(); ++it) {
-                bool isLocal = KBibTeX::isLocalOrRelative(*it);
-                kDebug() << "testing " << (*it).prettyUrl() << isLocal;
+                bool isLocal = isLocalOrRelative(*it);
                 anyRemote |= !isLocal;
                 if (!onlyLocalFilesButton->isChecked() && !isLocal) continue;
 
                 KIO::StatJob *job = KIO::stat(*it, KIO::StatJob::SourceSide, 3, KIO::HideProgressInfo);
                 runningJobs << job;
                 job->ui()->setWindow(p);
-                connect(job, SIGNAL(result(KJob *)), p, SLOT(statFinished(KJob *)));
+                connect(job, SIGNAL(result(KJob*)), p, SLOT(statFinished(KJob*)));
             }
             if (urlList.isEmpty()) {
                 /// Case no URLs associated with this entry.
                 /// For-loop above was never executed.
-                showMessage(i18n("No documents to show."));
+                showMessage(i18n("No documents to show.")); // krazy:exclude=qmethods
                 p->setCursor(Qt::ArrowCursor);
             } else if (runningJobs.isEmpty()) {
                 /// Case no stat jobs are running. As there were URLs (tested in
                 /// previous condition), this implies that there were remote
                 /// references that were ignored by executing "continue" above.
                 /// Give user hint that by enabling remote files, more can be shown.
-                showMessage(i18n("<qt>No documents to show.<br/><a href=\"disableonlylocalfiles\">Disable the restriction</a> to local files to see remote documents.</qt>"));
+                showMessage(i18n("<qt>No documents to show.<br/><a href=\"disableonlylocalfiles\">Disable the restriction</a> to local files to see remote documents.</qt>")); // krazy:exclude=qmethods
                 p->setCursor(Qt::ArrowCursor);
             }
         } else
@@ -395,14 +391,10 @@ public:
             for (int j = 0; j < actionProperties.count(); ++j) {
                 QDomNode actionNode = actionProperties.at(j);
                 if (actionNode.nodeName() == QLatin1String("Action")) {
-                    kDebug() << actionNode.attributes().namedItem("name").isNull() << actionNode.attributes().namedItem("name").isAttr();
-
                     const QString actionName = actionNode.attributes().namedItem("name").toAttr().nodeValue();
                     const QString actionShortcut = actionNode.attributes().namedItem("shortcut").toAttr().value();
-                    kDebug() << actionName << actionShortcut;
                     QAction *action = part->actionCollection()->action(actionName);
                     if (action != NULL) {
-                        kDebug() << "setting shortcut" << actionShortcut << "to action" << actionName;
                         action->setShortcut(QKeySequence(actionShortcut));
                     } else
                         kDebug() << "Could not locate an action with name " << actionName << "for shortcut" << actionShortcut;
@@ -425,7 +417,7 @@ public:
             stackedWidget->setCurrentIndex(swpOkular);
             stackedWidget->widget(swpOkular)->setEnabled(true);
             setupToolMenuBarForPart(okularPart);
-#ifdef HAVE_QTWEBKIT
+#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
         } else if (widget == htmlWidget) {
             stackedWidget->setCurrentIndex(swpHTML);
             stackedWidget->widget(swpHTML)->setEnabled(true);
@@ -438,7 +430,7 @@ public:
         } else if (widget == message) {
             stackedWidget->setCurrentIndex(swpMessage);
         } else
-            showMessage(i18n("Cannot show requested part"));
+            showMessage(i18n("Cannot show requested part")); // krazy:exclude=qmethods
     }
 
     bool showUrl(const struct UrlInfo &urlInfo) {
@@ -451,7 +443,7 @@ public:
             stackedWidget->widget(swpOkular)->setEnabled(false);
             okularPart->closeUrl();
         }
-#ifdef HAVE_QTWEBKIT
+#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
         htmlWidget->stop();
 #else // HAVE_QTWEBKIT
         htmlPart->closeUrl();
@@ -459,26 +451,29 @@ public:
 
         if (okularPart != NULL && okularMimetypes.contains(urlInfo.mimeType)) {
             p->setCursor(Qt::BusyCursor);
-            showMessage(i18n("Loading ..."));
-            okularPart->openUrl(urlInfo.url);
-            return true;
+            showMessage(i18n("Loading...")); // krazy:exclude=qmethods
+            return okularPart->openUrl(urlInfo.url);
         } else if (htmlMimetypes.contains(urlInfo.mimeType)) {
             p->setCursor(Qt::BusyCursor);
-            showMessage(i18n("Loading ..."));
-#ifdef HAVE_QTWEBKIT
+            showMessage(i18n("Loading...")); // krazy:exclude=qmethods
+#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
             htmlWidget->load(urlInfo.url);
-#else // HAVE_QTWEBKIT
-            htmlPart->openUrl(urlInfo.url);
-#endif // HAVE_QTWEBKIT
             return true;
+#else // HAVE_QTWEBKIT
+            return htmlPart->openUrl(urlInfo.url);
+#endif // HAVE_QTWEBKIT
         } else if (imageMimetypes.contains(urlInfo.mimeType)) {
             p->setCursor(Qt::BusyCursor);
             message->setPixmap(QPixmap(urlInfo.url.pathOrUrl()));
             showPart(NULL, message);
             p->unsetCursor();
             return true;
-        } else
-            showMessage(i18n("<qt>Don't know how to show mimetype '%1'.</qt>", urlInfo.mimeType));
+        } else {
+            QString additionalInformation;
+            if (urlInfo.mimeType == QLatin1String("application/pdf"))
+                additionalInformation = i18nc("Additional information in case there is not KPart available for mime type 'application/pdf'", "<br/><br/>Please install <a href=\"https://userbase.kde.org/Okular\">Okular</a> to make use of its PDF viewing component.");
+            showMessage(i18nc("First parameter is mime type, second parameter is optional information (may be empty)", "<qt>Don't know how to show mimetype '%1'.%2</qt>", urlInfo.mimeType, additionalInformation)); // krazy:exclude=qmethods
+        }
 
         return false;
     }
@@ -496,7 +491,7 @@ public:
         UrlInfo result;
         result.url = url;
 
-        if (!KBibTeX::isLocalOrRelative(url) && url.fileName().isEmpty()) {
+        if (!isLocalOrRelative(url) && url.fileName().isEmpty()) {
             /// URLs not pointing to a specific file should be opened with a web browser component
             result.icon = KIcon("text-html");
             result.mimeType = QLatin1String("text/html");
@@ -504,7 +499,7 @@ public:
         }
 
         int accuracy = 0;
-        KMimeType::Ptr mimeTypePtr = KMimeType::findByUrl(url, 0, KBibTeX::isLocalOrRelative(url), true, &accuracy);
+        KMimeType::Ptr mimeTypePtr = KMimeType::findByUrl(url, 0, isLocalOrRelative(url), true, &accuracy);
         if (accuracy < 50) {
             mimeTypePtr = KMimeType::findByPath(url.fileName(), 0, true, &accuracy);
         }
@@ -515,14 +510,13 @@ public:
             /// application/octet-stream is a fall-back if KDE did not know better
             result.icon = KIcon("text-html");
             result.mimeType = QLatin1String("text/html");
-        } else if (result.mimeType == QLatin1String("inode/directory") && (result.url.protocol() == QLatin1String("http") || result.url.protocol() == QLatin1String("https"))) {
+        } else if ((result.mimeType.isEmpty() || result.mimeType == QLatin1String("inode/directory")) && (result.url.protocol() == QLatin1String("http") || result.url.protocol() == QLatin1String("https"))) {
             /// directory via http means normal webpage (not browsable directory)
             result.icon = KIcon("text-html");
             result.mimeType = QLatin1String("text/html");
         }
 
         if (url.pathOrUrl().startsWith(arXivPDFUrlStart)) {
-            kDebug() << "URL looks like a PDF url from arXiv";
             result.icon = KIcon("application-pdf");
             result.mimeType = QLatin1String("application/pdf");
         }
@@ -568,7 +562,7 @@ DocumentPreview::~DocumentPreview()
     delete d;
 }
 
-void DocumentPreview::setElement(QSharedPointer<Element> element, const File *)
+void DocumentPreview::setElement(QSharedPointer<Element> element, File *)
 {
     d->entry = element.dynamicCast<const Entry>();
     d->update();
@@ -605,11 +599,11 @@ void DocumentPreview::statFinished(KJob *kjob)
     KIO::StatJob *job = static_cast<KIO::StatJob *>(kjob);
     d->runningJobs.removeOne(job);
     if (!job->error()) {
-#if KDE_VERSION_MINOR >= 4
+#if KDE_IS_VERSION(4, 4, 0)
         const KUrl url = job->mostLocalUrl();
-#else // KDE_VERSION_MINOR
+#else // KDE_IS_VERSION
         const KUrl url = job->url();
-#endif // KDE_VERSION_MINOR
+#endif // KDE_IS_VERSION
         DocumentPreviewPrivate::UrlInfo urlInfo = d->urlMetaInfo(url);
         setCursor(d->runningJobs.isEmpty() ? Qt::ArrowCursor : Qt::BusyCursor);
         d->addUrl(urlInfo);
@@ -626,10 +620,10 @@ void DocumentPreview::statFinished(KJob *kjob)
             if (d->anyRemote && !d->onlyLocalFilesButton->isChecked()) {
                 /// There are some remote URLs to probe,
                 /// but user was only looking for local files
-                d->showMessage(i18n("<qt>No documents to show.<br/><a href=\"disableonlylocalfiles\">Disable the restriction</a> to local files to see remote documents.</qt>"));
+                d->showMessage(i18n("<qt>No documents to show.<br/><a href=\"disableonlylocalfiles\">Disable the restriction</a> to local files to see remote documents.</qt>")); // krazy:exclude=qmethods
             } else {
                 /// No stat job at all succeeded. Show message to user.
-                d->showMessage(i18n("No documents to show.\nSome URLs or files could not be retrieved."));
+                d->showMessage(i18n("No documents to show.\nSome URLs or files could not be retrieved.")); // krazy:exclude=qmethods
             }
         }
     }
@@ -638,11 +632,21 @@ void DocumentPreview::statFinished(KJob *kjob)
 void DocumentPreview::loadingFinished()
 {
     setCursor(Qt::ArrowCursor);
-    d->showPart(dynamic_cast<KParts::ReadOnlyPart *>(sender()), dynamic_cast<QWidget *>(sender()));
+    d->showPart(qobject_cast<KParts::ReadOnlyPart *>(sender()), qobject_cast<QWidget *>(sender()));
 }
 
 void DocumentPreview::linkActivated(const QString &link)
 {
     if (link == QLatin1String("disableonlylocalfiles"))
         d->onlyLocalFilesButton->setChecked(true);
+    else if (link.startsWith(QLatin1String("http://")) || link.startsWith(QLatin1String("https://"))) {
+        const KUrl urlToOpen = KUrl::fromUserInput(link);
+        if (urlToOpen.isValid()) {
+            /// Guess mime type for url to open
+            KMimeType::Ptr mimeType = FileInfo::mimeTypeForUrl(urlToOpen);
+            const QString mimeTypeName = mimeType->name();
+            /// Ask KDE subsystem to open url in viewer matching mime type
+            KRun::runUrl(urlToOpen, mimeTypeName, this, false, false);
+        }
+    }
 }
