@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "searchform.h"
 
 #include <QLayout>
 #include <QMap>
@@ -44,31 +43,35 @@
 #include <KConfigGroup>
 #include <KTabWidget>
 
-#include <onlinesearchabstract.h>
-#include <onlinesearchgeneral.h>
-#include <onlinesearchbibsonomy.h>
-#include <onlinesearchgooglescholar.h>
-#include <onlinesearchpubmed.h>
-#include <onlinesearchieeexplore.h>
-#include <onlinesearchacmportal.h>
-#include <onlinesearchsciencedirect.h>
-#include <onlinesearchspringerlink.h>
-#include <onlinesearcharxiv.h>
-#include <onlinesearchjstor.h>
-#include <onlinesearchinspirehep.h>
-#include <onlinesearchsoanasaads.h>
-#include <onlinesearchisbndb.h>
-#include <onlinesearchmathscinet.h>
-#include <onlinesearchingentaconnect.h>
-#include <fileexporterbibtex.h>
-#include <element.h>
-#include <file.h>
-#include <comment.h>
-#include <openfileinfo.h>
-#include <bibtexeditor.h>
-#include <bibtexfilemodel.h>
-#include <searchresults.h>
-#include "searchform.h"
+#include "element.h"
+#include "file.h"
+#include "comment.h"
+#include "fileexporterbibtex.h"
+#include "onlinesearchabstract.h"
+#include "onlinesearchgeneral.h"
+#include "onlinesearchbibsonomy.h"
+#include "onlinesearchgooglescholar.h"
+#include "onlinesearchpubmed.h"
+#include "onlinesearchieeexplore.h"
+#include "onlinesearchacmportal.h"
+#include "onlinesearchsciencedirect.h"
+#include "onlinesearchspringerlink.h"
+#include "onlinesearcharxiv.h"
+#include "onlinesearchjstor.h"
+#include "onlinesearchmathscinet.h"
+#include "onlinesearchmrlookup.h"
+#include "onlinesearchinspirehep.h"
+#include "onlinesearchcernds.h"
+#include "onlinesearchingentaconnect.h"
+#include "onlinesearchsoanasaads.h"
+#include "onlinesearchisbndb.h"
+#include "onlinesearchideasrepec.h"
+#include "onlinesearchdoi.h"
+#include "onlinesearchoclcworldcat.h"
+#include "openfileinfo.h"
+#include "fileview.h"
+#include "filemodel.h"
+#include "searchresults.h"
 
 const int HomepageRole = Qt::UserRole + 5;
 const int WidgetRole = Qt::UserRole + 6;
@@ -107,7 +110,7 @@ public:
     }
 
     OnlineSearchQueryFormAbstract *currentQueryForm() {
-        QScrollArea *area = dynamic_cast<QScrollArea *>(queryTermsStack->currentWidget());
+        QScrollArea *area = qobject_cast<QScrollArea *>(queryTermsStack->currentWidget());
         return formToScrollArea.key(area, NULL);
     }
 
@@ -158,8 +161,8 @@ public:
 
         enginesList = new QListWidget(listContainer);
         layout->addWidget(enginesList, 0, 0, 1, 1);
-        connect(enginesList, SIGNAL(itemChanged(QListWidgetItem *)), p, SLOT(itemCheckChanged(QListWidgetItem *)));
-        connect(enginesList, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)), p, SLOT(enginesListCurrentChanged(QListWidgetItem *, QListWidgetItem *)));
+        connect(enginesList, SIGNAL(itemChanged(QListWidgetItem*)), p, SLOT(itemCheckChanged(QListWidgetItem*)));
+        connect(enginesList, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), p, SLOT(enginesListCurrentChanged(QListWidgetItem*,QListWidgetItem*)));
         enginesList->setSelectionMode(QAbstractItemView::NoSelection);
 
         actionOpenHomepage = new KAction(KIcon("internet-web-browser"), i18n("Go to Homepage"), p);
@@ -219,12 +222,17 @@ public:
         addEngine(new OnlineSearchIngentaConnect(p));
         addEngine(new OnlineSearchJStor(p));
         addEngine(new OnlineSearchMathSciNet(p));
+        addEngine(new OnlineSearchMRLookup(p));
         addEngine(new OnlineSearchInspireHep(p));
+        addEngine(new OnlineSearchCERNDS(p));
         addEngine(new OnlineSearchPubMed(p));
         addEngine(new OnlineSearchScienceDirect(p));
         addEngine(new OnlineSearchSpringerLink(p));
         addEngine(new OnlineSearchSOANASAADS(p));
         addEngine(new OnlineSearchIsbnDB(p));
+        addEngine(new OnlineSearchIDEASRePEc(p));
+        addEngine(new OnlineSearchDOI(p));
+        // TODO needs a valid WSKey // addEngine(new OnlineSearchOCLCWorldCat(p));
 
         p->itemCheckChanged(NULL);
         updateGUI();
@@ -251,7 +259,7 @@ public:
         itemToOnlineSearch.insert(item, engine);
         connect(engine, SIGNAL(foundEntry(QSharedPointer<Entry>)), p, SLOT(foundEntry(QSharedPointer<Entry>)));
         connect(engine, SIGNAL(stoppedSearch(int)), p, SLOT(stoppedSearch(int)));
-        connect(engine, SIGNAL(progress(int, int)), p, SLOT(updateProgress(int, int)));
+        connect(engine, SIGNAL(progress(int,int)), p, SLOT(updateProgress(int,int)));
     }
 
     void switchToSearch() {
@@ -302,7 +310,7 @@ public:
         }
 
         OnlineSearchQueryFormAbstract *currentQueryWidget = NULL;
-        if (checkedEngines.size() == 1)
+        if (cursor != NULL && checkedEngines.size() == 1)
             currentQueryWidget = cursor->data(WidgetRole).value<OnlineSearchQueryFormAbstract *>();
         if (currentQueryWidget == NULL)
             currentQueryWidget = generalQueryTermsForm;
@@ -348,7 +356,7 @@ void SearchForm::updatedConfiguration()
     d->loadEngines();
 }
 
-void SearchForm::setElement(QSharedPointer<Element> element, const File *)
+void SearchForm::setElement(QSharedPointer<Element> element, File *)
 {
     d->currentEntry = element.dynamicCast<const Entry>();
     d->useEntryButton->setEnabled(!d->currentEntry.isNull() && d->tabWidget->currentIndex() == 0);
