@@ -1,27 +1,28 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "mainwindow.h"
 
 #include <QDockWidget>
 #include <QDragEnterEvent>
 #include <QDropEvent>
 #include <QLabel>
+#include <QtCore/QPointer>
+#include <QTimer>
 
 #include <KIO/NetAccess>
 #include <KApplication>
@@ -37,8 +38,9 @@
 
 #include "kbibtexnamespace.h"
 #include "preferences/kbibtexpreferencesdialog.h"
-#include "mainwindow.h"
 #include "valuelist.h"
+#include "zoterobrowser.h"
+#include "statistics.h"
 #include "documentlist.h"
 #include "mdiwidget.h"
 #include "referencepreview.h"
@@ -46,14 +48,16 @@
 #include "searchform.h"
 #include "searchresults.h"
 #include "elementform.h"
-#include "bibtexeditor.h"
-#include "documentlist.h"
+#include "fileview.h"
+#include "filesettings.h"
 #include "xsltransform.h"
+#include "bibliographyservice.h"
+#include "bibutils.h"
 
 class KBibTeXMainWindow::KBibTeXMainWindowPrivate
 {
 private:
-    KBibTeXMainWindow *p;
+    // UNUSED KBibTeXMainWindow *p;
 
 public:
     KAction *actionClose;
@@ -61,22 +65,29 @@ public:
     QDockWidget *dockReferencePreview;
     QDockWidget *dockDocumentPreview;
     QDockWidget *dockValueList;
+    QDockWidget *dockZotero;
+    QDockWidget *dockStatistics;
     QDockWidget *dockSearchForm;
     QDockWidget *dockSearchResults;
     QDockWidget *dockElementForm;
+    QDockWidget *dockFileSettings;
     DocumentList *listDocumentList;
     MDIWidget *mdiWidget;
     ReferencePreview *referencePreview;
     DocumentPreview *documentPreview;
+    FileSettings *fileSettings;
     ValueList *valueList;
+    ZoteroBrowser *zotero;
+    Statistics *statistics;
     SearchForm *searchForm;
     SearchResults *searchResults;
     ElementForm *elementForm;
     KMenu *actionMenuRecentFilesMenu;
 
-    KBibTeXMainWindowPrivate(KBibTeXMainWindow *parent)
-            : p(parent) {
-        // nothing
+    KBibTeXMainWindowPrivate(KBibTeXMainWindow */* UNUSED parent*/)
+    // UNUSED : p(parent)
+    {
+        /// nothing
     }
 
     ~KBibTeXMainWindowPrivate() {
@@ -117,6 +128,7 @@ KBibTeXMainWindow::KBibTeXMainWindow()
      * Docklets (a.k.a. panels) will be added by default to the following
      * positions unless otherwise configured by the user.
      * - "List of Values" on the left
+     * - "Statistics" on the left
      * - "List of Documents" on the left in the same tab
      * - "Online Search" on the left in a new tab
      * - "Reference Preview" on the left in the same tab
@@ -131,19 +143,27 @@ KBibTeXMainWindow::KBibTeXMainWindow()
     d->dockDocumentList->setWidget(d->listDocumentList);
     d->dockDocumentList->setObjectName("dockDocumentList");
     d->dockDocumentList->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
-    connect(d->listDocumentList, SIGNAL(openFile(const KUrl &)), this, SLOT(openDocument(const KUrl &)));
+    connect(d->listDocumentList, SIGNAL(openFile(KUrl)), this, SLOT(openDocument(KUrl)));
     showPanelsMenu->addAction(d->dockDocumentList->toggleViewAction());
 
     d->dockValueList = new QDockWidget(i18n("List of Values"), this);
     d->dockValueList->setAllowedAreas(Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea | Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
     addDockWidget(Qt::LeftDockWidgetArea, d->dockValueList);
-    tabifyDockWidget(d->dockDocumentList, d->dockValueList);
     d->valueList = new ValueList(d->dockValueList);
     d->dockValueList->setWidget(d->valueList);
     d->dockValueList->setObjectName("dockValueList");
     d->dockValueList->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
     showPanelsMenu->addAction(d->dockValueList->toggleViewAction());
 
+    d->dockStatistics = new QDockWidget(i18n("Statistics"), this);
+    d->dockStatistics->setAllowedAreas(Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea | Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
+    addDockWidget(Qt::LeftDockWidgetArea, d->dockStatistics);
+    d->statistics = new Statistics(d->dockStatistics);
+    d->dockStatistics->setWidget(d->statistics);
+    d->dockStatistics->setObjectName("dockStatistics");
+    d->dockStatistics->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
+    showPanelsMenu->addAction(d->dockStatistics->toggleViewAction());
+
     d->dockSearchResults = new QDockWidget(i18n("Search Results"), this);
     d->dockSearchResults->setAllowedAreas(Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea | Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
     addDockWidget(Qt::BottomDockWidgetArea, d->dockSearchResults);
@@ -153,7 +173,7 @@ KBibTeXMainWindow::KBibTeXMainWindow()
     d->dockSearchResults->setObjectName("dockResultsFrom");
     d->dockSearchResults->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
     showPanelsMenu->addAction(d->dockSearchResults->toggleViewAction());
-    connect(d->mdiWidget, SIGNAL(documentSwitch(BibTeXEditor *, BibTeXEditor *)), d->searchResults, SLOT(documentSwitched(BibTeXEditor *, BibTeXEditor *)));
+    connect(d->mdiWidget, SIGNAL(documentSwitch(FileView*,FileView*)), d->searchResults, SLOT(documentSwitched(FileView*,FileView*)));
 
     d->dockSearchForm = new QDockWidget(i18n("Online Search"), this);
     d->dockSearchForm->setAllowedAreas(Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea | Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
@@ -165,10 +185,18 @@ KBibTeXMainWindow::KBibTeXMainWindow()
     d->dockSearchForm->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
     showPanelsMenu->addAction(d->dockSearchForm->toggleViewAction());
 
+    d->dockZotero = new QDockWidget(i18n("Zotero"), this);
+    d->dockZotero->setAllowedAreas(Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea | Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
+    addDockWidget(Qt::LeftDockWidgetArea, d->dockZotero);
+    d->zotero = new ZoteroBrowser(d->searchResults, d->dockZotero);
+    d->dockZotero->setWidget(d->zotero);
+    d->dockZotero->setObjectName("dockZotero");
+    d->dockZotero->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
+    showPanelsMenu->addAction(d->dockZotero->toggleViewAction());
+
     d->dockReferencePreview = new QDockWidget(i18n("Reference Preview"), this);
     d->dockReferencePreview->setAllowedAreas(Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea | Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
     addDockWidget(Qt::LeftDockWidgetArea, d->dockReferencePreview);
-    tabifyDockWidget(d->dockReferencePreview, d->dockSearchForm);
     d->referencePreview = new ReferencePreview(d->dockReferencePreview);
     d->dockReferencePreview->setWidget(d->referencePreview);
     d->dockReferencePreview->setObjectName("dockReferencePreview");
@@ -196,6 +224,22 @@ KBibTeXMainWindow::KBibTeXMainWindow()
     d->dockElementForm->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
     showPanelsMenu->addAction(d->dockElementForm->toggleViewAction());
 
+    d->dockFileSettings = new QDockWidget(i18n("File Settings"), this);
+    d->dockFileSettings->setAllowedAreas(Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea | Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
+    addDockWidget(Qt::LeftDockWidgetArea, d->dockFileSettings);
+    d->fileSettings = new FileSettings(d->dockFileSettings);
+    d->dockFileSettings->setWidget(d->fileSettings);
+    d->dockFileSettings->setObjectName("dockFileSettings");
+    d->dockFileSettings->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
+    showPanelsMenu->addAction(d->dockFileSettings->toggleViewAction());
+
+    tabifyDockWidget(d->dockFileSettings, d->dockSearchForm);
+    tabifyDockWidget(d->dockZotero, d->dockSearchForm);
+    tabifyDockWidget(d->dockValueList, d->dockStatistics);
+    tabifyDockWidget(d->dockStatistics, d->dockFileSettings);
+    tabifyDockWidget(d->dockSearchForm, d->dockReferencePreview);
+    tabifyDockWidget(d->dockFileSettings, d->dockDocumentList);
+
     actionCollection()->addAction(KStandardAction::New, this, SLOT(newDocument()));
     actionCollection()->addAction(KStandardAction::Open, this, SLOT(openDocumentDialog()));
     d->actionClose = actionCollection()->addAction(KStandardAction::Close, this, SLOT(closeDocument()));
@@ -203,19 +247,19 @@ KBibTeXMainWindow::KBibTeXMainWindow()
     actionCollection()->addAction(KStandardAction::Quit, this, SLOT(queryCloseAll()));
     actionCollection()->addAction(KStandardAction::Preferences, this, SLOT(showPreferences()));
 
-    connect(d->mdiWidget, SIGNAL(documentSwitch(BibTeXEditor *, BibTeXEditor *)), this, SLOT(documentSwitched(BibTeXEditor *, BibTeXEditor *)));
-    connect(d->mdiWidget, SIGNAL(activePartChanged(KParts::Part *)), this, SLOT(createGUI(KParts::Part *)));
+    connect(d->mdiWidget, SIGNAL(documentSwitch(FileView*,FileView*)), this, SLOT(documentSwitched(FileView*,FileView*)));
+    connect(d->mdiWidget, SIGNAL(activePartChanged(KParts::Part*)), this, SLOT(createGUI(KParts::Part*)));
     connect(d->mdiWidget, SIGNAL(documentNew()), this, SLOT(newDocument()));
     connect(d->mdiWidget, SIGNAL(documentOpen()), this, SLOT(openDocumentDialog()));
     connect(d->mdiWidget, SIGNAL(documentOpenURL(KUrl)), this, SLOT(openDocument(KUrl)));
-    connect(d->mdiWidget->getOpenFileInfoManager(), SIGNAL(currentChanged(OpenFileInfo *, KService::Ptr)), d->mdiWidget, SLOT(setFile(OpenFileInfo *, KService::Ptr)));
+    connect(d->mdiWidget->getOpenFileInfoManager(), SIGNAL(currentChanged(OpenFileInfo*,KService::Ptr)), d->mdiWidget, SLOT(setFile(OpenFileInfo*,KService::Ptr)));
     connect(d->mdiWidget->getOpenFileInfoManager(), SIGNAL(flagsChanged(OpenFileInfo::StatusFlags)), this, SLOT(documentListsChanged(OpenFileInfo::StatusFlags)));
     connect(d->mdiWidget, SIGNAL(setCaption(QString)), this, SLOT(setCaption(QString)));
 
     documentListsChanged(OpenFileInfo::RecentlyUsed); /// force initialization of menu of recently used files
 
     setupControllers();
-    setupGUI();
+    setupGUI(KXmlGuiWindow::Create | KXmlGuiWindow::Save | KXmlGuiWindow::Keys | KXmlGuiWindow::ToolBar);
 
     setCorner(Qt::TopLeftCorner, Qt::LeftDockWidgetArea);
     setCorner(Qt::TopRightCorner, Qt::RightDockWidgetArea);
@@ -223,6 +267,8 @@ KBibTeXMainWindow::KBibTeXMainWindow()
     setCorner(Qt::BottomRightCorner, Qt::RightDockWidgetArea);
 
     setAcceptDrops(true);
+
+    QTimer::singleShot(500, this, SLOT(delayed()));
 }
 
 KBibTeXMainWindow::~KBibTeXMainWindow()
@@ -236,16 +282,6 @@ void KBibTeXMainWindow::setupControllers()
     // TODO
 }
 
-void KBibTeXMainWindow::saveProperties(KConfigGroup &/*configGroup*/)
-{
-    // TODO
-}
-
-void KBibTeXMainWindow::readProperties(const KConfigGroup &/*configGroup*/)
-{
-    // TODO
-}
-
 void KBibTeXMainWindow::dragEnterEvent(QDragEnterEvent *event)
 {
     if (event->mimeData()->hasUrls())
@@ -288,8 +324,11 @@ void KBibTeXMainWindow::openDocumentDialog()
         if (url.isValid()) startDir = url.path();
     }
 
-    // TODO application/x-research-info-systems application/x-endnote-refer
-    KUrl url = KFileDialog::getOpenUrl(startDir, QLatin1String("text/x-bibtex application/x-research-info-systems application/xml all/all"), this);
+    QString supportedMimeTypes = QLatin1String("text/x-bibtex application/x-research-info-systems application/xml");
+    if (BibUtils::available())
+        supportedMimeTypes += QLatin1String(" application/x-isi-export-format application/x-endnote-refer");
+    supportedMimeTypes += QLatin1String(" all/all");
+    KUrl url = KFileDialog::getOpenUrl(startDir, supportedMimeTypes, this);
     if (!url.isEmpty()) {
         openDocument(url);
     }
@@ -318,11 +357,12 @@ void KBibTeXMainWindow::closeEvent(QCloseEvent *event)
 
 void KBibTeXMainWindow::showPreferences()
 {
-    KBibTeXPreferencesDialog dlg(this);
-    dlg.exec();
+    QPointer<KBibTeXPreferencesDialog> dlg = new KBibTeXPreferencesDialog(this);
+    dlg->exec();
+    delete dlg;
 }
 
-void KBibTeXMainWindow::documentSwitched(BibTeXEditor *oldEditor, BibTeXEditor *newEditor)
+void KBibTeXMainWindow::documentSwitched(FileView *oldFileView, FileView *newFileView)
 {
     OpenFileInfo *openFileInfo = d->mdiWidget->currentFile();
     bool validFile = openFileInfo != NULL;
@@ -330,32 +370,42 @@ void KBibTeXMainWindow::documentSwitched(BibTeXEditor *oldEditor, BibTeXEditor *
 
     setCaption(validFile ? i18n("%1 - KBibTeX", openFileInfo->shortCaption()) : i18n("KBibTeX"));
 
-    d->referencePreview->setEnabled(newEditor != NULL);
-    d->elementForm->setEnabled(newEditor != NULL);
-    d->documentPreview->setEnabled(newEditor != NULL);
-    if (oldEditor != NULL) {
-        disconnect(oldEditor, SIGNAL(currentElementChanged(QSharedPointer<Element>, const File *)), d->referencePreview, SLOT(setElement(QSharedPointer<Element>, const File *)));
-        disconnect(oldEditor, SIGNAL(currentElementChanged(QSharedPointer<Element>, const File *)), d->elementForm, SLOT(setElement(QSharedPointer<Element>, const File *)));
-        disconnect(oldEditor, SIGNAL(currentElementChanged(QSharedPointer<Element>, const File *)), d->documentPreview, SLOT(setElement(QSharedPointer<Element>, const File *)));
-        disconnect(oldEditor, SIGNAL(currentElementChanged(QSharedPointer<Element>, const File *)), d->searchForm, SLOT(setElement(QSharedPointer<Element>, const File *)));
-        disconnect(oldEditor, SIGNAL(modified()), d->valueList, SLOT(update()));
-        disconnect(d->elementForm, SIGNAL(elementModified()), oldEditor, SLOT(externalModification()));
+    d->fileSettings->setEnabled(newFileView != NULL);
+    d->referencePreview->setEnabled(newFileView != NULL);
+    d->elementForm->setEnabled(newFileView != NULL);
+    d->documentPreview->setEnabled(newFileView != NULL);
+    if (oldFileView != NULL) {
+        disconnect(oldFileView, SIGNAL(currentElementChanged(QSharedPointer<Element>,File*)), d->referencePreview, SLOT(setElement(QSharedPointer<Element>,File*)));
+        disconnect(oldFileView, SIGNAL(currentElementChanged(QSharedPointer<Element>,File*)), d->elementForm, SLOT(setElement(QSharedPointer<Element>,File*)));
+        disconnect(oldFileView, SIGNAL(currentElementChanged(QSharedPointer<Element>,File*)), d->documentPreview, SLOT(setElement(QSharedPointer<Element>,File*)));
+        disconnect(oldFileView, SIGNAL(currentElementChanged(QSharedPointer<Element>,File*)), d->searchForm, SLOT(setElement(QSharedPointer<Element>,File*)));
+        disconnect(oldFileView, SIGNAL(modified()), d->valueList, SLOT(update()));
+        disconnect(oldFileView, SIGNAL(modified()), d->statistics, SLOT(update()));
+        // FIXME disconnect(oldEditor, SIGNAL(modified()), d->elementForm, SLOT(refreshElement()));
+        disconnect(d->elementForm, SIGNAL(elementModified()), oldFileView, SLOT(externalModification()));
     }
-    if (newEditor != NULL) {
-        connect(newEditor, SIGNAL(currentElementChanged(QSharedPointer<Element>, const File *)), d->referencePreview, SLOT(setElement(QSharedPointer<Element>, const File *)));
-        connect(newEditor, SIGNAL(currentElementChanged(QSharedPointer<Element>, const File *)), d->elementForm, SLOT(setElement(QSharedPointer<Element>, const File *)));
-        connect(newEditor, SIGNAL(currentElementChanged(QSharedPointer<Element>, const File *)), d->documentPreview, SLOT(setElement(QSharedPointer<Element>, const File *)));
-        connect(newEditor, SIGNAL(currentElementChanged(QSharedPointer<Element>, const File *)), d->searchForm, SLOT(setElement(QSharedPointer<Element>, const File *)));
-        connect(newEditor, SIGNAL(modified()), d->valueList, SLOT(update()));
-        connect(d->elementForm, SIGNAL(elementModified()), newEditor, SLOT(externalModification()));
+    if (newFileView != NULL) {
+        connect(newFileView, SIGNAL(currentElementChanged(QSharedPointer<Element>,File*)), d->referencePreview, SLOT(setElement(QSharedPointer<Element>,File*)));
+        connect(newFileView, SIGNAL(currentElementChanged(QSharedPointer<Element>,File*)), d->elementForm, SLOT(setElement(QSharedPointer<Element>,File*)));
+        connect(newFileView, SIGNAL(currentElementChanged(QSharedPointer<Element>,File*)), d->documentPreview, SLOT(setElement(QSharedPointer<Element>,File*)));
+        connect(newFileView, SIGNAL(currentElementChanged(QSharedPointer<Element>,File*)), d->searchForm, SLOT(setElement(QSharedPointer<Element>,File*)));
+        connect(newFileView, SIGNAL(modified()), d->valueList, SLOT(update()));
+        connect(newFileView, SIGNAL(modified()), d->statistics, SLOT(update()));
+        // FIXME connect(newEditor, SIGNAL(modified()), d->elementForm, SLOT(refreshElement()));
+        connect(d->elementForm, SIGNAL(elementModified()), newFileView, SLOT(externalModification()));
     }
 
     d->documentPreview->setBibTeXUrl(validFile ? openFileInfo->url() : KUrl());
     d->referencePreview->setElement(QSharedPointer<Element>(), NULL);
     d->elementForm->setElement(QSharedPointer<Element>(), NULL);
     d->documentPreview->setElement(QSharedPointer<Element>(), NULL);
-    d->valueList->setEditor(newEditor);
-    d->referencePreview->setEditor(newEditor);
+    d->valueList->setFileView(newFileView);
+    d->fileSettings->setFileView(newFileView);
+    if (newFileView != NULL && newFileView->fileModel() != NULL)
+        d->statistics->setFile(newFileView->fileModel()->bibliographyFile(), newFileView->selectionModel());
+    else
+        d->statistics->setFile(NULL, NULL);
+    d->referencePreview->setFileView(newFileView);
 }
 
 void KBibTeXMainWindow::showSearchResults()
@@ -376,7 +426,7 @@ void KBibTeXMainWindow::documentListsChanged(OpenFileInfo::StatusFlags statusFla
             const QString squeezedFullCap = squeeze_text(cur->fullCaption(), squeezeLen);
             KAction *action = new KAction(QString("%1 [%2]").arg(squeezedShortCap).arg(squeezedFullCap), this);
             action->setData(cur->url());
-            action->setIcon(KIcon(cur->mimeType().replace("/", "-")));
+            action->setIcon(KIcon(cur->mimeType().replace(QLatin1Char('/'), QLatin1Char('-'))));
             d->actionMenuRecentFilesMenu->addAction(action);
             connect(action, SIGNAL(triggered()), this, SLOT(openRecentFile()));
         }
@@ -395,3 +445,30 @@ void KBibTeXMainWindow::queryCloseAll()
     if (d->mdiWidget->getOpenFileInfoManager()->queryCloseAll())
         kapp->quit();
 }
+
+void KBibTeXMainWindow::delayed() {
+    /// Static variable, memorizes the dynamically created
+    /// BibliographyService instance and allows to tell if
+    /// this slot was called for the first or second time.
+    static BibliographyService *bs = NULL;
+
+    if (bs == NULL) {
+        /// First call to this slot
+        bs = new BibliographyService(this);
+        if (!bs->isKBibTeXdefault() && KMessageBox::questionYesNo(this, i18n("KBibTeX is not the default editor for its bibliography formats like BibTeX or RIS."), i18n("Default Bibliography Editor"), KGuiItem(i18n("Set as Default Editor")), KGuiItem(i18n("Keep settings unchanged"))) == KMessageBox::Yes) {
+            bs->setKBibTeXasDefault();
+            /// QTimer calls this slot again, but as 'bs' will not be NULL,
+            /// the 'if' construct's 'else' path will be followed.
+            QTimer::singleShot(5000, this, SLOT(delayed()));
+        } else {
+            /// KBibTeX is default application or user doesn't care,
+            /// therefore clean up memory
+            delete bs;
+            bs = NULL;
+        }
+    } else {
+        /// Second call to this slot. This time, clean up memory.
+        bs->deleteLater();
+        bs = NULL;
+    }
+}
