@@ -1,22 +1,21 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "mdiwidget.h"
 
 #include <QVector>
 #include <QPair>
@@ -33,28 +32,33 @@
 #include <KPushButton>
 #include <KIcon>
 #include <KMessageBox>
-#include <kmimetypetrader.h>
 #include <kparts/part.h>
-#include <kio/netaccess.h>
+#include <KConfigGroup>
+#include <KSharedConfig>
 
 #include "kbibtexnamespace.h"
-#include "mdiwidget.h"
-
-const int URLRole = Qt::UserRole + 235;
-const int SortRole = Qt::UserRole + 236;
+#include "partwidget.h"
 
 class LRUItemModel : public QAbstractItemModel
 {
 private:
     OpenFileInfoManager *ofim;
+
 public:
+    static const int URLRole = Qt::UserRole + 235;
+    static const int SortRole = Qt::UserRole + 236;
+
     LRUItemModel(OpenFileInfoManager *openFileInfoManager, QObject *parent = NULL)
             : QAbstractItemModel(parent), ofim(openFileInfoManager) {
         // nothing
     }
 
+    void reset() {
+        QAbstractItemModel::reset();
+    }
+
     int columnCount(const QModelIndex &) const {
-        return 2;
+        return 3;
     }
 
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const {
@@ -62,10 +66,12 @@ public:
         if (index.row() < ofiList.count()) {
             OpenFileInfo *ofiItem = ofiList[index.row()];
             if (index.column() == 0) {
-                if (role == Qt::DisplayRole || role == SortRole)
-                    return ofiItem->url().fileName();
-                else if (role == Qt::DecorationRole)
-                    return KIcon(ofiItem->mimeType().replace("/", "-"));
+                if (role == Qt::DisplayRole || role == SortRole) {
+                    const KUrl url = ofiItem->url();
+                    const QString fileName = url.fileName();
+                    return fileName.isEmpty() ? squeeze_text(url.pathOrUrl(), 32) : fileName;
+                } else if (role == Qt::DecorationRole)
+                    return KIcon(ofiItem->mimeType().replace(QLatin1Char('/'), QLatin1Char('-')));
                 else if (role == Qt::ToolTipRole)
                     return squeeze_text(ofiItem->url().pathOrUrl(), 64);
             } else if (index.column() == 1) {
@@ -73,23 +79,22 @@ public:
                     return ofiItem->lastAccess().toString(Qt::TextDate);
                 else if (role == SortRole)
                     return ofiItem->lastAccess().toTime_t();
+            } else if (index.column() == 2) {
+                if (role == Qt::DisplayRole || role == Qt::ToolTipRole || role == SortRole)
+                    return ofiItem->url().pathOrUrl();
             }
-            if (role == URLRole) {
-                KUrl url = ofiItem->url();
-                if (url.isLocalFile())
-                    return KUrl::fromLocalFile(url.path());
-                else
-                    return url;
-            }
+            if (role == URLRole)
+                return QVariant::fromValue<QUrl>(ofiItem->url());
         }
 
         return QVariant();
     }
 
     QVariant headerData(int section, Qt::Orientation , int role = Qt::DisplayRole) const {
-        if (role != Qt::DisplayRole || section >= 2)return QVariant();
+        if (role != Qt::DisplayRole || section > 2) return QVariant();
         else if (section == 0) return i18n("Filename");
-        else return i18n("Date/time of last use");
+        else if (section == 1) return i18n("Date/time of last use");
+        else return i18n("Full filename");
     }
 
     QModelIndex index(int row, int column, const QModelIndex &) const {
@@ -115,6 +120,9 @@ private:
     LRUItemModel *modelLRU;
     QVector<QWidget *> welcomeWidgets;
 
+    static const QString configGroupName;
+    static const QString configHeaderState;
+
     void createWelcomeWidget() {
         welcomeWidget = new QWidget(p);
         QGridLayout *layout = new QGridLayout(welcomeWidget);
@@ -139,7 +147,7 @@ private:
         layout->addWidget(buttonNew, 2, 2, 1, 1, Qt::AlignLeft | Qt::AlignBottom);
         connect(buttonNew, SIGNAL(clicked()), p, SIGNAL(documentNew()));
 
-        KPushButton *buttonOpen = new KPushButton(KIcon("document-open"), i18n("Open ..."), welcomeWidget);
+        KPushButton *buttonOpen = new KPushButton(KIcon("document-open"), i18n("Open..."), welcomeWidget);
         layout->addWidget(buttonOpen, 2, 4, 1, 1, Qt::AlignRight | Qt::AlignBottom);
         connect(buttonOpen, SIGNAL(clicked()), p, SIGNAL(documentOpen()));
 
@@ -156,6 +164,20 @@ private:
         p->addWidget(welcomeWidget);
     }
 
+    void saveColumnsState() {
+        QByteArray headerState = listLRU->header()->saveState();
+        KConfigGroup configGroup(config, configGroupName);
+        configGroup.writeEntry(configHeaderState, headerState);
+        config->sync();
+    }
+
+    void restoreColumnsState() {
+        KConfigGroup configGroup(config, configGroupName);
+        QByteArray headerState = configGroup.readEntry(configHeaderState, QByteArray());
+        if (!headerState.isEmpty())
+            listLRU->header()->restoreState(headerState);
+    }
+
 public:
     MDIWidget *p;
     OpenFileInfoManager *ofim;
@@ -163,21 +185,25 @@ public:
     QWidget *welcomeWidget;
     QSignalMapper signalMapperCompleted;
 
+    KSharedConfigPtr config;
+
     MDIWidgetPrivate(MDIWidget *parent)
-            : p(parent), ofim(new OpenFileInfoManager), currentFile(NULL) {
+            : p(parent), ofim(new OpenFileInfoManager(parent)), currentFile(NULL), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))) {
         createWelcomeWidget();
 
         modelLRU = new LRUItemModel(ofim, listLRU);
         QSortFilterProxyModel *sfpm = new QSortFilterProxyModel(listLRU);
         sfpm->setSourceModel(modelLRU);
-        sfpm->setSortRole(SortRole);
+        sfpm->setSortRole(LRUItemModel::SortRole);
         listLRU->setModel(sfpm);
-        updateLRU();
 
-        connect(&signalMapperCompleted, SIGNAL(mapped(QObject *)), p, SLOT(slotCompleted(QObject *)));
+        connect(&signalMapperCompleted, SIGNAL(mapped(QObject*)), p, SLOT(slotCompleted(QObject*)));
+
+        restoreColumnsState();
     }
 
     ~MDIWidgetPrivate() {
+        saveColumnsState();
         delete ofim;
         delete welcomeWidget;
     }
@@ -189,11 +215,13 @@ public:
     }
 
     void updateLRU() {
-
-        listLRU->reset();
+        modelLRU->reset();
     }
 };
 
+const QString MDIWidget::MDIWidgetPrivate::configGroupName = QLatin1String("WelcomeWidget");
+const QString MDIWidget::MDIWidgetPrivate::configHeaderState = QLatin1String("LRUlistHeaderState");
+
 MDIWidget::MDIWidget(QWidget *parent)
         : QStackedWidget(parent), d(new MDIWidgetPrivate(this))
 {
@@ -207,7 +235,7 @@ MDIWidget::~MDIWidget()
 
 void MDIWidget::setFile(OpenFileInfo *openFileInfo, KService::Ptr servicePtr)
 {
-    BibTeXEditor *oldEditor = NULL;
+    FileView *oldEditor = NULL;
     bool hasChanged = true;
 
     KParts::Part *part = openFileInfo == NULL ? NULL : openFileInfo->part(this, servicePtr);
@@ -221,9 +249,10 @@ void MDIWidget::setFile(OpenFileInfo *openFileInfo, KService::Ptr servicePtr)
     }
 
     if (indexOf(widget) >= 0) {
-        oldEditor = dynamic_cast<BibTeXEditor *>(currentWidget());
+        PartWidget *currentPartWidget = qobject_cast<PartWidget *>(currentWidget());
+        oldEditor = currentPartWidget == NULL ? NULL : currentPartWidget->fileView();
         hasChanged = widget != currentWidget();
-    } else {
+    } else if (openFileInfo != NULL) {
         addWidget(widget);
         d->addToMapper(openFileInfo);
     }
@@ -231,7 +260,8 @@ void MDIWidget::setFile(OpenFileInfo *openFileInfo, KService::Ptr servicePtr)
     d->currentFile = openFileInfo;
 
     if (hasChanged) {
-        BibTeXEditor *newEditor = dynamic_cast<BibTeXEditor *>(widget);
+        PartWidget *newPartWidget = qobject_cast<PartWidget *>(widget);
+        FileView *newEditor = newPartWidget == NULL ? NULL : newPartWidget->fileView();
         emit activePartChanged(part);
         emit documentSwitch(oldEditor, newEditor);
     }
@@ -246,10 +276,10 @@ void MDIWidget::setFile(OpenFileInfo *openFileInfo, KService::Ptr servicePtr)
         emit setCaption("");
 }
 
-BibTeXEditor *MDIWidget::editor()
+FileView *MDIWidget::fileView()
 {
     OpenFileInfo *ofi = d->ofim->currentFile();
-    return dynamic_cast<BibTeXEditor *>(ofi->part(this)->widget());
+    return qobject_cast<PartWidget *>(ofi->part(this)->widget())->fileView();
 }
 
 OpenFileInfo *MDIWidget::currentFile()
@@ -287,7 +317,7 @@ void MDIWidget::slotStatusFlagsChanged(OpenFileInfo::StatusFlags statusFlags)
 
 void MDIWidget::slotOpenLRU(const QModelIndex &index)
 {
-    KUrl url = index.data(URLRole).toUrl();
+    QUrl url = index.data(LRUItemModel::URLRole).toUrl();
     if (url.isValid())
         emit documentOpenURL(url);
 }
