@@ -1,27 +1,27 @@
 /***************************************************************************
-*   Copyright (C) 2004-2012 by Thomas Fischer                             *
-*   fischer@unix-ag.uni-kl.de                                             *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the                         *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ ***************************************************************************/
+
+#include "openfileinfo.h"
 
 #include <QString>
 #include <QLatin1String>
 #include <QTimer>
 #include <QFileInfo>
+#include <QWidget>
 
 #include <KConfig>
 #include <KConfigGroup>
@@ -29,9 +29,9 @@
 #include <KMimeTypeTrader>
 #include <KUrl>
 #include <kparts/part.h>
+#include <KIO/NetAccess>
 
-#include <fileimporterpdf.h>
-#include "openfileinfo.h"
+#include "fileimporterpdf.h"
 
 class OpenFileInfo::OpenFileInfoPrivate
 {
@@ -59,12 +59,14 @@ public:
         :  m_counter(-1), p(p), part(NULL), internalServicePtr(KService::Ptr()), internalWidgetParent(NULL), flags(0) {
         this->openFileInfoManager = openFileInfoManager;
         this->url = url;
+        if (this->url.scheme().isEmpty())
+            kWarning() << "No scheme specified for URL" << this->url.pathOrUrl();
         this->mimeType = mimeType;
     }
 
     ~OpenFileInfoPrivate() {
         if (part != NULL) {
-            KParts::ReadWritePart *rwp = dynamic_cast<KParts::ReadWritePart *>(part);
+            KParts::ReadWritePart *rwp = qobject_cast<KParts::ReadWritePart *>(part);
             if (rwp != NULL)
                 rwp->closeUrl(true);
             delete part;
@@ -84,7 +86,7 @@ public:
             Q_ASSERT_X(part != NULL, "KParts::ReadOnlyPart *OpenFileInfo::OpenFileInfoPrivate::createPart(QWidget *newWidgetParent, KService::Ptr newServicePtr = KService::Ptr())", "Part is NULL");
             return part;
         } else if (part != NULL) {
-            KParts::ReadWritePart *rwp = dynamic_cast<KParts::ReadWritePart *>(part);
+            KParts::ReadWritePart *rwp = qobject_cast<KParts::ReadWritePart *>(part);
             if (rwp != NULL)
                 rwp->closeUrl(true);
             part->deleteLater();
@@ -171,6 +173,8 @@ void OpenFileInfo::setUrl(const KUrl &url)
 {
     Q_ASSERT_X(url.isValid(), "void OpenFileInfo::setUrl(const KUrl&)", "URL is not valid");
     d->url = url;
+    if (d->url.scheme().isEmpty())
+        kWarning() << "No scheme specified for URL" << d->url.pathOrUrl();
     d->mimeType = FileInfo::mimeTypeForUrl(url)->name();
     addFlags(OpenFileInfo::HasName);
 }
@@ -182,7 +186,7 @@ KUrl OpenFileInfo::url() const
 
 bool OpenFileInfo::isModified() const
 {
-    KParts::ReadWritePart *rwPart = dynamic_cast< KParts::ReadWritePart *>(d->part);
+    KParts::ReadWritePart *rwPart = qobject_cast< KParts::ReadWritePart *>(d->part);
     if (rwPart == NULL)
         return false;
     else
@@ -191,7 +195,7 @@ bool OpenFileInfo::isModified() const
 
 bool OpenFileInfo::save()
 {
-    KParts::ReadWritePart *rwPart = dynamic_cast< KParts::ReadWritePart *>(d->part);
+    KParts::ReadWritePart *rwPart = qobject_cast< KParts::ReadWritePart *>(d->part);
     if (rwPart == NULL)
         return true;
     else
@@ -205,7 +209,7 @@ bool OpenFileInfo::close()
         return true;
     }
 
-    KParts::ReadWritePart *rwp = dynamic_cast<KParts::ReadWritePart *>(d->part);
+    KParts::ReadWritePart *rwp = qobject_cast<KParts::ReadWritePart *>(d->part);
     if (rwp == NULL || rwp->closeUrl(true)) {
         d->part->deleteLater();
         d->part = NULL;
@@ -250,6 +254,8 @@ void OpenFileInfo::setFlags(StatusFlags statusFlags)
 {
     /// disallow files without name or valid url to become favorites
     if (!d->url.isValid() || !d->flags.testFlag(HasName)) statusFlags &= ~Favorite;
+    /// files that got opened are by definition recently used files
+    if (!d->url.isValid() && d->flags.testFlag(Open)) statusFlags &= RecentlyUsed;
 
     bool hasChanged = d->flags != statusFlags;
     d->flags = statusFlags;
@@ -316,6 +322,7 @@ private:
     static const QString configGroupNameFavorites;
     static const QString configGroupNameOpen;
     static const int maxNumRecentlyUsedFiles, maxNumFavoriteFiles, maxNumOpenFiles;
+    QWidget *widget;
 
 public:
     OpenFileInfoManager *p;
@@ -323,8 +330,8 @@ public:
     OpenFileInfoManager::OpenFileInfoList openFileInfoList;
     OpenFileInfo *currentFileInfo;
 
-    OpenFileInfoManagerPrivate(OpenFileInfoManager *parent)
-            : p(parent), currentFileInfo(NULL) {
+    OpenFileInfoManagerPrivate(OpenFileInfoManager *parent, QWidget *w)
+            : widget(w), p(parent), currentFileInfo(NULL) {
         // nothing
     }
 
@@ -343,23 +350,10 @@ public:
         return left->lastAccess() > right->lastAccess(); /// reverse sorting!
     }
 
-    void restorePreviouslyOpenedFiles() {
-        KSharedConfigPtr config = KSharedConfig::openConfig("kbibtexrc");
-
-        KConfigGroup cg(config, configGroupNameOpen);
-        for (int i = 0; i < maxNumOpenFiles; ++i) {
-            KUrl fileUrl = KUrl(cg.readEntry(QString("%1-%2").arg(OpenFileInfo::OpenFileInfoPrivate::keyURL).arg(i), ""));
-            if (!fileUrl.isValid()) break;
-            OpenFileInfo *ofi = p->contains(fileUrl);
-            if (ofi == NULL)
-                ofi = p->open(fileUrl);
-            p->setCurrentFile(ofi);
-        }
-    }
-
     void readConfig() {
         readConfig(OpenFileInfo::RecentlyUsed, configGroupNameRecentlyUsed, maxNumRecentlyUsedFiles);
         readConfig(OpenFileInfo::Favorite, configGroupNameFavorites, maxNumFavoriteFiles);
+        readConfig(OpenFileInfo::Open, configGroupNameOpen, maxNumOpenFiles);
     }
 
     void writeConfig() {
@@ -371,13 +365,20 @@ public:
     void readConfig(OpenFileInfo::StatusFlag statusFlag, const QString &configGroupName, int maxNumFiles) {
         KSharedConfigPtr config = KSharedConfig::openConfig("kbibtexrc");
 
+        bool isFirst = true;
         KConfigGroup cg(config, configGroupName);
         for (int i = 0; i < maxNumFiles; ++i) {
-            KUrl fileUrl = KUrl(cg.readEntry(QString("%1-%2").arg(OpenFileInfo::OpenFileInfoPrivate::keyURL).arg(i), ""));
+            KUrl fileUrl = KUrl::fromUserInput(cg.readEntry(QString("%1-%2").arg(OpenFileInfo::OpenFileInfoPrivate::keyURL).arg(i), QString()));
             if (!fileUrl.isValid()) break;
+            if (fileUrl.scheme().isEmpty())
+                fileUrl.setScheme(QLatin1String("file"));
 
             /// For local files, test if they exist; ignore local files that do not exist
-            if (fileUrl.isLocalFile() && !QFileInfo(fileUrl.pathOrUrl()).exists()) continue;
+            if (fileUrl.isLocalFile()) {
+                if (!QFileInfo(fileUrl.pathOrUrl()).exists())
+                    continue;
+            } else if (!KIO::NetAccess::exists(fileUrl, KIO::NetAccess::SourceSide, widget))
+                continue;
 
             OpenFileInfo *ofi = p->contains(fileUrl);
             if (ofi == NULL) {
@@ -386,6 +387,11 @@ public:
             ofi->addFlags(statusFlag);
             ofi->addFlags(OpenFileInfo::HasName);
             ofi->setLastAccess(QDateTime::fromString(cg.readEntry(QString("%1-%2").arg(OpenFileInfo::OpenFileInfoPrivate::keyLastAccess).arg(i), ""), OpenFileInfo::OpenFileInfoPrivate::dateTimeFormat));
+            if (isFirst) {
+                isFirst = false;
+                if (statusFlag == OpenFileInfo::Open)
+                    p->setCurrentFile(ofi);
+            }
         }
     }
 
@@ -417,16 +423,14 @@ const int OpenFileInfoManager::OpenFileInfoManagerPrivate::maxNumRecentlyUsedFil
 const int OpenFileInfoManager::OpenFileInfoManagerPrivate::maxNumOpenFiles = 16;
 
 
-OpenFileInfoManager::OpenFileInfoManager()
-        : d(new OpenFileInfoManagerPrivate(this))
+OpenFileInfoManager::OpenFileInfoManager(QWidget *widget)
+        : QObject(widget), d(new OpenFileInfoManagerPrivate(this, widget))
 {
-    d->readConfig();
-    QTimer::singleShot(500, this, SLOT(restorePreviouslyOpenedFiles()));
+    QTimer::singleShot(300, this, SLOT(delayedReadConfig()));
 }
 
 OpenFileInfoManager::~OpenFileInfoManager()
 {
-    d->writeConfig();
     delete d;
 }
 
@@ -449,7 +453,7 @@ OpenFileInfo *OpenFileInfoManager::open(const KUrl &url)
         result = new OpenFileInfo(this, url);
         connect(result, SIGNAL(flagsChanged(OpenFileInfo::StatusFlags)), this, SIGNAL(flagsChanged(OpenFileInfo::StatusFlags)));
         d->openFileInfoList << result;
-    }
+    } /// else: file was already open, re-use and return existing OpenFileInfo pointer
     result->setLastAccess();
     return result;
 }
@@ -482,7 +486,7 @@ bool OpenFileInfoManager::changeUrl(OpenFileInfo *openFileInfo, const KUrl &url)
     if (!url.equals(oldUrl) && oldUrl.isValid()) {
         /// current document was most probabily renamed (e.g. due to "Save As")
         /// add old URL to recently used files, but exclude the open files list
-        OpenFileInfo *ofi = open(oldUrl);
+        OpenFileInfo *ofi = open(oldUrl); // krazy:exclude=syscalls
         OpenFileInfo::StatusFlags statusFlags = (openFileInfo->flags() & (~OpenFileInfo::Open)) | OpenFileInfo::RecentlyUsed;
         ofi->setFlags(statusFlags);
     }
@@ -573,6 +577,8 @@ bool OpenFileInfoManager::queryCloseAll()
         foreach(OpenFileInfo *openFileInfo, restoreLaterList) {
             openFileInfo->addFlags(OpenFileInfo::Open);
         }
+
+        d->writeConfig();
     }
 
     return isClosing;
@@ -615,20 +621,18 @@ OpenFileInfoManager::OpenFileInfoList OpenFileInfoManager::filteredItems(OpenFil
     else if (required == OpenFileInfo::Favorite || required == OpenFileInfo::Open)
         qSort(result.begin(), result.end(), OpenFileInfoManagerPrivate::byNameLessThan);
 
-
     return result;
 }
 
 void OpenFileInfoManager::deferredListsChanged()
 {
-    kDebug() << "deferredListsChanged" << endl;
     OpenFileInfo::StatusFlags statusFlags = OpenFileInfo::Open;
     statusFlags |= OpenFileInfo::RecentlyUsed;
     statusFlags |= OpenFileInfo::Favorite;
     emit flagsChanged(statusFlags);
 }
 
-void OpenFileInfoManager::restorePreviouslyOpenedFiles()
+void OpenFileInfoManager::delayedReadConfig()
 {
-    d->restorePreviouslyOpenedFiles();
+    d->readConfig();
 }
